# NOTE: This is a direct translation from the VMware libview WrapLabel.
# The original C++ source can be found in http://view.sourceforge.net/.
# Both the copyright and permission notices of it are reproduced below.

# #########################################################################
# Copyright (c) 2005 VMware, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# #########################################################################



# NOTE: This translation was based on the work of Gian Mario Tagliaretti
# for Meld (http://meld.sourceforge.net/), with extra fixes and cleanups.



import gtk as Gtk

import pango as Pango



class WrapLabel(Gtk.Label):
    __gtype_name__ = u'WrapLabel'


    def _set_wrap_width(self, width):
        if width == 0:
            return

        self.get_layout().set_width(width * Pango.SCALE)

        if self.wrap_width != width:
            self.wrap_width = width

            self.queue_resize()

        pixel_size = self.get_layout().get_pixel_size()

        self.wrap_height = pixel_size[1]


    def __init__(self, text=None):
        self.wrap_width = 0

        self.wrap_height = 0

        Gtk.Label.__init__(self, text)
        self.set_alignment(0.0, 0.5)
        self.set_line_wrap(True)
        self.set_line_wrap_mode(Pango.WRAP_WORD_CHAR)


    def set_text(self, text):
        Gtk.Label.set_text(self, text)

        self._set_wrap_width(self.wrap_width)


    def set_markup(self, markup):
        Gtk.Label.set_markup(self, markup)

        self._set_wrap_width(self.wrap_width)


    def do_size_allocate(self, allocation):
        Gtk.Label.do_size_allocate(self, allocation)

        self._set_wrap_width(allocation.width)


    def do_size_request(self, requisition):
        requisition.width = 0

        requisition.height = self.wrap_height
