# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import pynotify

import gtk as Gtk

from glib import GError

from pynotify import Notification

from gtk import gdk as Gdk



DEFAULT_BORDER = 6


NICK_DIMENSION = 15


PROPERTY_SIGNAL = u'notify::'


CLOSE_TOOLTIP = u'Close'


CONNECTING_TEXT = u'Connecting\u2026'



def _escape(text):
    start = 0

    end = 0

    slices = []

    while end < len(text):
        char = text[end]

        if char == u'&':
            name = u'amp'
        elif char == u'<':
            name = u'lt'
        elif char == u'>':
            name = u'gt'
        else:
            name = None

        if name is not None:
            slices.append(text[start:end])

            slices.append(u'&{};'.format(name))

            start = end + 1

        end += 1

    if start < len(text):
        slices.append(text[start:end])

    return u''.join(slices)


def notify(summary, body, pixbuf):
    if u'body-markup' in pynotify.get_server_caps():
        summary = _escape(summary)

        body = _escape(body)

    notification = Notification(summary, body, None)

    notification.set_icon_from_pixbuf(pixbuf)

    notification.set_hint(u'transient', True)

    try:
        notification.show()
    except GError:
        pass


def show_url(url):
    screen = Gdk.screen_get_default()

    timestamp = Gtk.get_current_event_time()

    Gtk.show_uri(screen, url, timestamp)


def run_open_dialog(toplevel, title, specs, previous_path):
    dialog = Gtk.FileChooserDialog(title, toplevel, Gtk.FILE_CHOOSER_ACTION_OPEN, (Gtk.STOCK_CANCEL, Gtk.RESPONSE_CANCEL, Gtk.STOCK_OPEN, Gtk.RESPONSE_ACCEPT), None)

    for name, patterns in specs:
        filter = Gtk.FileFilter()

        filter.set_name(name)

        for pattern in patterns:
            filter.add_pattern(pattern)

        dialog.add_filter(filter)

    if previous_path is not None:
        dialog.set_filename(previous_path)

    toplevel.on_dialog_run()

    path = (dialog.get_filename() if dialog.run() == Gtk.RESPONSE_ACCEPT else None)

    dialog.destroy()

    toplevel.on_dialog_destroy()

    return path


def run_confirmation_dialog(toplevel, title, primary_markup, secondary_markup):
    dialog = Gtk.MessageDialog(toplevel, Gtk.DIALOG_MODAL, Gtk.MESSAGE_WARNING, Gtk.BUTTONS_OK_CANCEL, None)
    dialog.set_markup(u'<big><b>{}</b></big>'.format(primary_markup))
    dialog.format_secondary_markup(secondary_markup)
    dialog.set_title(title)

    toplevel.on_dialog_run()

    success = (dialog.run() == Gtk.RESPONSE_OK)

    dialog.destroy()

    toplevel.on_dialog_destroy()

    return success


def run_error_dialog(toplevel, title, primary_text, secondary_markup):
    dialog = Gtk.MessageDialog(toplevel, Gtk.DIALOG_MODAL, Gtk.MESSAGE_ERROR, Gtk.BUTTONS_CLOSE, primary_text)
    dialog.format_secondary_markup(secondary_markup)
    dialog.set_title(title)

    toplevel.on_dialog_run()

    dialog.run()

    dialog.destroy()

    toplevel.on_dialog_destroy()


def run_generic_error_dialog(toplevel, secondary_markup):
    run_error_dialog(toplevel, u'Error', u'Could not execute the operation', secondary_markup)


def NOTIMPLEMENTEDYET(toplevel):
    run_generic_error_dialog(toplevel, u'NOT IMPLEMENTED YET')



import frontend, dialog, color, font, theme, basic, status, twitter, header, indicator, stream, shell
