# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from __future__ import division


from threading import Condition


import numpy

from gtk import gdk as Gdk



def _activate_value(value):
    return min(long(1.2 * value), 255)


def activate_pixbuf(pixbuf):
    array = pixbuf.get_pixels_array()

    has_alpha = pixbuf.get_has_alpha()

    pixbuf_active = Gdk.Pixbuf(pixbuf.get_colorspace(),
                               has_alpha,
                               pixbuf.get_bits_per_sample(),
                               pixbuf.get_width(),
                               pixbuf.get_height())

    array_active = pixbuf_active.get_pixels_array()

    activate_array = numpy.vectorize(_activate_value, [numpy.uint8])

    array_active[:,:,:3] = activate_array(array[:,:,:3])

    if has_alpha:
        array_active[:,:,3] = array[:,:,3]

    return pixbuf_active


def resize_pixbuf(pixbuf_normal, size):
    width_normal = pixbuf_normal.get_width()

    height_normal = pixbuf_normal.get_height()

    factor = size / max(width_normal, height_normal)

    width = long(factor * width_normal + 0.5)

    height = long(factor * height_normal + 0.5)

    pixbuf = Gdk.Pixbuf(pixbuf_normal.get_colorspace(),
                        pixbuf_normal.get_has_alpha(),
                        pixbuf_normal.get_bits_per_sample(),
                        width,
                        height)

    pixbuf_normal.scale(pixbuf, 0, 0, width, height, 0, 0, factor, factor, Gdk.INTERP_BILINEAR)

    return pixbuf


def transparent_pixbuf(size):
    pixbuf = Gdk.Pixbuf(Gdk.COLORSPACE_RGB, True, 8, size, size)

    pixbuf.fill(0x00000000)

    return pixbuf



class FilesystemPool(object):
    def __init__(self):
        self.hashes = set()

        self.condition = Condition()


    def acquire(self, hash):
        self.condition.acquire()

        while hash in self.hashes:
            self.condition.wait()

        self.hashes.add(hash)

        self.condition.release()


    def release(self, hash):
        self.condition.acquire()

        self.hashes.remove(hash)

        self.condition.notify()

        self.condition.release()



class ConnectionPool(object):
    def __init__(self):
        self.count = 20

        self.condition = Condition()


    def acquire(self):
        self.condition.acquire()

        while self.count == 0:
            self.condition.wait()

        self.count -= 1

        self.condition.release()


    def release(self):
        self.condition.acquire()

        self.count += 1

        self.condition.notify()

        self.condition.release()



import emblem, button, stamp, identifier, avatar, preview, frontend
