# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gobject as GObject

import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch

from polly.gui.basic.avatar import BackendCache



class Avatar(Gtk.Alignment):
#    def __del__(self):
#        print u'DEBUG: avatar deleted'
    def _advance(self):
        if self.framebuffer is not None:
            self.index = (self.index + 1) % self.framebuffer.length

            if self.inside and self.pressed:
                self.image.set_from_pixbuf(self.framebuffer.pixbufs_active[self.index])
            else:
                self.image.set_from_pixbuf(self.framebuffer.pixbufs[self.index])

            if self.inside:
                self.uid = GObject.timeout_add(self.framebuffer.delays[self.index], self._advance)
            else:
                self.uid = None

        return False


    def _leave(self):
        self.inside = False

        self.get_window().set_cursor(self.cursor)

        if self.spinner is None:
            if self.framebuffer is None:
                self.image.set_from_pixbuf(self.pixbuf)
            else:
                self.image.set_from_pixbuf(self.framebuffer.pixbufs[self.index])


    def _update_image(self):
        self.pixbuf = self.backend.pixbuf

        self.pixbuf_active = self.backend.pixbuf_active

        if self.backend.framebuffer is None:
            self.framebuffer = None

            self.index = None

            if self.inside and self.pressed:
                self.image.set_from_pixbuf(self.pixbuf_active)
            else:
                self.image.set_from_pixbuf(self.pixbuf)
        else:
            self.framebuffer = self.backend.framebuffer

            self.index = self.framebuffer.length - 1

            if self.uid is not None:
                GObject.source_remove(self.uid)

            self._advance()

        self.box.set_size_request(self.backend.size, self.backend.size)


    def _update(self):
        self.backend.first_thread.join()

        self.backend.block_updates()

        Gdk.threads_enter()

        self.box.remove(self.spinner)

        self.spinner = None

        self.image = Gtk.Image()
        self.image.show()

        self.box.add(self.image)

        self._update_image()

        Gdk.threads_leave()

        self.backend_uid = self.backend.notifier.connect(self._on_backend_changed)

        self.backend.allow_updates()


    def _on_expose_event_after(self, widget, event):
        if self.box.is_focus():
            allocation = self.box.get_allocation()

            context = self.box.get_window().cairo_create()

            context.set_source_color(self.parent_style.bg[Gtk.STATE_NORMAL])
            context.rectangle(0, 0, allocation.width, allocation.height)
            context.stroke()

            context.set_dash([1.0, 1.0])

            context.set_source_color(self.parent_style.fg[Gtk.STATE_NORMAL])
            context.rectangle(0, 0, allocation.width, allocation.height)
            context.stroke()

        return False


    def _on_expose_event(self, widget, event):
        while True:
            parent = widget.get_parent()

            if parent.get_has_window():
                break

            widget = parent

        self.parent_style = parent.get_style()

        context = self.box.get_window().cairo_create()

        context.set_source_color(self.parent_style.bg[Gtk.STATE_NORMAL])

        context.paint()

        return False


    def _on_enter_notify_event(self, widget, event):
        self.inside = True

        self.get_window().set_cursor(self.cursor_hover)

        if self.spinner is None:
            if self.framebuffer is None:
                if self.pressed:
                    self.image.set_from_pixbuf(self.pixbuf_active)
            else:
                if self.pressed:
                    self.image.set_from_pixbuf(self.framebuffer.pixbufs_active[self.index])

                if self.uid is not None:
                    GObject.source_remove(self.uid)

                self._advance()

        return False


    def _on_leave_notify_event(self, widget, event):
        self._leave()

        return False


    def _on_button_press_event(self, widget, event):
        if event.button == 1:
            self.pressed = True

            self.box.grab_focus()

            if self.spinner is None:
                if self.framebuffer is None:
                    self.image.set_from_pixbuf(self.pixbuf_active)
                else:
                    self.image.set_from_pixbuf(self.framebuffer.pixbufs_active[self.index])

        return False


    def _on_button_release_event(self, widget, event):
        if event.button == 1:
            if self.pressed:
                self.pressed = False

                if self.inside:
                    if self.spinner is None:
                        if self.framebuffer is None:
                            self.image.set_from_pixbuf(self.pixbuf)
                        else:
                            self.image.set_from_pixbuf(self.framebuffer.pixbufs[self.index])

                    self.callback()

        return False


    def _on_focus_in_event(self, widget, event):
        self.box.queue_draw()

        return False


    def _on_focus_out_event(self, widget, event):
        self.box.queue_draw()

        return False


    def _on_key_press_event(self, widget, event):
        if (event.keyval == Gtk.keysyms.KP_Enter or
            event.keyval == Gtk.keysyms.Return or
            event.keyval == Gtk.keysyms.space):
            self.callback()

            return False

        toplevel = self.get_toplevel()

        return toplevel.press(event)


    def _on_unrealize(self, widget):
        if self.inside:
            self._leave()

        self.pressed = False


    def _on_backend_changed(self):
        Gdk.threads_enter()

        self._update_image()

        Gdk.threads_leave()


    def __init__(self, user_backend, dimension, spacing, callback, font_scheme, backend_cache):
        self.user_backend = user_backend

        self.callback = callback

        self.inside = False

        self.pressed = False

        self.uid = None

        Gdk.threads_enter()

        self.cursor = Gdk.Cursor(Gdk.ARROW)

        self.cursor_hover = Gdk.Cursor(Gdk.HAND2)

        self.spinner = Gtk.Spinner()
        self.spinner.start()
        self.spinner.show()

        self.box = Gtk.EventBox()
        self.box.set_app_paintable(True)
        self.box.set_can_focus(True)
        self.box.connect_after(u'expose-event', self._on_expose_event_after)
        self.box.connect(u'expose-event', self._on_expose_event)
        self.box.connect(u'enter-notify-event', self._on_enter_notify_event)
        self.box.connect(u'leave-notify-event', self._on_leave_notify_event)
        self.box.connect_after(u'button-press-event', self._on_button_press_event)
        self.box.connect(u'button-release-event', self._on_button_release_event)
        self.box.connect(u'focus-in-event', self._on_focus_in_event)
        self.box.connect(u'focus-out-event', self._on_focus_out_event)
        self.box.connect(u'key-press-event', self._on_key_press_event)
        self.box.connect(u'unrealize', self._on_unrealize)
        self.box.add(self.spinner)
        self.box.show()

        super(Avatar, self).__init__(0.5, 0.0, 0.0, 0.0)
        self.add(self.box)
        self.show()

        Gdk.threads_leave()

        font_scheme.block_updates()

        unit = dimension * font_scheme.unit

        void = spacing * font_scheme.void

        Gdk.threads_enter()

        self.spinner.set_size_request(unit + void, unit + void)

        Gdk.threads_leave()

        font_scheme.allow_updates()

        self.backend = backend_cache.acquire(user_backend, dimension, spacing)

        self.thread = dispatch(self._update)


    def isolate(self):
        self.thread.join()

        self.backend.block_updates()

        self.backend.notifier.disconnect(self.backend_uid)

        self.backend.allow_updates()

        self.backend.release()

        self.callback = None



class Factory(object):
    def __init__(self, cache_path, cache_path_lock, font_scheme, connection_pool):
        self.font_scheme = font_scheme

        self.backend_cache = BackendCache(cache_path, cache_path_lock, font_scheme, connection_pool)


    def reload(self):
        self.backend_cache.reload()


    def build(self, user_backend, dimension, spacing, callback):
        return Avatar(user_backend, dimension, spacing, callback, self.font_scheme, self.backend_cache)
