# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

from gtk import gdk as Gdk


from polly.gui import PROPERTY_SIGNAL

from polly.gui.basic.button import BackendCache



class Button(Gtk.Alignment):
#    def __del__(self):
#        print u'DEBUG: button deleted'
    def _leave(self):
        self.inside = False

        self.get_window().set_cursor(self.cursor)

        self.image.set_from_pixbuf(self.pixbuf)


    def _update_pixbufs(self):
        if self.toggled:
            self.pixbuf = self.pixbuf_normal

            self.pixbuf_active = self.pixbuf_normal_active
        else:
            self.pixbuf = self.pixbuf_unsaturated

            self.pixbuf_active = self.pixbuf_unsaturated_active


    def _update_image(self):
        if self.inside and self.pressed:
            self.image.set_from_pixbuf(self.pixbuf_active)
        else:
            self.image.set_from_pixbuf(self.pixbuf)


    def _on_expose_event_after(self, widget, event):
        if self.box.is_focus():
            allocation = self.box.get_allocation()

            context = self.box.get_window().cairo_create()

            context.set_source_color(self.parent_style.bg[Gtk.STATE_NORMAL])
            context.rectangle(0, 0, allocation.width, allocation.height)
            context.stroke()

            context.set_dash([1.0, 1.0])

            context.set_source_color(self.parent_style.fg[Gtk.STATE_NORMAL])
            context.rectangle(0, 0, allocation.width, allocation.height)
            context.stroke()

        return False


    def _on_expose_event(self, widget, event):
        while True:
            parent = widget.get_parent()

            if parent.get_has_window():
                break

            widget = parent

        self.parent_style = parent.get_style()

        context = self.box.get_window().cairo_create()

        context.set_source_color(self.parent_style.bg[Gtk.STATE_NORMAL])

        context.paint()

        if self.broken:
            allocation = self.box.get_allocation()

            context.set_source_color(self.parent_style.fg[Gtk.STATE_NORMAL])
            context.rectangle(0, 0, allocation.width, allocation.height)
            context.stroke()

        return False


    def _on_enter_notify_event(self, widget, event):
        self.inside = True

        self.get_window().set_cursor(self.cursor_hover)

        if self.pressed:
            self.image.set_from_pixbuf(self.pixbuf_active)

        return False


    def _on_leave_notify_event(self, widget, event):
        self._leave()

        return False


    def _on_button_press_event(self, widget, event):
        if event.button == 1:
            self.pressed = True

            self.box.grab_focus()

            self.image.set_from_pixbuf(self.pixbuf_active)

        return False


    def _on_button_release_event(self, widget, event):
        if event.button == 1:
            if self.pressed:
                self.pressed = False

                if self.inside:
                    self.image.set_from_pixbuf(self.pixbuf)

                    self.callback()

        return False


    def _on_focus_in_event(self, widget, event):
        self.box.queue_draw()

        return False


    def _on_focus_out_event(self, widget, event):
        self.box.queue_draw()

        return False


    def _on_key_press_event(self, widget, event):
        if (event.keyval == Gtk.keysyms.KP_Enter or
            event.keyval == Gtk.keysyms.Return or
            event.keyval == Gtk.keysyms.space):
            self.callback()

            return False

        toplevel = self.get_toplevel()

        return toplevel.press(event)


    def _on_unrealize(self, widget):
        if self.inside:
            self._leave()

        self.pressed = False


    def _on_sensitive_changed(self, object, pspec):
        if not self.get_sensitive():
            self._on_unrealize(object)


    def _on_backend_changed(self):
        Gdk.threads_enter()

        self.broken = self.backend.broken

        self.pixbuf_normal = self.backend.pixbuf_normal

        self.pixbuf_normal_active = self.backend.pixbuf_normal_active

        self.pixbuf_unsaturated = self.backend.pixbuf_unsaturated

        self.pixbuf_unsaturated_active = self.backend.pixbuf_unsaturated_active

        self._update_pixbufs()

        self._update_image()

        self.box.queue_draw()

        self.switch_box.set_size_request(self.backend.size, self.backend.size)

        Gdk.threads_leave()


    def __init__(self, backend, callback):
        self.backend = backend

        self.callback = callback

        self.inside = False

        self.pressed = False

        self.toggled = False

        Gdk.threads_enter()

        self.cursor = Gdk.Cursor(Gdk.ARROW)

        self.cursor_hover = Gdk.Cursor(Gdk.HAND2)

        self.spinner = Gtk.Spinner()

        self.image = Gtk.Image()
        self.image.show()

        self.box = Gtk.EventBox()
        self.box.set_app_paintable(True)
        self.box.set_can_focus(True)
        self.box.connect_after(u'expose-event', self._on_expose_event_after)
        self.box.connect(u'expose-event', self._on_expose_event)
        self.box.connect(u'enter-notify-event', self._on_enter_notify_event)
        self.box.connect(u'leave-notify-event', self._on_leave_notify_event)
        self.box.connect_after(u'button-press-event', self._on_button_press_event)
        self.box.connect(u'button-release-event', self._on_button_release_event)
        self.box.connect(u'focus-in-event', self._on_focus_in_event)
        self.box.connect(u'focus-out-event', self._on_focus_out_event)
        self.box.connect(u'key-press-event', self._on_key_press_event)
        self.box.connect(u'unrealize', self._on_unrealize)
        self.box.add(self.image)
        self.box.show()

        self.switch_box = Gtk.HBox(False, 0)
        self.switch_box.pack_start(self.spinner, True, True, 0)
        self.switch_box.pack_start(self.box, True, True, 0)
        self.switch_box.show()

        super(Button, self).__init__(0.5, 0.5, 0.0, 0.0)
        self.connect(PROPERTY_SIGNAL + u'sensitive', self._on_sensitive_changed)
        self.add(self.switch_box)
        self.show()

        Gdk.threads_leave()

        self.backend.block_updates()

        self._on_backend_changed()

        self.backend_uid = self.backend.notifier.connect(self._on_backend_changed)

        self.backend.allow_updates()


    def isolate(self):
        self.backend.block_updates()

        self.backend.notifier.disconnect(self.backend_uid)

        self.backend.allow_updates()

        self.callback = None


    def set_toggled(self, toggled):
        if self.toggled != toggled:
            self.toggled = toggled

            self._update_pixbufs()

            self._update_image()


    def set_spinning(self, spinning):
        if spinning:
            self.spinner.start()

            if self.box.is_focus():
                ancestor = self.box.get_ancestor(Gtk.Window)

                if ancestor is not None:
                    ancestor.child_focus(Gtk.DIR_TAB_BACKWARD)

            self.box.hide()

            self.spinner.show()
        else:
            self.spinner.hide()

            self.box.show()

            self.spinner.stop()



class Factory(object):
    def __init__(self, font_scheme, iconset_controller):
        self.backend_cache = BackendCache(font_scheme, iconset_controller)


    def clear(self):
        self.backend_cache.clear()


    def build(self, name, callback):
        backend = self.backend_cache.acquire(name)

        return Button(backend, callback)
