# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

from gtk import gdk as Gdk


from polly.gui.basic.emblem import BackendCache



class Emblem(Gtk.EventBox):
#    def __del__(self):
#        print u'DEBUG: emblem deleted'
    def _on_expose_event(self, widget, event):
        while True:
            parent = widget.get_parent()

            if parent.get_has_window():
                break

            widget = parent

        parent_style = parent.get_style()

        context = self.get_window().cairo_create()

        context.set_source_color(parent_style.bg[Gtk.STATE_NORMAL])

        context.paint()

        if self.broken:
            allocation = self.get_allocation()

            context.set_source_color(parent_style.fg[Gtk.STATE_NORMAL])
            context.rectangle(0, 0, allocation.width, allocation.height)
            context.stroke()

        return False


    def _on_backend_changed(self):
        Gdk.threads_enter()

        self.broken = self.backend.broken

        self.image.set_from_pixbuf(self.backend.pixbuf)

        self.set_size_request(self.backend.size, self.backend.size)

        self.queue_draw()

        Gdk.threads_leave()


    def __init__(self, backend):
        self.backend = backend

        Gdk.threads_enter()

        self.image = Gtk.Image()
        self.image.show()

        super(Emblem, self).__init__()
        self.set_app_paintable(True)
        self.connect(u'expose-event', self._on_expose_event)
        self.add(self.image)
        self.show()

        Gdk.threads_leave()

        self.backend.block_updates()

        self._on_backend_changed()

        self.backend_uid = self.backend.notifier.connect(self._on_backend_changed)

        self.backend.allow_updates()


    def isolate(self):
        self.backend.block_updates()

        self.backend.notifier.disconnect(self.backend_uid)

        self.backend.allow_updates()



class Factory(object):
    def __init__(self, font_scheme, iconset_controller):
        self.backend_cache = BackendCache(font_scheme, iconset_controller)


    def clear(self):
        self.backend_cache.clear()


    def build(self, name):
        backend = self.backend_cache.acquire(name)

        return Emblem(backend)
