# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch

from polly.twitter.lead import user

from polly.gui.basic.identifier import BackendCache



class Identifier(Gtk.Alignment):
#    def __del__(self):
#        print u'DEBUG: identifier deleted'
    def _update(self):
        self.backend.first_thread.join()

        self.backend.block_updates()

        self._on_backend_changed()

        self.backend_uid = self.backend.notifier.connect(self._on_backend_changed)

        self.backend.allow_updates()


    def _on_user_nick_changed(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        Gdk.threads_enter()

        self.image.set_tooltip_text(user_nick)

        Gdk.threads_leave()


    def _on_backend_changed(self):
        Gdk.threads_enter()

        self.image.set_from_pixbuf(self.backend.pixbuf)

        self.switch_box.set_size_request(self.backend.size, self.backend.size)

        Gdk.threads_leave()


    def __init__(self, user_backend, backend_cache):
        self.user_backend = user_backend

        Gdk.threads_enter()

        self.spinner = Gtk.Spinner()
        self.spinner.set_tooltip_text(u'Processing\u2026')

        self.image = Gtk.Image()
        self.image.show()

        self.switch_box = Gtk.HBox(False, 0)
        self.switch_box.pack_start(self.spinner, True, True, 0)
        self.switch_box.pack_start(self.image, True, True, 0)
        self.switch_box.show()

        super(Identifier, self).__init__(0.5, 0.5, 0.0, 0.0)
        self.add(self.switch_box)
        self.show()

        Gdk.threads_leave()

        self._on_user_nick_changed()

        self.user_backend.block_updates()

        self.user_nick_id = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.user_backend.allow_updates()

        self.backend = backend_cache.acquire(self.user_backend)

        self.thread = dispatch(self._update)


    def isolate(self):
        self.thread.join()

        self.backend.block_updates()

        self.backend.notifier.disconnect(self.backend_uid)

        self.backend.allow_updates()

        self.backend.release()

        self.user_backend.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_id)

        self.user_backend.allow_updates()


    def set_spinning(self, spinning):
        Gdk.threads_enter()

        if spinning:
            self.spinner.start()

            self.image.hide()

            self.spinner.show()
        else:
            self.spinner.hide() 

            self.image.show()

            self.spinner.stop()

        Gdk.threads_leave()



class Factory(object):
    def __init__(self, cache_path, cache_path_lock, font_scheme, connection_pool):
        self.backend_cache = BackendCache(cache_path, cache_path_lock, font_scheme, connection_pool)


    def reload(self):
        self.backend_cache.reload()


    def build(self, user_backend):
        return Identifier(user_backend, self.backend_cache)
