# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch

from polly.gui.basic.stamp import BackendCache



class Stamp(Gtk.Alignment):
#    def __del__(self):
#        print u'DEBUG: stamp deleted'
    def _update_image(self):
        self.image.set_from_pixbuf(self.backend.pixbuf)

        self.image.set_size_request(self.backend.size, self.backend.size)


    def _update(self):
        self.backend.first_thread.join()

        self.backend.block_updates()

        Gdk.threads_enter()

        self.remove(self.spinner)

        self.spinner = None

        self.image = Gtk.Image()
        self.image.show()

        self.add(self.image)

        self._update_image()

        Gdk.threads_leave()

        self.backend_uid = self.backend.notifier.connect(self._on_backend_changed)

        self.backend.allow_updates()


    def _on_backend_changed(self):
        Gdk.threads_enter()

        self._update_image()

        Gdk.threads_leave()


    def __init__(self, url, dimension, font_scheme, backend_cache):
        Gdk.threads_enter()

        self.spinner = Gtk.Spinner()
        self.spinner.start()
        self.spinner.show()

        super(Stamp, self).__init__(0.5, 0.0, 0.0, 0.0)
        self.add(self.spinner)
        self.show()

        Gdk.threads_leave()

        font_scheme.block_updates()

        unit = dimension * font_scheme.unit

        Gdk.threads_enter()

        self.spinner.set_size_request(unit, unit)

        Gdk.threads_leave()

        font_scheme.allow_updates()

        self.backend = backend_cache.acquire(url, dimension)

        self.thread = dispatch(self._update)


    def isolate(self):
        self.thread.join()

        self.backend.block_updates()

        self.backend.notifier.disconnect(self.backend_uid)

        self.backend.allow_updates()

        self.backend.release()



class Factory(object):
    def __init__(self, cache_path, cache_path_lock, font_scheme, connection_pool):
        self.font_scheme = font_scheme

        self.backend_cache = BackendCache(cache_path, cache_path_lock, font_scheme, connection_pool)


    def reload(self):
        self.backend_cache.reload()


    def build(self, url, dimension):
        return Stamp(url, dimension, self.font_scheme, self.backend_cache)
