# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################'''



import gtk as Gtk


from polly.gui import DEFAULT_BORDER



class Dialog(Gtk.Dialog):
    def _on_response(self, dialog, response_id):
        self.running = False


    def __init__(self, title, toplevel):
        super(Dialog, self).__init__(title, toplevel, Gtk.DIALOG_MODAL, None)
        self.set_resizable(False)
        self.connect(u'response', self._on_response)

        self.toplevel = toplevel

        self.running = True

        self.content_frame = Gtk.Frame(None)
        self.content_frame.set_border_width(DEFAULT_BORDER)
        self.content_frame.show()

        self.get_content_area().pack_start(self.content_frame, True, True, 0)

        width, height = Gtk.icon_size_lookup(Gtk.ICON_SIZE_BUTTON)

        self.spinner = Gtk.Spinner()
        self.spinner.set_size_request(width, height)

        self.image = Gtk.image_new_from_stock(Gtk.STOCK_DIALOG_ERROR, Gtk.ICON_SIZE_BUTTON)

        switch_box = Gtk.HBox(False, 0)
        switch_box.pack_start(self.spinner, True, 0)
        switch_box.pack_start(self.image, True, 0)
        switch_box.show()

        alignment = Gtk.Alignment(1.0, 0.5, 0.0, 0.0)
        alignment.add(switch_box)
        alignment.show()

        self.action_box = self.get_action_area()
        self.action_box.pack_start(alignment, True, True, 0)

        self.thread = None


    def on_dialog_run(self):
        pass


    def on_dialog_destroy(self):
        pass


    def run_and_destroy(self):
        self.toplevel.on_dialog_run()

        self.run()

        self.destroy()

        self.toplevel.on_dialog_destroy()

        return self.thread


    def start(self):
        self.image.hide()

        self.spinner.start()

        self.spinner.show()


    def stop(self, message):
        self.spinner.hide()

        self.spinner.stop()

        if message is not None:
            self.image.set_tooltip_text(message)

            self.image.show()
