# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk


from polly import ENCODING

from polly.proxy import DEFAULT_PORT, LOWER_PORT, UPPER_PORT, ControllerBackend



class Controller(object):
    def _enabled_callback(self):
        enabled = self.backend.enabled

        self.enabled_checkbutton.set_active(enabled)


    def _host_callback(self):
        host = self.backend.host

        self.host_entry.set_text(host)


    def _port_callback(self):
        port = self.backend.port

        self.port_spinbutton.set_value(port)


    def _type_callback(self):
        type = self.backend.type

        self.type_radiobuttons[type].set_active(True)


    def _on_enabled_checkbutton_toggled(self, togglebutton):
        enabled = self.enabled_checkbutton.get_active()

        self.backend.set_enabled(enabled)


    def _on_host_entry_activate(self, entry):
        host = self.host_entry.get_text()

        self.backend.set_host(host.decode(ENCODING))


    def _on_port_spinbutton_output(self, spin_button):
        port = self.port_spinbutton.get_value_as_int()

        self.backend.set_port(port)

        return False


    def _on_type_radiobutton_toggled(self, togglebutton):
        if togglebutton.get_active():
            type = self.type_radiobuttons.index(togglebutton)

            self.backend.set_type(type)


    def __init__(self, setting_factory):
        self.backend = ControllerBackend(self._enabled_callback, self._host_callback, self._port_callback, self._type_callback, setting_factory)

        self.enabled_checkbutton = Gtk.CheckButton(u'Use a proxy server for connecting', False)
        self.enabled_checkbutton.connect(u'toggled', self._on_enabled_checkbutton_toggled)
        self.enabled_checkbutton.show()

        self.host_entry = Gtk.Entry(0)
        self.host_entry.connect(u'activate', self._on_host_entry_activate)
        self.host_entry.show()

        port_adjustment = Gtk.Adjustment(float(DEFAULT_PORT), float(LOWER_PORT), float(UPPER_PORT), 100.0, 1000.0, 0.0)

        self.port_spinbutton = Gtk.SpinButton(port_adjustment, 0.0, 0)
        self.port_spinbutton.connect(u'output', self._on_port_spinbutton_output)
        self.port_spinbutton.show()

        self.type_radiobuttons = []

        self.type_radiobuttons.append(Gtk.RadioButton(None,
                                                      u'SOCKS4', False))
        self.type_radiobuttons[0].connect(u'toggled', self._on_type_radiobutton_toggled)
        self.type_radiobuttons[0].show()

        self.type_radiobuttons.append(Gtk.RadioButton(self.type_radiobuttons[0],
                                                      u'SOCKS5', False))
        self.type_radiobuttons[1].connect(u'toggled', self._on_type_radiobutton_toggled)
        self.type_radiobuttons[1].show()

        self.type_radiobuttons.append(Gtk.RadioButton(self.type_radiobuttons[0],
                                                      u'HTTP', False))
        self.type_radiobuttons[2].connect(u'toggled', self._on_type_radiobutton_toggled)
        self.type_radiobuttons[2].show()

        self._enabled_callback()

        self._host_callback()

        self._port_callback()

        self._type_callback()


    def isolate(self):
        self.backend.isolate()
