# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk


from polly.shortener import DEFAULT_MINIMUM, LOWER_MINIMUM, UPPER_MINIMUM, ControllerBackend



class Controller(object):
    def _enabled_callback(self):
        enabled = self.backend.enabled

        self.enabled_checkbutton.set_active(enabled)


    def _minimum_callback(self):
        minimum = self.backend.minimum

        self.minimum_spinbutton.set_value(minimum)


    def _name_callback(self):
        index = 0

        for name in self.backend.shorteners.viewkeys():
            if name == self.backend.name:
                break

            index += 1

        self.name_combobox.set_active(index)


    def _on_enabled_checkbutton_toggled(self, togglebutton):
        enabled = self.enabled_checkbutton.get_active()

        self.backend.set_enabled(enabled)


    def _on_minimum_spinbutton_output(self, spin_button):
        minimum = self.minimum_spinbutton.get_value_as_int()

        self.backend.set_minimum(minimum)

        return False


    def _on_name_combobox_changed(self, widget):
        name = self.name_combobox.get_active_text()

        self.backend.set_name(name)


    def __init__(self, setting_factory):
        self.backend = ControllerBackend(self._enabled_callback, self._minimum_callback, self._name_callback, setting_factory)

        self.enabled_checkbutton = Gtk.CheckButton(u'Automatically shorten pasted URLs', False)
        self.enabled_checkbutton.connect(u'toggled', self._on_enabled_checkbutton_toggled)
        self.enabled_checkbutton.show()

        minimum_adjustment = Gtk.Adjustment(float(DEFAULT_MINIMUM), float(LOWER_MINIMUM), float(UPPER_MINIMUM), 1.0, 10.0, 0.0)

        self.minimum_spinbutton = Gtk.SpinButton(minimum_adjustment, 0.0, 0)
        self.minimum_spinbutton.connect(u'output', self._on_minimum_spinbutton_output)
        self.minimum_spinbutton.show()

        self.name_combobox = Gtk.combo_box_new_text()

        for name in self.backend.shorteners.viewkeys():
            self.name_combobox.append_text(name)

        self.name_combobox.connect(u'changed', self._on_name_combobox_changed)
        self.name_combobox.show()

        self._enabled_callback()

        self._minimum_callback()

        self._name_callback()


    def isolate(self):
        self.backend.isolate()
