# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk


from polly.twitter.media import RunnerBackend

from polly.gui import DEFAULT_BORDER



TITLE = u'Attach media file'



class Runner(object):
    def _name_callback(self):
        index = 0

        for name in self.backend.providers.viewkeys():
            if name == self.backend.name:
                break

            index += 1

        self.name_combobox.set_active(index)


    def _on_name_combobox_changed(self, widget):
        name = self.name_combobox.get_active_text()

        self.backend.set_name(name)


    def __init__(self, setting_factory, proxy_controller):
        self.backend = RunnerBackend(self._name_callback, setting_factory, proxy_controller)

        path_label = Gtk.Label(u'File:')
        path_label.show()

        self.path_filechooserbutton = Gtk.FileChooserButton(TITLE, None)
        self.path_filechooserbutton.set_size_request(30 * DEFAULT_BORDER, -1)
        self.path_filechooserbutton.show()

        path_box = Gtk.HBox(False, DEFAULT_BORDER)
        path_box.pack_start(path_label, False, False, 0)
        path_box.pack_start(self.path_filechooserbutton, True, True, 0)
        path_box.show()

        name_label = Gtk.Label(u'Upload to:')
        name_label.show()

        self.name_combobox = Gtk.combo_box_new_text()

        for name in self.backend.providers.viewkeys():
            self.name_combobox.append_text(name)

        self.name_combobox.set_size_request(30 * DEFAULT_BORDER, -1)
        self.name_combobox.connect(u'changed', self._on_name_combobox_changed)
        self.name_combobox.show()

        name_box = Gtk.HBox(False, DEFAULT_BORDER)
        name_box.pack_start(name_label, False, False, 0)
        name_box.pack_start(self.name_combobox, True, True, 0)
        name_box.show()

        box = Gtk.VBox(True, 2 * DEFAULT_BORDER)
        box.set_border_width(2 * DEFAULT_BORDER)
        box.pack_start(path_box, True, False, 0)
        box.pack_start(name_box, True, False, 0)
        box.show()

        self.frame = Gtk.Frame(None)
        self.frame.set_border_width(DEFAULT_BORDER)
        self.frame.add(box)
        self.frame.show()

        self._name_callback()


    def isolate(self):
        self.frame.destroy()

        self.backend.isolate()


    def run(self, toplevel):
        dialog = Gtk.Dialog(TITLE, toplevel, Gtk.DIALOG_MODAL, (Gtk.STOCK_CANCEL, Gtk.RESPONSE_REJECT, Gtk.STOCK_OK, Gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)

        dialog.get_content_area().add(self.frame)

        toplevel.on_dialog_run()

        path = (self.path_filechooserbutton.get_filename() if dialog.run() == Gtk.RESPONSE_ACCEPT else None)

        dialog.get_content_area().remove(self.frame)

        dialog.destroy()

        toplevel.on_dialog_destroy()

        return self.backend.name, path
