# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import os


from indicate import indicate_server_ref_default, Indicator


from polly import ENCODING



class Server(object):
    def _on_server_display(self, arg0, arg1):
        if self.main_window is not None:
            if self.main_window.is_active():
                self.main_window.hide()
            else:
                self.main_window.present()


    def __init__(self, name, desktop_prefixes):
        self.libindicate = indicate_server_ref_default()

        for desktop_prefix in desktop_prefixes:
            desktop_path = os.path.join(desktop_prefix, u'applications', u'{}.desktop'.format(name))

            if os.path.isfile(desktop_path):
                self.libindicate.set_desktop_file(desktop_path)

                break

        self.libindicate.set_type(u'message')
        self.libindicate.show()

        self.main_window = None

        self.uid = self.libindicate.connect(u'server-display', self._on_server_display)


    def isolate(self):
        self.libindicate.disconnect(self.uid)


    def build(self, stream):
        return Client(self, stream)



class Client(object):
    def _set_active(self, active):
        active = u'true' if active else u'false'

        self.indicator.set_property(u'draw-attention', active)


    def _on_user_display(self, indicateindicator, arg1):
        toplevel = self.stream.get_toplevel()

        toplevel.present()

        self.stream.popup()

        self.stream.enter()

        if self.stream.get_focus_child() is None:
            self.stream.key_box.grab_focus()


    def __init__(self, server, stream):
        self.stream = stream

        self.indicator = Indicator()

        self.uid = self.indicator.connect(u'user-display', self._on_user_display)


    def isolate(self):
        self.indicator.disconnect(self.uid)

        self.stream = None


    def set_name(self, name):
        self.indicator.set_property(u'name', name)


    def set_count(self, count):
        if count == 0:
            self.hide()
        else:
            self.indicator.set_property(u'count', bytes(count).decode(ENCODING))

            self.indicator.show()


    def hide(self):
        self.indicator.hide()


    def reset(self):
        self.set_count(0)

        self._set_active(False)


    def activate(self):
        self._set_active(True)
