# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk


from polly import ENCODING

from polly.gui import DEFAULT_BORDER



TOOLTIP = u'All items already read'



class Server(object):
    def _on_activate(self, status_icon):
        if self.main_window is not None:
            if self.main_window.is_active():
                self.main_window.hide()
            else:
                self.main_window.present()


    def _on_popup_menu(self, status_icon, button, activate_time):
        if self.main_window is not None:
            self.menu.popup(None, None, Gtk.status_icon_position_menu, button, activate_time, status_icon)


    def _on_quit_menuitem_activate(self, menuitem):
        self.main_window.quit()


    def __init__(self, name, desktop_prefixes):
        self.visible_count = 0

        self.active_count = 0

        self.normal_name = u'{}-normal'.format(name)

        self.active_name = u'{}-active'.format(name)

        self.separator = Gtk.SeparatorMenuItem()

        quit_menuitem = Gtk.MenuItem(u'Quit', False)
        quit_menuitem.connect(u'activate', self._on_quit_menuitem_activate)
        quit_menuitem.show()

        self.menu = Gtk.Menu()
        self.menu.append(self.separator)
        self.menu.append(quit_menuitem)

        self.statusicon = Gtk.status_icon_new_from_icon_name(self.normal_name)
        self.statusicon.set_tooltip(TOOLTIP)

        self.main_window = None

        self.activate_uid = self.statusicon.connect(u'activate', self._on_activate)

        self.popup_menu_uid = self.statusicon.connect(u'popup-menu', self._on_popup_menu)


    def isolate(self):
        self.statusicon.disconnect(self.popup_menu_uid)

        self.statusicon.disconnect(self.activate_uid)

        self.menu.destroy()


    def build(self, stream):
        return Client(self, stream)



class Client(object):
    def _set_active(self, active):
        if active:
            if self.server.active_count == 0:
                self.server.statusicon.set_from_icon_name(self.server.active_name)
                self.server.statusicon.set_tooltip(u'You have unread items')

            self.server.active_count += 1
        else:
            self.server.active_count -= 1

            if self.server.active_count == 0:
                self.server.statusicon.set_from_icon_name(self.server.normal_name)
                self.server.statusicon.set_tooltip(TOOLTIP)


    def _on_activate(self, menuitem):
        toplevel = self.stream.get_toplevel()

        toplevel.present()

        self.stream.popup()

        self.stream.enter()

        if self.stream.get_focus_child() is None:
            self.stream.key_box.grab_focus()


    def __init__(self, server, stream):
        self.server = server

        self.stream = stream

        self.visible = False

        self.name_label = Gtk.Label(None)
        self.name_label.set_alignment(0.0, 0.5)
        self.name_label.show()

        self.count_label = Gtk.Label(None)
        self.count_label.set_alignment(1.0, 0.5)
        self.count_label.show()

        box = Gtk.HBox(False, DEFAULT_BORDER)
        box.pack_start(self.name_label, True, True, 0)
        box.pack_start(self.count_label, False, False, 0)
        box.show()

        self.menuitem = Gtk.MenuItem(None, False)
        self.menuitem.connect(u'activate', self._on_activate)
        self.menuitem.add(box)

        self.server.menu.append(self.menuitem)


    def isolate(self):
        self.menuitem.destroy()

        self.stream = None


    def set_name(self, name):
        self.name_label.set_text(name)


    def set_count(self, count):
        if count == 0:
            self.hide()
        else:
            self.count_label.set_text(bytes(count).decode(ENCODING))

            if not self.visible:
                self.visible = True

                self.server.menu.reorder_child(self.menuitem, 0)

                self.menuitem.show()

                if self.server.visible_count == 0:
                    self.server.separator.show()

                self.server.visible_count += 1


    def hide(self):
        if self.visible:
            self.server.visible_count -= 1

            if self.server.visible_count == 0:
                self.server.separator.hide()

            self.menuitem.hide()

            self.visible = False


    def reset(self):
        self.set_count(0)

        self._set_active(False)


    def activate(self):
        self._set_active(True)
