# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch

from polly.gui import run_confirmation_dialog, NOTIMPLEMENTEDYET

from polly.gui.stream.posts import tweets, messages

from polly.gui.stream.leads import users, groups



class Bar(Gtk.MenuBar):
    def _reload_task(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        self.main_window.stream_manager.stamp_factory.reload()

        self.main_window.account_manager.identifier_factory.reload()

        self.main_window.stream_manager.avatar_factory.reload()

        self.main_window.stream_manager.preview_factory.reload()

        Gdk.threads_enter()

        self.reload_subitem.set_sensitive(True)

        Gdk.threads_leave()


    def _new_stream(self, callback):
        account = self.main_window.main_panel.get_current_account()

        if account is not None:
            callback(account)


    def _new_tweets_home_mentions(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_home_mentions)


    def _new_tweets_home(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_home)


    def _new_tweets_mentions(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_mentions)


    def _new_tweets_sent_by_me(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_sent_by_me)


    def _new_tweets_favorited_by_me(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_favorited_by_me)


    def _new_tweets_retweeted_by_me(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_retweeted_by_me)


    def _new_tweets_retweeted_to_me(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_retweeted_to_me)


    def _new_tweets_retweeted_from_me(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_tweets_retweeted_from_me)


    def _new_users_my_followers(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_users_my_followers)


    def _new_users_my_friends(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_users_my_friends)


    def _new_groups_my_memberships(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_groups_my_memberships)


    def _new_groups_my_subscriptions(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_groups_my_subscriptions)


    def _new_groups_my_groups(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_groups_my_groups)


    def _new_messages_all(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_messages_all)


    def _new_messages_received(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_messages_received)


    def _new_messages_sent(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_messages_sent)


    def _new_trends(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_trends)


    def _new_categories(self, menuitem):
        self._new_stream(self.main_window.stream_manager.new_categories)


    def _search(self, menuitem):
        account = self.main_window.main_panel.get_current_account()

        if account is not None:
            NOTIMPLEMENTEDYET(self.get_toplevel())


    def _reload(self, menuitem):
        self.reload_subitem.set_sensitive(False)

        self.thread = dispatch(self._reload_task, self.thread)


    def _refresh(self, menuitem):
        self.main_window.stream_manager.refresh_all()


    def _close(self, menuitem):
        self.main_window.hide()


    def _quit(self, menuitem):
        if not self.main_window.main_panel.get_adding():
            self.main_window.quit()


    def _create_group(self, menuitem):
        self.main_window.main_panel.create_group(self.main_window.stream_manager, None)


    def _change_image(self, menuitem):
        self.main_window.main_panel.change_image()


    def _change_details(self, menuitem):
        self.main_window.main_panel.change_details()


    def _add_account(self, menuitem):
        self.main_window.main_panel.add_account()


    def _remove_account(self, menuitem):
        if not self.main_window.main_panel.get_sending():
            account = self.main_window.main_panel.get_current_account()

            if account is not None:
                if run_confirmation_dialog(self.get_toplevel(), u'Remove current account',
                                           u'Are you sure you want to remove the account <i>{}</i>?'.format(account.get_user_nick()),
                                           u'All streams of this account will be closed.'):
                    stream_thread = self.main_window.stream_manager.close_all(account)

                    self.main_window.account_manager.remove(account, stream_thread)


    def _preferences(self, menuitem):
        self.main_window.setting_window.present()


    def _about(self, menuitem):
        self.main_window.run_about_dialog()


    def __init__(self, main_window, accel_group):
        self.main_window = main_window

        file_separators = []

        new_tweets_separators = []

        new_tweets_home_mentions_subsubitem = Gtk.MenuItem(tweets.HOME_MENTIONS_NAME, False)
        new_tweets_home_mentions_subsubitem.connect(u'activate', self._new_tweets_home_mentions)
        new_tweets_home_mentions_subsubitem.show()

        new_tweets_separators.append(Gtk.SeparatorMenuItem())
        new_tweets_separators[0].show()

        new_tweets_home_subsubitem = Gtk.MenuItem(tweets.HOME_NAME, False)
        new_tweets_home_subsubitem.connect(u'activate', self._new_tweets_home)
        new_tweets_home_subsubitem.show()

        new_tweets_mentions_subsubitem = Gtk.MenuItem(tweets.MENTIONS_NAME, False)
        new_tweets_mentions_subsubitem.connect(u'activate', self._new_tweets_mentions)
        new_tweets_mentions_subsubitem.show()

        new_tweets_sent_by_me_subsubitem = Gtk.MenuItem(tweets.SENT_BY_ME_NAME, False)
        new_tweets_sent_by_me_subsubitem.connect(u'activate', self._new_tweets_sent_by_me)
        new_tweets_sent_by_me_subsubitem.show()

        new_tweets_separators.append(Gtk.SeparatorMenuItem())
        new_tweets_separators[1].show()

        new_tweets_favorited_by_me_subsubitem = Gtk.MenuItem(tweets.FAVORITED_BY_ME_NAME, False)
        new_tweets_favorited_by_me_subsubitem.connect(u'activate', self._new_tweets_favorited_by_me)
        new_tweets_favorited_by_me_subsubitem.show()

        new_tweets_retweeted_by_me_subsubitem = Gtk.MenuItem(tweets.RETWEETED_BY_ME_NAME, False)
        new_tweets_retweeted_by_me_subsubitem.connect(u'activate', self._new_tweets_retweeted_by_me)
        new_tweets_retweeted_by_me_subsubitem.show()

        new_tweets_retweeted_to_me_subsubitem = Gtk.MenuItem(tweets.RETWEETED_TO_ME_NAME, False)
        new_tweets_retweeted_to_me_subsubitem.connect(u'activate', self._new_tweets_retweeted_to_me)
        new_tweets_retweeted_to_me_subsubitem.show()

        new_tweets_retweeted_from_me_subsubitem = Gtk.MenuItem(tweets.RETWEETED_FROM_ME_NAME, False)
        new_tweets_retweeted_from_me_subsubitem.connect(u'activate', self._new_tweets_retweeted_from_me)
        new_tweets_retweeted_from_me_subsubitem.show()

        new_tweets_subsubmenu = Gtk.Menu()
        new_tweets_subsubmenu.append(new_tweets_home_mentions_subsubitem)
        new_tweets_subsubmenu.append(new_tweets_separators[0])
        new_tweets_subsubmenu.append(new_tweets_home_subsubitem)
        new_tweets_subsubmenu.append(new_tweets_mentions_subsubitem)
        new_tweets_subsubmenu.append(new_tweets_sent_by_me_subsubitem)
        new_tweets_subsubmenu.append(new_tweets_separators[1])
        new_tweets_subsubmenu.append(new_tweets_favorited_by_me_subsubitem)
        new_tweets_subsubmenu.append(new_tweets_retweeted_by_me_subsubitem)
        new_tweets_subsubmenu.append(new_tweets_retweeted_to_me_subsubitem)
        new_tweets_subsubmenu.append(new_tweets_retweeted_from_me_subsubitem)

        new_tweets_subitem = Gtk.MenuItem(u'New _tweet stream', True)
        new_tweets_subitem.set_submenu(new_tweets_subsubmenu)
        new_tweets_subitem.show()

        new_users_followers_subsubitem = Gtk.MenuItem(users.FOLLOWERS_NAME, False)
        new_users_followers_subsubitem.connect(u'activate', self._new_users_my_followers)
        new_users_followers_subsubitem.show()

        new_users_friends_subsubitem = Gtk.MenuItem(users.FRIENDS_NAME, False)
        new_users_friends_subsubitem.connect(u'activate', self._new_users_my_friends)
        new_users_friends_subsubitem.show()

        new_users_subsubmenu = Gtk.Menu()
        new_users_subsubmenu.append(new_users_followers_subsubitem)
        new_users_subsubmenu.append(new_users_friends_subsubitem)

        new_users_subitem = Gtk.MenuItem(u'New _user stream', True)
        new_users_subitem.set_submenu(new_users_subsubmenu)
        new_users_subitem.show()

        new_groups_my_memberships_subsubitem = Gtk.MenuItem(groups.MY_MEMBERSHIPS_NAME, False)
        new_groups_my_memberships_subsubitem.connect(u'activate', self._new_groups_my_memberships)
        new_groups_my_memberships_subsubitem.show()

        new_groups_my_subscriptions_subsubitem = Gtk.MenuItem(groups.MY_SUBSCRIPTIONS_NAME, False)
        new_groups_my_subscriptions_subsubitem.connect(u'activate', self._new_groups_my_subscriptions)
        new_groups_my_subscriptions_subsubitem.show()

        new_groups_my_groups_subsubitem = Gtk.MenuItem(groups.MY_GROUPS_NAME, False)
        new_groups_my_groups_subsubitem.connect(u'activate', self._new_groups_my_groups)
        new_groups_my_groups_subsubitem.show()

        new_groups_subsubmenu = Gtk.Menu()
        new_groups_subsubmenu.append(new_groups_my_memberships_subsubitem)
        new_groups_subsubmenu.append(new_groups_my_subscriptions_subsubitem)
        new_groups_subsubmenu.append(new_groups_my_groups_subsubitem)

        new_groups_subitem = Gtk.MenuItem(u'New _list stream', True)
        new_groups_subitem.set_submenu(new_groups_subsubmenu)
        new_groups_subitem.show()

        new_messages_all_subsubitem = Gtk.MenuItem(messages.ALL_NAME, False)
        new_messages_all_subsubitem.connect(u'activate', self._new_messages_all)
        new_messages_all_subsubitem.show()

        new_messages_received_subsubitem = Gtk.MenuItem(messages.RECEIVED_NAME, False)
        new_messages_received_subsubitem.connect(u'activate', self._new_messages_received)
        new_messages_received_subsubitem.show()

        new_messages_sent_subsubitem = Gtk.MenuItem(messages.SENT_NAME, False)
        new_messages_sent_subsubitem.connect(u'activate', self._new_messages_sent)
        new_messages_sent_subsubitem.show()

        new_messages_subsubmenu = Gtk.Menu()
        new_messages_subsubmenu.append(new_messages_all_subsubitem)
        new_messages_subsubmenu.append(new_messages_received_subsubitem)
        new_messages_subsubmenu.append(new_messages_sent_subsubitem)

        new_messages_subitem = Gtk.MenuItem(u'New _message stream', True)
        new_messages_subitem.set_submenu(new_messages_subsubmenu)
        new_messages_subitem.show()

        file_separators.append(Gtk.SeparatorMenuItem())
        file_separators[0].show()

        trends_subitem = Gtk.MenuItem(u'_Worldwide trends', True)
        trends_subitem.connect(u'activate', self._new_trends)
        trends_subitem.show()

        categories_subitem = Gtk.MenuItem(u'_Suggestion categories', True)
        categories_subitem.connect(u'activate', self._new_categories)
        categories_subitem.show()

        search_subitem = Gtk.MenuItem(u'Searc_h\u2026', True)
        search_subitem.connect(u'activate', self._search)
        search_subitem.add_accelerator(u'activate', accel_group, Gtk.keysyms.f, Gdk.CONTROL_MASK, Gtk.ACCEL_VISIBLE)
        search_subitem.show()

        file_separators.append(Gtk.SeparatorMenuItem())
        file_separators[1].show()

        self.reload_subitem = Gtk.MenuItem(u'Reload _images', True)
        self.reload_subitem.connect(u'activate', self._reload)
        self.reload_subitem.show()

        refresh_subitem = Gtk.MenuItem(u'_Refresh', True)
        refresh_subitem.connect(u'activate', self._refresh)
        refresh_subitem.add_accelerator(u'activate', accel_group, Gtk.keysyms.r, Gdk.CONTROL_MASK, Gtk.ACCEL_VISIBLE)
        refresh_subitem.show()

        file_separators.append(Gtk.SeparatorMenuItem())
        file_separators[2].show()

        close_subitem = Gtk.MenuItem(u'Clos_e', True)
        close_subitem.connect(u'activate', self._close)
        close_subitem.add_accelerator(u'activate', accel_group, Gtk.keysyms.w, Gdk.CONTROL_MASK, Gtk.ACCEL_VISIBLE)
        close_subitem.show()

        quit_subitem = Gtk.MenuItem(u'_Quit', True)
        quit_subitem.connect(u'activate', self._quit)
        quit_subitem.add_accelerator(u'activate', accel_group, Gtk.keysyms.q, Gdk.CONTROL_MASK, Gtk.ACCEL_VISIBLE)
        quit_subitem.show()

        file_submenu = Gtk.Menu()
        file_submenu.append(new_tweets_subitem)
        file_submenu.append(new_users_subitem)
        file_submenu.append(new_groups_subitem)
        file_submenu.append(new_messages_subitem)
        file_submenu.append(file_separators[0])
        file_submenu.append(trends_subitem)
        file_submenu.append(categories_subitem)
        file_submenu.append(search_subitem)
        file_submenu.append(file_separators[1])
        file_submenu.append(self.reload_subitem)
        file_submenu.append(refresh_subitem)
        file_submenu.append(file_separators[2])
        file_submenu.append(close_subitem)
        file_submenu.append(quit_subitem)

        self.file_item = Gtk.MenuItem(u'_File', True)
        self.file_item.set_submenu(file_submenu)
        self.file_item.show()

        edit_separators = []

        create_group_subitem = Gtk.MenuItem(u'Create _list\u2026', True)
        create_group_subitem.connect(u'activate', self._create_group)
        create_group_subitem.show()

        edit_separators.append(Gtk.SeparatorMenuItem())
        edit_separators[0].show()

        change_image_subitem = Gtk.MenuItem(u'Change profile _image\u2026', True)
        change_image_subitem.connect(u'activate', self._change_image)
        change_image_subitem.show()

        change_details_subitem = Gtk.MenuItem(u'Change profile _details\u2026', True)
        change_details_subitem.connect(u'activate', self._change_details)
        change_details_subitem.show()

        edit_separators.append(Gtk.SeparatorMenuItem())
        edit_separators[1].show()

        add_account_subitem = Gtk.MenuItem(u'_Add new account', True)
        add_account_subitem.connect(u'activate', self._add_account)
        add_account_subitem.show()

        remove_account_subitem = Gtk.MenuItem(u'_Remove current account\u2026', True)
        remove_account_subitem.connect(u'activate', self._remove_account)
        remove_account_subitem.show()

        edit_separators.append(Gtk.SeparatorMenuItem())
        edit_separators[2].show()

        preferences_subitem = Gtk.MenuItem(u'_Preferences', True)
        preferences_subitem.connect(u'activate', self._preferences)
        preferences_subitem.show()

        edit_submenu = Gtk.Menu()
        edit_submenu.append(create_group_subitem)
        edit_submenu.append(edit_separators[0])
        edit_submenu.append(change_image_subitem)
        edit_submenu.append(change_details_subitem)
        edit_submenu.append(edit_separators[1])
        edit_submenu.append(add_account_subitem)
        edit_submenu.append(remove_account_subitem)
        edit_submenu.append(edit_separators[2])
        edit_submenu.append(preferences_subitem)

        self.edit_item = Gtk.MenuItem(u'_Edit', True)
        self.edit_item.set_submenu(edit_submenu)
        self.edit_item.show()

        about_subitem = Gtk.MenuItem(u'_About', True)
        about_subitem.connect(u'activate', self._about)
        about_subitem.show()

        help_submenu = Gtk.Menu()
        help_submenu.append(about_subitem)

        self.help_item = Gtk.MenuItem(u'_Help', True)
        self.help_item.set_submenu(help_submenu)
        self.help_item.show()

        self.items = [self.file_item,
                      self.edit_item,
                      self.help_item]

        super(Bar, self).__init__()

        for item in self.items:
            self.append(item)

        self.show()

        self.thread = None


    def isolate(self):
        if self.thread is not None:
            self.thread.join()


    def set_sensitive(self, sensitive):
        for item in self.items:
            item.set_sensitive(sensitive)
