# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk


from polly.gui import DEFAULT_BORDER



class Window(Gtk.Window):
    def _update_shortening_enabled(self):
        enabled = self.shortening_enabled_checkbutton.get_active()

        self.shortening_minimum_box.set_sensitive(enabled)

        self.shortening_name_box.set_sensitive(enabled)


    def _update_proxy_enabled(self):
        enabled = self.proxy_enabled_checkbutton.get_active()

        self.proxy_url_box.set_sensitive(enabled)

        self.proxy_type_box.set_sensitive(enabled)


    def _on_shortening_enabled_checkbutton_toggled(self, togglebutton):
        self._update_shortening_enabled()


    def _on_proxy_enabled_checkbutton_toggled(self, togglebutton):
        self._update_proxy_enabled()


    def _on_close_button_clicked(self, button):
        self.hide()


    def _on_accel(self, accel_group, acceleratable, keyval, modifier):
        self.hide()


    def __init__(self, proxy_controller, shortener_controller, iconset_controller, account_manager, main_panel, stream_manager):
        general_stream_interval_left_label = Gtk.Label(u'Refresh streams each')
        general_stream_interval_left_label.show()

        general_stream_interval_right_label = Gtk.Label(u'minutes')
        general_stream_interval_right_label.set_alignment(0.0, 0.5)
        general_stream_interval_right_label.show()

        general_stream_interval_box = Gtk.HBox(False, DEFAULT_BORDER)
        general_stream_interval_box.pack_start(general_stream_interval_left_label, False, False, 0)
        general_stream_interval_box.pack_start(stream_manager.interval_spinbutton, False, False, 0)
        general_stream_interval_box.pack_start(general_stream_interval_right_label, True, True, 0)
        general_stream_interval_box.show()

        general_stream_maximum_left_label = Gtk.Label(u'Keep only the')
        general_stream_maximum_left_label.show()

        general_stream_maximum_right_label = Gtk.Label(u'most recent posts')
        general_stream_maximum_right_label.set_alignment(0.0, 0.5)
        general_stream_maximum_right_label.show()

        general_stream_maximum_box = Gtk.HBox(False, DEFAULT_BORDER)
        general_stream_maximum_box.pack_start(general_stream_maximum_left_label, False, False, 0)
        general_stream_maximum_box.pack_start(stream_manager.maximum_spinbutton, False, False, 0)
        general_stream_maximum_box.pack_start(general_stream_maximum_right_label, True, True, 0)
        general_stream_maximum_box.show()

        general_box = Gtk.VBox(True, 2 * DEFAULT_BORDER)
        general_box.set_border_width(2 * DEFAULT_BORDER)
        general_box.pack_start(general_stream_interval_box, True, False, 0)
        general_box.pack_start(general_stream_maximum_box, True, False, 0)
        general_box.show()

        general_label = Gtk.Label(u'General')
        general_label.show()

        interface_hide_on_box = Gtk.VBox(True, DEFAULT_BORDER)
        interface_hide_on_box.pack_start(main_panel.hide_on_send_checkbutton, True, False, 0)
        interface_hide_on_box.pack_start(main_panel.hide_on_cancel_checkbutton, True, False, 0)
        interface_hide_on_box.show()

        interface_iconset_label = Gtk.Label(u'Iconset:')
        interface_iconset_label.show()

        interface_iconset_box = Gtk.HBox(False, DEFAULT_BORDER)
        interface_iconset_box.pack_start(interface_iconset_label, False, False, 0)
        interface_iconset_box.pack_start(iconset_controller.name_combobox, True, True, 0)
        interface_iconset_box.pack_start(iconset_controller.add_button, False, False, 0)
        interface_iconset_box.pack_start(iconset_controller.remove_button, False, False, 0)
        interface_iconset_box.show()

        interface_box = Gtk.VBox(False, 2 * DEFAULT_BORDER)
        interface_box.set_border_width(2 * DEFAULT_BORDER)
        interface_box.pack_start(stream_manager.preview_checkbutton, True, False, 0)
        interface_box.pack_start(interface_hide_on_box, True, False, 0)
        interface_box.pack_start(interface_iconset_box, True, False, 0)
        interface_box.show()

        interface_label = Gtk.Label(u'Interface')
        interface_label.show()

        composing_override_box = Gtk.VBox(True, DEFAULT_BORDER)
        composing_override_box.pack_start(main_panel.entry_controller.override_radiobuttons[0], True, False, 0)
        composing_override_box.pack_start(main_panel.entry_controller.override_radiobuttons[1], True, False, 0)
        composing_override_box.show()

        composing_box = Gtk.VBox(False, 2 * DEFAULT_BORDER)
        composing_box.set_border_width(2 * DEFAULT_BORDER)
        composing_box.pack_start(main_panel.entry_controller.check_checkbutton, True, False, 0)
        composing_box.pack_start(composing_override_box, True, False, 0)
        composing_box.show()

        composing_label = Gtk.Label(u'Composing')
        composing_label.show()

        self.shortening_enabled_checkbutton = shortener_controller.enabled_checkbutton
        self.shortening_enabled_checkbutton.connect(u'toggled', self._on_shortening_enabled_checkbutton_toggled)

        shortening_minimum_left_label = Gtk.Label(u'Only shorten URLs with')
        shortening_minimum_left_label.show()

        shortening_minimum_right_label = Gtk.Label(u'characters or more')
        shortening_minimum_right_label.set_alignment(0.0, 0.5)
        shortening_minimum_right_label.show()

        self.shortening_minimum_box = Gtk.HBox(False, DEFAULT_BORDER)
        self.shortening_minimum_box.pack_start(shortening_minimum_left_label, False, False, 0)
        self.shortening_minimum_box.pack_start(shortener_controller.minimum_spinbutton, False, False, 0)
        self.shortening_minimum_box.pack_start(shortening_minimum_right_label, True, True, 0)
        self.shortening_minimum_box.show()

        shortening_name_label = Gtk.Label(u'Shortener:')
        shortening_name_label.show()

        self.shortening_name_box = Gtk.HBox(False, DEFAULT_BORDER)
        self.shortening_name_box.pack_start(shortening_name_label, False, False, 0)
        self.shortening_name_box.pack_start(shortener_controller.name_combobox, True, True, 0)
        self.shortening_name_box.show()

        shortening_box = Gtk.VBox(True, 2 * DEFAULT_BORDER)
        shortening_box.set_border_width(2 * DEFAULT_BORDER)
        shortening_box.pack_start(self.shortening_enabled_checkbutton, True, False, 0)
        shortening_box.pack_start(self.shortening_minimum_box, True, False, 0)
        shortening_box.pack_start(self.shortening_name_box, True, False, 0)
        shortening_box.show()

        shortening_label = Gtk.Label(u'Shortening')
        shortening_label.show()

        notification_box = Gtk.VBox(True, 2 * DEFAULT_BORDER)
        notification_box.set_border_width(2 * DEFAULT_BORDER)
        notification_box.pack_start(stream_manager.policy_radiobuttons[0], True, False, 0)
        notification_box.pack_start(stream_manager.policy_radiobuttons[1], True, False, 0)
        notification_box.pack_start(stream_manager.policy_radiobuttons[2], True, False, 0)
        notification_box.pack_start(stream_manager.policy_radiobuttons[3], True, False, 0)
        notification_box.pack_start(stream_manager.policy_radiobuttons[4], True, False, 0)
        notification_box.show()

        notification_label = Gtk.Label(u'Notification')
        notification_label.show()

        self.proxy_enabled_checkbutton = proxy_controller.enabled_checkbutton
        self.proxy_enabled_checkbutton.connect(u'toggled', self._on_proxy_enabled_checkbutton_toggled)

        proxy_url_host_label = Gtk.Label(u'Host:')
        proxy_url_host_label.show()

        proxy_url_port_label = Gtk.Label(u'Port:')
        proxy_url_port_label.show()

        self.proxy_url_box = Gtk.HBox(False, DEFAULT_BORDER)
        self.proxy_url_box.pack_start(proxy_url_host_label, False, False, 0)
        self.proxy_url_box.pack_start(proxy_controller.host_entry, True, True, 0)
        self.proxy_url_box.pack_start(proxy_url_port_label, False, False, 0)
        self.proxy_url_box.pack_start(proxy_controller.port_spinbutton, False, False, 0)
        self.proxy_url_box.show()

        self.proxy_type_box = Gtk.HBox(True, DEFAULT_BORDER)
        self.proxy_type_box.pack_start(proxy_controller.type_radiobuttons[0], True, True, 0)
        self.proxy_type_box.pack_start(proxy_controller.type_radiobuttons[1], True, True, 0)
        self.proxy_type_box.pack_start(proxy_controller.type_radiobuttons[2], True, True, 0)
        self.proxy_type_box.show()

        proxy_box = Gtk.VBox(True, 2 * DEFAULT_BORDER)
        proxy_box.set_border_width(2 * DEFAULT_BORDER)
        proxy_box.pack_start(self.proxy_enabled_checkbutton, True, False, 0)
        proxy_box.pack_start(self.proxy_url_box, True, False, 0)
        proxy_box.pack_start(self.proxy_type_box, True, False, 0)
        proxy_box.show()

        proxy_label = Gtk.Label(u'Proxy')
        proxy_label.show()

        notebook = Gtk.Notebook()
        notebook.append_page(general_box, general_label)
        notebook.append_page(interface_box, interface_label)
        notebook.append_page(composing_box, composing_label)
        notebook.append_page(shortening_box, shortening_label)
        notebook.append_page(notification_box, notification_label)
        notebook.append_page(proxy_box, proxy_label)
        notebook.show()

        close_button = Gtk.Button(None, Gtk.STOCK_CLOSE, False)
        close_button.connect(u'clicked', self._on_close_button_clicked)
        close_button.show()

        alignment = Gtk.Alignment(1.0, 0.5, 0.0, 0.0)
        alignment.add(close_button)
        alignment.show()

        box = Gtk.VBox(False, DEFAULT_BORDER)
        box.set_border_width(DEFAULT_BORDER)
        box.pack_start(notebook, True, True, 0)
        box.pack_start(alignment, False, False, 0)
        box.show()

        accel_group = Gtk.AccelGroup()
        accel_group.connect_group(Gtk.keysyms.Escape, 0, Gtk.ACCEL_VISIBLE, self._on_accel)

        super(Window, self).__init__(Gtk.WINDOW_TOPLEVEL)
        self.set_resizable(False)
        self.set_title(u'Preferences')
        self.connect(u'delete-event', self.hide_on_delete)
        self.add_accel_group(accel_group)
        self.add(box)

        self._update_shortening_enabled()

        self._update_proxy_enabled()


    def on_dialog_run(self):
        pass


    def on_dialog_destroy(self):
        pass
