# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import os, shutil

from threading import Lock, Condition

from time import localtime, sleep, strftime, time


import gtk as Gtk

import pango as Pango

from gtk import gdk as Gdk


from polly import dispatch



class Bar(Gtk.VBox):
    def _wait(self):
        self.condition.acquire()

        if not self.finished:
            self.condition.wait(10.0)

        self.condition.release()

        Gdk.threads_enter()

        self.hide()

        Gdk.threads_leave()


    def _notify(self):
        self.condition.acquire()

        self.finished = True

        self.condition.notify()

        self.condition.release()

        if self.thread is not None:
            self.thread.join()


    def __init__(self, cache_path, cache_path_lock):
        self.path = os.path.join(cache_path, u'log')

        self.lock = Lock()

        self.condition = Condition()

        separator = Gtk.HSeparator()
        separator.show()

        self.label = Gtk.Label(None)
        self.label.modify_fg(Gtk.STATE_NORMAL, Gdk.color_parse(u'white'))
        self.label.set_ellipsize(Pango.ELLIPSIZE_END)
        self.label.show()

        box = Gtk.EventBox()
        box.modify_bg(Gtk.STATE_NORMAL, Gdk.color_parse(u'red'))
        box.add(self.label)
        box.show()

        super(Bar, self).__init__(False, 0)
        self.pack_start(separator, False, False, 0)
        self.pack_start(box, True, True, 0)

        cache_path_lock.acquire()

        try:
            if not os.path.isdir(cache_path):
                os.makedirs(cache_path)
        except OSError:
            pass
        finally:
            cache_path_lock.release()

        if os.path.exists(self.path):
            try:
                if os.path.isfile(self.path):
                    os.remove(self.path)
                elif os.path.isdir(self.path):
                    shutil.rmtree(self.path)
            except OSError:
                pass

        self.thread = None


    def isolate(self):
        self._notify()


    def popup(self, text):
        datetime = localtime(time())

        large_time = strftime(u'%a %d %b %Y %H:%M:%S', datetime)

        self.lock.acquire()

        self._notify()

        sleep(0.040)

        Gdk.threads_enter()

        self.label.set_text(text)
        self.label.set_tooltip_text(text)

        self.show()

        Gdk.threads_leave()

        self.finished = False

        self.thread = dispatch(self._wait)

        try:
            with open(self.path, u'a') as file:
                file.write(u'{} {}\n'.format(large_time, text))
        except IOError:
            pass

        self.lock.release()
