# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from threading import Lock, Condition


import gtk as Gtk

import pango as Pango

from gtk import gdk as Gdk


from polly import dispatch

from polly.gui import DEFAULT_BORDER, NICK_DIMENSION, CLOSE_TOOLTIP, CONNECTING_TEXT

from polly.gui.twitter import post



PAGE_SIZE = 20



class Stream(Gtk.EventBox):
    def __del__(self):
        print u'DEBUG: STREAM DELETED'
    def _build_menu(self):
        if self.refreshable:
            refresh_menuitem = Gtk.MenuItem(u'Refresh this stream', False)
            refresh_menuitem.connect(u'activate', self._on_refresh_menuitem_activate)
            refresh_menuitem.show()

            self.menu.append(refresh_menuitem)


    def _update_normal_fg(self):
        normal_fg = self.color_scheme.normal_fg

        self.viewport.modify_fg(Gtk.STATE_NORMAL, normal_fg)

        for spinner in self.spinners:
            spinner.modify_fg(Gtk.STATE_NORMAL, normal_fg)


    def _update_normal_bg(self):
        normal_bg = self.color_scheme.normal_bg

        self.viewport.modify_bg(Gtk.STATE_NORMAL, normal_bg)


    def _update_font(self):
        unit = self.font_scheme.unit

        char = self.font_scheme.char

        width = self.fixed_width + self.unit_dimension * unit + self.char_dimension * char

        height = self.fixed_height + post.AVATAR_DIMENSION * unit

        self.drag_box.set_size_request(-1, unit)

        self.viewport.set_size_request(width, height)


    def _on_close_button_clicked(self):
        self.manager.close(self)


    def _on_expose_event(self, widget, event):
        if self.key_box.is_focus():
            while True:
                parent = widget.get_parent()

                if parent.get_has_window():
                    break

                widget = parent

            parent_style = parent.get_style()

            parent_style.paint_focus(self.key_box.get_window(), Gtk.STATE_NORMAL, event.area, self.key_box, u'eventbox', 0, 0, -1, -1)

        return False


    def _on_focus_in_event(self, widget, event):
        if self.dropped:
            toplevel = self.get_toplevel()

            if toplevel.focused:
                self.popup()

                self.enter()

        self.key_box.queue_draw()

        return False


    def _on_focus_out_event(self, widget, event):
        self.key_box.queue_draw()

        return False


    def _on_inner_key_press_event(self, widget, event):
        toplevel = self.get_toplevel()

        return toplevel.press(event)


    def _on_enter_notify_event(self, widget, event):
        self.enter()

        return False


    def _on_leave_notify_event(self, widget, event):
        allocation = self.drag_box.get_allocation()

        if (event.x < 0 or event.x >= allocation.width or
            event.y < 0 or event.y >= allocation.height):
            self.leave()

        return False


    def _on_unrealize(self, widget):
        if self.inside:
            self.leave()


    def _on_button_press_event(self, widget, event):
        if event.button == 3:
            self.menu.popup(None, None, None, 3, event.time, None)

        return False


    def _on_button_release_event(self, widget, event):
        if event.button == 1:
            self.key_box.grab_focus()

        return False


    def _on_outer_key_press_event(self, widget, event):
        if (event.keyval == Gtk.keysyms.Menu or
           (event.keyval == Gtk.keysyms.F10 and event.state & Gdk.SHIFT_MASK)):
            time = Gtk.get_current_event_time()

            self.menu.popup(None, None, None, 0, time, None)

        return False


    def _on_drag_begin(self, widget, drag_context):
        self.dropped = False

        self.manager.drag_source = self

        pixmap = self.viewport.get_snapshot()

        bits_per_sample = pixmap.get_depth() / 3

        width, height = pixmap.get_size()

        width -= 2 * DEFAULT_BORDER

        height -= 2 * DEFAULT_BORDER

        pixbuf_large = Gdk.Pixbuf(Gdk.COLORSPACE_RGB, False, bits_per_sample, width, height)

        pixbuf_large.get_from_drawable(pixmap, pixmap.get_colormap(), DEFAULT_BORDER, DEFAULT_BORDER, 0, 0, width, height)

        pixbuf_small = Gdk.Pixbuf(Gdk.COLORSPACE_RGB, False, bits_per_sample, width / 5, height / 5)

        pixbuf_large.scale(pixbuf_small, 0, 0, width / 5, height / 5, 0, 0, 0.2, 0.2, Gdk.INTERP_BILINEAR)

        drag_context.set_icon_pixbuf(pixbuf_small, 0, 0)

        self.queue_draw()


    def _on_drag_end(self, widget, drag_context):
        self.manager.drag_dest = None

        self.manager.drag_scroll = 0

        self.manager.drag_source = None

        self.dropped = True


    def _on_refresh_menuitem_activate(self, menuitem):
        self.refresh(False)


    def _on_color_normal_fg_changed(self):
        Gdk.threads_enter()

        self._update_normal_fg()

        self.queue_draw()

        Gdk.threads_leave()


    def _on_color_normal_bg_changed(self):
        Gdk.threads_enter()

        self._update_normal_bg()

        self.queue_draw()

        Gdk.threads_leave()


    def _on_font_changed(self):
        Gdk.threads_enter()

        self._update_font()

        Gdk.threads_leave()


    def _finalize(self):
        pass


    def _update_loop(self):
        self.close_button = self.button_factory.build(u'close', self._on_close_button_clicked)

        self.identifier = self.account.build_identifier()

        Gdk.threads_enter()

        self.close_button.set_toggled(True)
        self.close_button.box.set_tooltip_text(CLOSE_TOOLTIP)

        self.button_box.pack_end(self.close_button, True, True, 0)

        self.tab_box.pack_end(self.identifier, False, False, 0)

        Gdk.threads_leave()

        self._build_buttons()

        while True:
            self.stream_lock.acquire()

            self.identifier.set_spinning(True)

            self._update()

            self.identifier.set_spinning(False)

            self.stream_lock.release()

            self.update_condition.acquire()

            if not self.update_finished:
                if self.refreshable:
                    timeout = 60.0 * self.manager.get_interval()

                    self.update_condition.wait(timeout)
                else:
                    self.update_condition.wait()

            update_finished = self.update_finished

            self.update_condition.release()

            if update_finished:
                break

        self._finalize()

        self._isolate_buttons()

        self.identifier.isolate()

        self.close_button.isolate()


    def __init__(self, key, account, parameter, refreshable, color_scheme, font_scheme, button_factory, status_bar, manager):
        self.key = key

        self.account = account

        self.parameter = parameter

        self.refreshable = refreshable

        self.color_scheme = color_scheme

        self.font_scheme = font_scheme

        self.button_factory = button_factory

        self.status_bar = status_bar

        self.manager = manager

        self.inside = False

        self.dropped = True

        self.update_finished = False

        self.fixed_width = 7 * DEFAULT_BORDER

        self.fixed_height = 2 * DEFAULT_BORDER

        self.unit_dimension = post.AVATAR_DIMENSION + 4

        self.char_dimension = long(0.8 * NICK_DIMENSION + 0.5)

        self.stream_lock = Lock()

        self.update_condition = Condition()

        self.button_box = Gtk.HBox(True, DEFAULT_BORDER)

        self.label = Gtk.Label(CONNECTING_TEXT)
        self.label.set_alignment(0.0, 0.5)
        self.label.set_ellipsize(Pango.ELLIPSIZE_END)
        self.label.show()

        self.key_box = Gtk.EventBox()
        self.key_box.set_can_focus(True)
        self.key_box.connect_after(u'expose-event', self._on_expose_event)
        self.key_box.connect(u'focus-in-event', self._on_focus_in_event)
        self.key_box.connect(u'focus-out-event', self._on_focus_out_event)
        self.key_box.connect(u'key-press-event', self._on_inner_key_press_event)
        self.key_box.add(self.label)
        self.key_box.show()

        self.tab_box = Gtk.HBox(False, DEFAULT_BORDER)
        self.tab_box.pack_end(self.button_box, False, False, 0)
        self.tab_box.pack_end(self.key_box, True, True, 0)
        self.tab_box.show()

        self.drag_box = Gtk.EventBox()
        self.drag_box.drag_source_set(Gdk.BUTTON1_MASK, [], 0)
        self.drag_box.connect(u'enter-notify-event', self._on_enter_notify_event)
        self.drag_box.connect(u'leave-notify-event', self._on_leave_notify_event)
        self.drag_box.connect(u'unrealize', self._on_unrealize)
        self.drag_box.connect(u'button-press-event', self._on_button_press_event)
        self.drag_box.connect(u'button-release-event', self._on_button_release_event)
        self.drag_box.connect(u'key-press-event', self._on_outer_key_press_event)
        self.drag_box.connect(u'drag-begin', self._on_drag_begin)
        self.drag_box.connect(u'drag-end', self._on_drag_end)
        self.drag_box.add(self.tab_box)
        self.drag_box.show()

        self.inner_box = Gtk.VBox(False, 0)
        self.inner_box.show()

        scrolledwindow = Gtk.ScrolledWindow(None, None)
        scrolledwindow.set_policy(Gtk.POLICY_NEVER, Gtk.POLICY_ALWAYS)
        scrolledwindow.add_with_viewport(self.inner_box)
        scrolledwindow.show()

        self.adjustment = scrolledwindow.get_vadjustment()

        self.viewport = scrolledwindow.get_child()

        self.outer_box = Gtk.VBox(False, DEFAULT_BORDER)
        self.outer_box.set_border_width(2)
        self.outer_box.pack_start(self.drag_box, False, False, 0)
        self.outer_box.pack_start(scrolledwindow, True, True, 0)
        self.outer_box.show()

        super(Stream, self).__init__()
        self.add(self.outer_box)
        self.show()

        self.menu = Gtk.Menu()

        self._build_menu()

        self.spinners = []

        self.color_scheme.block_updates()

        self.font_scheme.block_updates()

        self._update_normal_fg()

        self._update_normal_bg()

        self._update_font()

        self.color_normal_fg_uid = self.color_scheme.normal_fg_notifier.connect(self._on_color_normal_fg_changed)

        self.color_normal_bg_uid = self.color_scheme.normal_bg_notifier.connect(self._on_color_normal_bg_changed)

        self.font_uid = self.font_scheme.notifier.connect(self._on_font_changed)

        self.color_scheme.allow_updates()

        self.font_scheme.allow_updates()

        self.current_item = None


    def isolate(self):
        self.font_scheme.block_updates()

        self.color_scheme.block_updates()

        self.font_scheme.notifier.disconnect(self.font_uid)

        self.color_scheme.normal_bg_notifier.disconnect(self.color_normal_bg_uid)

        self.color_scheme.normal_fg_notifier.disconnect(self.color_normal_fg_uid)

        self.font_scheme.allow_updates()

        self.color_scheme.allow_updates()

        Gdk.threads_enter()

        self.menu.destroy()

        Gdk.threads_leave()


    def start(self):
        self.update_thread = dispatch(self._update_loop)


    def refresh(self, update_finished):
        self.update_condition.acquire()

        self.update_finished = update_finished

        self.update_condition.notify()

        self.update_condition.release()


    def stop(self):
        self.refresh(True)

        self.update_thread.join()


    def synchronize(self):
        pass


    def hide_indicator(self):
        pass


    def reset_indicator(self):
        pass


    def enter(self):
        if not self.inside:
            if self.manager.current_stream is not None:
                self.manager.current_stream.leave()

            self.manager.current_stream = self

            self.inside = True

            self.button_box.show()


    def leave(self):
        self.button_box.hide()

        self.inside = False

        if self.manager.current_stream == self:
            self.manager.current_stream = None


    def popup(self):
        allocation = self.get_allocation()

        adjustment = self.manager.adjustment

        left = float(allocation.x)

        value = adjustment.get_value()

        if left < value:
            adjustment.set_value(left)
        else:
            right = left + float(allocation.width)

            page_size = adjustment.get_page_size()

            if right > value + page_size:
                adjustment.set_value(right - page_size)


    def popup_child(self, child, border):
        allocation = child.get_allocation()

        top = float(allocation.y - border)

        value = self.adjustment.get_value()

        if top < value:
            self.adjustment.set_value(top)
        else:
            bottom = top + float(allocation.height + 2 * border)

            page_size = self.adjustment.get_page_size()

            if bottom > value + page_size:
                self.adjustment.set_value(bottom - page_size)



import leads, posts, topics, manager
