# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from threading import Lock

from time import time


import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch

from polly.twitter.account import Error

from polly.gui import DEFAULT_BORDER, CONNECTING_TEXT

from polly.gui.stream import PAGE_SIZE, Stream



class Posts(Stream):
    def _build_menu(self):
        super(Posts, self)._build_menu()

        self.clear_menuitem = Gtk.MenuItem(u'Clear this stream', False)
        self.clear_menuitem.connect(u'activate', self._on_clear_menuitem_activate)
        self.clear_menuitem.show()

        self.menu.append(self.clear_menuitem)


    def _build_buttons(self):
        self.more_button = self.button_factory.build(u'more', self._on_more_button_clicked)

        Gdk.threads_enter()

        self.more_button.set_border_width(DEFAULT_BORDER)
        self.more_button.set_toggled(True)
        self.more_button.box.set_tooltip_text(u'Get more')
        self.more_button.spinner.set_tooltip_text(u'Getting more\u2026')

        self.more_button.box.connect(u'button-press-event', self._on_child_button_press_event)
        self.more_button.box.connect(u'focus-in-event', self._on_child_focus_in_event)

        self.inner_box.pack_start(self.more_button, False, False, 0)

        self.spinners.append(self.more_button.spinner)

        Gdk.threads_leave()


    def _isolate_buttons(self):
        self.more_button.isolate()


    def _set_indicator_name(self, indicator_name):
        self.indicator_name = indicator_name

        self.indicator_client.set_name(self.indicator_name)


    def _remove(self, previous_thread, post):
        if previous_thread is not None:
            previous_thread.join()

        post.isolate()

        Gdk.threads_enter()

        post.destroy()

        Gdk.threads_leave()


    def _clear(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        self.posts_lock.acquire()

        Gdk.threads_enter()

        posts = self.post_box.get_children()

        for post in posts:
            self.post_box.remove(post)

        Gdk.threads_leave()

        if self.since_id is not None:
            self.max_id = self.since_id + 1

        self.posts_lock.release()

        Gdk.threads_enter()

        self.clearings -= 1

        if self.clearings == 0:
            self.post_box.show()

        self.clear_menuitem.set_sensitive(True)

        Gdk.threads_leave()

        for post in posts:
            post.isolate()

            Gdk.threads_enter()

            post.destroy()

            Gdk.threads_leave()


    def _update(self):
        self.posts_lock.acquire()

        _, destroyed_posts = self._update_new(None)

        self.posts_lock.release()

        for post in destroyed_posts:
            post.isolate()

            Gdk.threads_enter()

            post.destroy()

            Gdk.threads_leave()


    def _update_more(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        self.posts_lock.acquire()

        self._update_old()

        self.posts_lock.release()

        Gdk.threads_enter()

        self.more_button.set_spinning(False)

        Gdk.threads_leave()


    def _update_new(self, max_id):
        updated_old = self.updated_old

        first_time = (self.since_id is None)

        built_posts = []

        try:
            if first_time:
                dataset = self._get_dataset(None, None, PAGE_SIZE)
            else:
                dataset = []

                while True:
                    sub_dataset = self._get_dataset(self.since_id - 1, max_id, 100)

                    if sub_dataset:
                        if sub_dataset[-1][0] == self.since_id:
                            dataset.extend(sub_dataset[:-1])

                            break
                        else:
                            dataset.extend(sub_dataset)

                            max_id = sub_dataset[-1][0] - 1
                    else:
                        break
        except Error as error:
            success = False

            self.status_bar.popup(error.message)
        else:
            success = True

            if dataset:
                timestamp = time()

                for data in reversed(dataset):
                    post = self._build(data)

                    post.synchronize(timestamp)

                    built_posts.append(post)

        if not updated_old:
            maximum = self.manager.get_maximum()

            Gdk.threads_enter()

            posts = self.post_box.get_children()

            Gdk.threads_leave()

            posts.reverse()

            if maximum > len(built_posts):
                candidate_posts = posts[:(len(built_posts) - maximum)]
            else:
                candidate_posts = posts

            destroyed_posts = []

            Gdk.threads_enter()

            height = long(self.adjustment.get_value() + self.adjustment.get_page_size())

            for post in candidate_posts:
                allocation = post.get_allocation()

                if allocation.y < height:
                    break

                destroyed_posts.append(post)

            for post in destroyed_posts:
                self.post_box.remove(post)

            Gdk.threads_leave()

            if destroyed_posts:
                if len(destroyed_posts) == len(posts):
                    self.max_id = self.since_id
                else:
                    index = posts.index(destroyed_posts[-1])

                    self.max_id = posts[index + 1].real_id

        if built_posts:
            self.since_id = built_posts[-1].real_id

            if self.max_id is None:
                self.max_id = built_posts[0].real_id

            Gdk.threads_enter()

            value = self.adjustment.get_value()

            if long(value) == 0:
                for post in built_posts:
                    post.revert_top_scrollings()
            else:
                for post in built_posts:
                    post.revert_two_scrollings()

            for post in built_posts:
                self.post_box.pack_end(post, False, False, 0)

            Gdk.threads_leave()

            if self.indicator_client:
                counts = {}

                for post in built_posts:
                    if not post.user_own:
                        counts[post.real_id] = post.log()

                if not first_time:
                    Gdk.threads_enter()

                    ancestor = self.get_ancestor(Gtk.Window)

                    if ancestor is None:
                        self.active = True
                    else:
                        self.active = ancestor.is_active()

                    active = self.active

                    Gdk.threads_leave()

                    if not active:
                        Gdk.threads_enter()

                        self.indicator_count += len(built_posts)

                        self.indicator_client.set_count(self.indicator_count)

                        Gdk.threads_leave()

                        policy = self.manager.get_policy()

                        if policy > 0:
                            for post in built_posts:
                                if not post.user_own:
                                    post.user_avatar.thread.join()

                                    Gdk.threads_enter()

                                    active = self.active

                                    Gdk.threads_leave()

                                    if active:
                                        break

                                    if counts[post.real_id] == 1:
                                        post.notify(policy)

        self.updated_old = False

        if updated_old:
            return success, []
        else:
            return success, destroyed_posts


    def _update_old(self):
        built_posts = []

        try:
            if self.max_id is None:
                dataset = self._get_dataset(None, None, PAGE_SIZE)
            else:
                dataset = self._get_dataset(None, self.max_id - 1, PAGE_SIZE)
        except Error as error:
            self.status_bar.popup(error.message)
        else:
            if dataset:
                timestamp = time()

                for data in dataset:
                    post = self._build(data)

                    post.synchronize(timestamp)

                    built_posts.append(post)

        if built_posts:
            if self.since_id is None:
                self.since_id = built_posts[0].real_id

            self.max_id = built_posts[-1].real_id

            for post in built_posts:
                Gdk.threads_enter()

                post.revert_top_scrollings()

                self.post_box.pack_end(post, False, False, 0)
                self.post_box.reorder_child(post, 0)

                Gdk.threads_leave()

        self.updated_old = True


    def _on_send_post(self):
        self.refresh(False)


    def _on_delete_post(self):
        self.posts_lock.acquire()

        Gdk.threads_enter()

        posts = self.post_box.get_children()

        Gdk.threads_leave()

        for post in posts:
            if post.real_id == self.account.backend.deleted_id:
                Gdk.threads_enter()

                self.post_box.remove(post)

                Gdk.threads_leave()

                self.delete_thread = dispatch(self._remove, self.delete_thread, post)

                break

        self.posts_lock.release()


    def _on_more_button_clicked(self):
        self.more_button.set_spinning(True)

        self.update_more_thread = dispatch(self._update_more, self.update_more_thread)


    def _on_clear_menuitem_activate(self, menuitem):
        self.clear_menuitem.set_sensitive(False)

        self.post_box.hide()

        self.clearings += 1

        self.clear_thread = dispatch(self._clear, self.clear_thread)


    def _on_child_button_press_event(self, widget, event):
        self.child_pressed = True

        return False


    def _on_child_focus_in_event(self, widget, event):
        if self.child_pressed:
            self.child_pressed = False
        else:
            toplevel = self.get_toplevel()

            if toplevel.focused:
                self.popup_child(self.more_button, 0)

        return False


    def __init__(self, key, account, parameter, refreshable, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Posts, self).__init__(key, account, parameter, refreshable, color_scheme, font_scheme, button_factory, status_bar, manager)

        self.avatar_factory = avatar_factory

        self.preview_factory = preview_factory

        self.main_panel = main_panel

        self.updated_old = False

        self.since_id = None

        self.max_id = None

        self.clearings = 0

        self.posts_lock = Lock()

        self.post_box = Gtk.VBox(False, 0)
        self.post_box.show()

        self.inner_box.pack_start(self.post_box, False, False, 0)

        if indicator_server:
            self.indicator_client = indicator_server.build(self)

            self.indicator_name = CONNECTING_TEXT

            self.indicator_client.set_name(self.indicator_name)

            self.indicator_count = 0

            self.indicator_client.set_count(self.indicator_count)
        else:
            self.indicator_client = None

        self.user_backend_cache = account.backend.user_backend_cache

        self.update_more_thread = None

        self.clear_thread = None

        self.delete_thread = None

        self.child_pressed = False


    def isolate(self):
        if self.delete_thread is not None:
            self.delete_thread.join()

        if self.clear_thread is not None:
            self.clear_thread.join()

        if self.update_more_thread is not None:
            self.update_more_thread.join()

        Gdk.threads_enter()

        posts = self.post_box.get_children()

        Gdk.threads_leave()

        for post in posts:
            post.isolate()

        if self.indicator_client:
            self.indicator_client.isolate()

        super(Posts, self).isolate()


    def synchronize(self):
        Gdk.threads_enter()

        posts = self.post_box.get_children()

        Gdk.threads_leave()

        timestamp = time()

        for post in posts:
            post.synchronize(timestamp)


    def hide_indicator(self):
        if self.indicator_client:
            self.indicator_client.hide()


    def reset_indicator(self):
        if self.indicator_client:
            self.indicator_count = 0

            self.indicator_client.reset()

            self.active = True


    def activate_indicator(self):
        Gdk.threads_enter()

        self.indicator_client.activate()

        Gdk.threads_leave()



import tweets, messages, results, realtime
