# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from gtk import gdk as Gdk


from polly.twitter.lead import user

from polly.gui.twitter.post.message import Message

from polly.gui.stream.posts import Posts



ALL_NAME = u'All messages'

RECEIVED_NAME = u'Received messages'

SENT_NAME = u'Sent messages'



class Messages(Posts):
    def _build(self, data):
        return Message(data[0], data[1], data[2], data[3], self, self.user_backend_cache, self.color_scheme, self.font_scheme, self.button_factory, self.avatar_factory, self.preview_factory, self.status_bar, self.main_panel)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Messages, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.account.backend.block_updates()

        self.delete_post_uid = self.account.backend.delete_message_notifier.connect(self._on_delete_post)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.delete_message_notifier.disconnect(self.delete_post_uid)

        self.account.backend.allow_updates()

        super(Messages, self).isolate()



class All(Messages):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_message_dataset_all(since_id, max_id, count)


    def _update_user_nick(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        self._set_indicator_name(u'@{}/Messages'.format(user_nick))


    def _on_user_nick_changed(self):
        Gdk.threads_enter()

        self._update_user_nick()

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(All, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = self.account.backend.user_backend

        self.label.set_text(ALL_NAME)
        self.label.set_tooltip_text(ALL_NAME)

        self.account.backend.block_updates()

        self.user_backend.block_updates()

        self._update_user_nick()

        self.send_post_uid = self.account.backend.send_message_notifier.connect(self._on_send_post)

        self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.account.backend.allow_updates()

        self.user_backend.allow_updates()


    def isolate(self):
        self.user_backend.block_updates()

        self.account.backend.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.account.backend.send_message_notifier.disconnect(self.send_post_uid)

        self.user_backend.allow_updates()

        self.account.backend.allow_updates()

        super(All, self).isolate()



class Received(Messages):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_message_dataset_received(since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(Received, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(RECEIVED_NAME)
        self.label.set_tooltip_text(RECEIVED_NAME)



class Sent(Messages):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_message_dataset_sent(since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(Sent, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(SENT_NAME)
        self.label.set_tooltip_text(SENT_NAME)

        self.account.backend.block_updates()

        self.send_post_uid = self.account.backend.send_message_notifier.connect(self._on_send_post)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.send_message_notifier.disconnect(self.send_post_uid)

        self.account.backend.allow_updates()

        super(Sent, self).isolate()
