# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from time import time


import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch

from polly.gui.stream.posts import Posts



class Consumer(Posts):
    def _consume(self, previous_thread, new_data):
        if previous_thread is not None:
            previous_thread.join()

        self.stream_lock.acquire()

        self.posts_lock.acquire()

        built_post = None

        if new_data[0] in self.deleted_ids:
            self.deleted_ids.remove(new_data[0])
        else:
            if self.since_id is None:
                self.identifier.set_spinning(True)

                _, destroyed_posts = self._update_new(new_data[0] - 1)

                for post in destroyed_posts:
                    post.isolate()

                    Gdk.threads_enter()

                    post.destroy()

                    Gdk.threads_leave()

                self.identifier.set_spinning(False)

                self.broken = False

                timestamp = time()

                built_post = self._build(new_data)

                built_post.synchronize(timestamp)

                self.since_id = built_post.real_id

                if self.max_id is None:
                    self.max_id = built_post.real_id

                Gdk.threads_enter()

                value = self.adjustment.get_value()

                if long(value) == 0:
                    built_post.revert_top_scrollings()
                else:
                    built_post.revert_two_scrollings()

                self.post_box.pack_end(built_post, False, False, 0)

                Gdk.threads_leave()
            else:
                if new_data[0] > self.since_id:
                    if self.broken:
                        self.identifier.set_spinning(True)

                        success, destroyed_posts = self._update_new(new_data[0] - 1)

                        for post in destroyed_posts:
                            post.isolate()

                            Gdk.threads_enter()

                            post.destroy()

                            Gdk.threads_leave()

                        self.identifier.set_spinning(False)

                        if success:
                            self.broken = False
                    else:
                        success = True

                    if success:
                        Gdk.threads_enter()

                        posts = self.post_box.get_children()

                        Gdk.threads_leave()

                        success = self._absent(posts, new_data)

                    if success:
                        timestamp = time()

                        built_post = self._build(new_data)

                        built_post.synchronize(timestamp)

                        self.since_id = built_post.real_id

                        if self.max_id is None:
                            self.max_id = built_post.real_id

                        Gdk.threads_enter()

                        value = self.adjustment.get_value()

                        if long(value) == 0:
                            built_post.revert_top_scrollings()
                        else:
                            built_post.revert_two_scrollings()

                        self.post_box.pack_end(built_post, False, False, 0)

                        Gdk.threads_leave()
                else:
                    Gdk.threads_enter()

                    posts = self.post_box.get_children()

                    Gdk.threads_leave()

                    if self._absent(posts, new_data):
                        for index in range(len(posts) - 1):
                            if new_data[0] < posts[index].real_id and new_data[0] > posts[index + 1].real_id:
                                timestamp = time()

                                built_post = self._build(new_data)

                                built_post.synchronize(timestamp)

                                index = len(posts) - index - 1

                                Gdk.threads_enter()

                                built_post.revert_top_scrollings()

                                self.post_box.pack_end(built_post, False, False, 0)
                                self.post_box.reorder_child(built_post, index)

                                Gdk.threads_leave()

                                break

        destroyed_posts = []

        if not self.updated_old:
            if built_post is not None:
                maximum = self.manager.get_maximum()

                Gdk.threads_enter()

                posts = self.post_box.get_children()

                height = long(self.adjustment.get_value() + self.adjustment.get_page_size())

                for post in reversed(posts[(maximum + 1):]):
                    allocation = post.get_allocation()

                    if allocation.y < height:
                        break

                    destroyed_posts.append(post)

                for post in destroyed_posts:
                    self.post_box.remove(post)

                Gdk.threads_leave()

                if destroyed_posts:
                    if len(destroyed_posts) == len(posts):
                        self.max_id = self.since_id
                    else:
                        index = posts.index(destroyed_posts[-1])

                        self.max_id = posts[index - 1].real_id

        if self.max_id is not None:
            self.deleted_ids = [deleted_id
                                for deleted_id in self.deleted_ids
                                if deleted_id > self.max_id]

        if self.indicator_client:
            if built_post is not None:
                count = built_post.log()

                Gdk.threads_enter()

                ancestor = self.get_ancestor(Gtk.Window)

                if ancestor is None:
                    self.active = True
                else:
                    self.active = ancestor.is_active()

                active = self.active

                Gdk.threads_leave()

                if not active:
                    Gdk.threads_enter()

                    self.indicator_count += 1

                    self.indicator_client.set_count(self.indicator_count)

                    Gdk.threads_leave()

                    policy = self.manager.get_policy()

                    if policy > 0:
                        if not built_post.user_own:
                            built_post.user_avatar.thread.join()

                            Gdk.threads_enter()

                            active = self.active

                            Gdk.threads_leave()

                            if not active:
                                if count == 1:
                                    built_post.notify(policy)

        self.posts_lock.release()

        for post in destroyed_posts:
            post.isolate()

            Gdk.threads_enter()

            post.destroy()

            Gdk.threads_leave()

        self.stream_lock.release()


    def _broken_task(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        self.stream_lock.acquire()

        self.broken = True

        self.stream_lock.release()


    def _on_delete_post(self):
        self.posts_lock.acquire()

        Gdk.threads_enter()

        posts = self.post_box.get_children()

        Gdk.threads_leave()

        absent = True

        for post in posts:
            if post.real_id == self.account.backend.deleted_id:
                Gdk.threads_enter()

                self.post_box.remove(post)

                Gdk.threads_leave()

                self.delete_thread = dispatch(self._remove, self.delete_thread, post)

                absent = False

                break

        if absent:
            if self.max_id is None:
                self.deleted_ids.append(self.account.backend.deleted_id)
            else:
                if self.max_id < self.account.backend.deleted_id:
                    self.deleted_ids.append(self.account.backend.deleted_id)

        self.posts_lock.release()


    def _on_new_post(self):
        self.realtime_thread = dispatch(self._consume, self.realtime_thread, self.account.producer.new_data)


    def _on_broken(self):
        self.realtime_thread = dispatch(self._broken_task, self.realtime_thread)


    def _on_backup(self):
        self.refresh(False)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Consumer, self).__init__(key, account, parameter, False, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.broken = True

        self.deleted_ids = []

        self.account.producer.block_updates()

        self.broken_uid = self.account.producer.broken_notifier.connect(self._on_broken)

        self.backup_uid = self.account.producer.backup_notifier.connect(self._on_backup)

        self.account.producer.allow_updates()

        self.realtime_thread = None


    def isolate(self):
        if self.realtime_thread is not None:
            self.realtime_thread.join()

        self.account.producer.block_updates()

        self.account.producer.backup_notifier.disconnect(self.backup_uid)

        self.account.producer.broken_notifier.disconnect(self.broken_uid)

        self.account.producer.allow_updates()

        super(Consumer, self).isolate()



import tweets, results
