# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from gtk import gdk as Gdk


from polly.twitter import ID_KEY

from polly.twitter.lead import user

from polly.twitter.account import MENTION_ONLY_KEY

from polly.gui.twitter.post.tweet import Tweet

from polly.gui.stream.posts.tweets import HOME_MENTIONS_NAME, HOME_NAME, MENTIONS_NAME, SENT_BY_ME_NAME, RETWEETED_BY_ME_NAME, RETWEETED_TO_ME_NAME

from polly.gui.stream.posts.realtime import Consumer



class Tweets(Consumer):
    def _build(self, data):
        return Tweet(data[0], data[1], data[2], data[3], self, self.user_backend_cache, self.tweet_backend_cache, self.color_scheme, self.font_scheme, self.button_factory, self.avatar_factory, self.preview_factory, self.status_bar, self.main_panel)


    def _absent(self, posts, new_data):
        for post in posts:
            if new_data[3][ID_KEY] == post.tweet_id:
                return False

        return True


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Tweets, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.tweet_backend_cache = account.backend.tweet_backend_cache

        self.account.backend.block_updates()

        self.delete_post_uid = self.account.backend.delete_tweet_notifier.connect(self._on_delete_post)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.delete_tweet_notifier.disconnect(self.delete_post_uid)

        self.account.backend.allow_updates()

        super(Tweets, self).isolate()



class HomeMentions(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        dataset = self.account.backend.get_tweet_dataset_home_mentions(since_id, max_id, count)

        dataset_home = [data
                        for data in dataset
                        if not data[3][MENTION_ONLY_KEY]]

        if dataset_home:
            self.account.database.insert(dataset_home)

        return dataset


    def _update_user_nick(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        self._set_indicator_name(u'@{}/{}'.format(user_nick, HOME_MENTIONS_NAME))


    def _on_user_nick_changed(self):
        Gdk.threads_enter()

        self._update_user_nick()

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(HomeMentions, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = self.account.backend.user_backend

        self.label.set_text(HOME_MENTIONS_NAME)
        self.label.set_tooltip_text(HOME_MENTIONS_NAME)

        self.account.producer.block_updates()

        self.user_backend.block_updates()

        self._update_user_nick()

        self.new_post_uid = self.account.producer.new_tweet_home_mentions_notifier.connect(self._on_new_post)

        self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.account.producer.allow_updates()

        self.user_backend.allow_updates()


    def isolate(self):
        self.user_backend.block_updates()

        self.account.producer.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.account.producer.new_tweet_home_mentions_notifier.disconnect(self.new_post_uid)

        self.user_backend.allow_updates()

        self.account.producer.allow_updates()

        super(HomeMentions, self).isolate()



class Home(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        dataset = self.account.backend.get_tweet_dataset_home(since_id, max_id, count)

        if dataset:
            self.account.database.insert(dataset)

        return dataset


    def _update_user_nick(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        self._set_indicator_name(u'@{}/{}'.format(user_nick, HOME_NAME))


    def _on_user_nick_changed(self):
        Gdk.threads_enter()

        self._update_user_nick()

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Home, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = self.account.backend.user_backend

        self.label.set_text(HOME_NAME)
        self.label.set_tooltip_text(HOME_NAME)

        self.account.producer.block_updates()

        self.user_backend.block_updates()

        self._update_user_nick()

        self.new_post_uid = self.account.producer.new_tweet_home_notifier.connect(self._on_new_post)

        self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.account.producer.allow_updates()

        self.user_backend.allow_updates()


    def isolate(self):
        self.user_backend.block_updates()

        self.account.producer.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.account.producer.new_tweet_home_notifier.disconnect(self.new_post_uid)

        self.user_backend.allow_updates()

        self.account.producer.allow_updates()

        super(Home, self).isolate()



class Mentions(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_mentions(since_id, max_id, count)


    def _update_user_nick(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        self._set_indicator_name(u'@{}/{}'.format(user_nick, MENTIONS_NAME))


    def _on_user_nick_changed(self):
        Gdk.threads_enter()

        self._update_user_nick()

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Mentions, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = self.account.backend.user_backend

        self.label.set_text(MENTIONS_NAME)
        self.label.set_tooltip_text(MENTIONS_NAME)

        self.account.producer.block_updates()

        self.user_backend.block_updates()

        self._update_user_nick()

        self.new_post_uid = self.account.producer.new_tweet_mentions_notifier.connect(self._on_new_post)

        self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.account.producer.allow_updates()

        self.user_backend.allow_updates()


    def isolate(self):
        self.user_backend.block_updates()

        self.account.producer.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.account.producer.new_tweet_mentions_notifier.disconnect(self.new_post_uid)

        self.user_backend.allow_updates()

        self.account.producer.allow_updates()

        super(Mentions, self).isolate()



class SentByMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        dataset = self.account.backend.get_tweet_dataset_sent(self.account.backend.user_backend.id, since_id, max_id, count, None)
 
        if dataset:
            self.account.database.insert(dataset)

        return dataset


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(SentByMe, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(SENT_BY_ME_NAME)
        self.label.set_tooltip_text(SENT_BY_ME_NAME)

        self.account.producer.block_updates()

        self.new_post_uid = self.account.producer.new_tweet_sent_by_me_notifier.connect(self._on_new_post)

        self.account.producer.allow_updates()


    def isolate(self):
        self.account.producer.block_updates()

        self.account.producer.new_tweet_sent_by_me_notifier.disconnect(self.new_post_uid)

        self.account.producer.allow_updates()

        super(SentByMe, self).isolate()



class RetweetedByMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_retweeted_by_me(since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(RetweetedByMe, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(RETWEETED_BY_ME_NAME)
        self.label.set_tooltip_text(RETWEETED_BY_ME_NAME)

        self.account.producer.block_updates()

        self.new_post_uid = self.account.producer.new_tweet_retweeted_by_me_notifier.connect(self._on_new_post)

        self.account.producer.allow_updates()


    def isolate(self):
        self.account.producer.block_updates()

        self.account.producer.new_tweet_retweeted_by_me_notifier.disconnect(self.new_post_uid)

        self.account.producer.allow_updates()

        super(RetweetedByMe, self).isolate()



class RetweetedToMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_retweeted_to_me(since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(RetweetedToMe, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(RETWEETED_TO_ME_NAME)
        self.label.set_tooltip_text(RETWEETED_TO_ME_NAME)

        self.account.producer.block_updates()

        self.new_post_uid = self.account.producer.new_tweet_retweeted_to_me_notifier.connect(self._on_new_post)

        self.account.producer.allow_updates()


    def isolate(self):
        self.account.producer.block_updates()

        self.account.producer.new_tweet_retweeted_to_me_notifier.disconnect(self.new_post_uid)

        self.account.producer.allow_updates()

        super(RetweetedToMe, self).isolate()
