# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from time import time


import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch

from polly.twitter.lead import user, group

from polly.twitter.post import tweet

from polly.twitter.account import MENTION_ONLY_KEY, PROTECTED_MESSAGE, Error, UnauthorizedError, ForbiddenError

from polly.gui.twitter.lead.user import User

from polly.gui.twitter.lead.group import Group

from polly.gui.twitter.post.tweet import Tweet

from polly.gui.stream.posts import Posts



HOME_MENTIONS_NAME = u'Home+Mentions'

HOME_NAME = u'Home'

MENTIONS_NAME = u'Mentions'

SENT_BY_ME_NAME = u'Sent by me'

FAVORITED_BY_ME_NAME = u'Favorited by me'

RETWEETED_BY_ME_NAME = u'Retweeted by me'

RETWEETED_TO_ME_NAME = u'Retweeted to me'

RETWEETED_FROM_ME_NAME = u'Retweeted from me'

CONTEXT_NAME = u'Context'



class Tweets(Posts):
    def _build(self, data):
        return Tweet(data[0], data[1], data[2], data[3], self, self.user_backend_cache, self.tweet_backend_cache, self.color_scheme, self.font_scheme, self.button_factory, self.avatar_factory, self.preview_factory, self.status_bar, self.main_panel)


    def __init__(self, key, account, parameter, refreshable, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Tweets, self).__init__(key, account, parameter, refreshable, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.tweet_backend_cache = account.backend.tweet_backend_cache

        self.account.backend.block_updates()

        self.delete_post_uid = self.account.backend.delete_tweet_notifier.connect(self._on_delete_post)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.delete_tweet_notifier.disconnect(self.delete_post_uid)

        self.account.backend.allow_updates()

        super(Tweets, self).isolate()



class HomeMentions(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        dataset = self.account.backend.get_tweet_dataset_home_mentions(since_id, max_id, count)

        dataset_home = [data
                        for data in dataset
                        if not data[3][MENTION_ONLY_KEY]]

        if dataset_home:
            self.account.database.insert(dataset_home)

        return dataset


    def _update_user_nick(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        self._set_indicator_name(u'@{}/{}'.format(user_nick, HOME_MENTIONS_NAME))


    def _on_user_nick_changed(self):
        Gdk.threads_enter()

        self._update_user_nick()

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(HomeMentions, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = self.account.backend.user_backend

        self.label.set_text(HOME_MENTIONS_NAME)
        self.label.set_tooltip_text(HOME_MENTIONS_NAME)

        self.account.backend.block_updates()

        self.user_backend.block_updates()

        self._update_user_nick()

        self.send_post_uid = self.account.backend.send_tweet_notifier.connect(self._on_send_post)

        self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.account.backend.allow_updates()

        self.user_backend.allow_updates()


    def isolate(self):
        self.user_backend.block_updates()

        self.account.backend.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.account.backend.send_tweet_notifier.disconnect(self.send_post_uid)

        self.user_backend.allow_updates()

        self.account.backend.allow_updates()

        super(HomeMentions, self).isolate()



class Home(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        dataset = self.account.backend.get_tweet_dataset_home(since_id, max_id, count)

        if dataset:
            self.account.database.insert(dataset)

        return dataset


    def _update_user_nick(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        self._set_indicator_name(u'@{}/{}'.format(user_nick, HOME_NAME))


    def _on_user_nick_changed(self):
        Gdk.threads_enter()

        self._update_user_nick()

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Home, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = self.account.backend.user_backend

        self.label.set_text(HOME_NAME)
        self.label.set_tooltip_text(HOME_NAME)

        self.account.backend.block_updates()

        self.user_backend.block_updates()

        self._update_user_nick()

        self.send_post_uid = self.account.backend.send_tweet_notifier.connect(self._on_send_post)

        self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.account.backend.allow_updates()

        self.user_backend.allow_updates()


    def isolate(self):
        self.user_backend.block_updates()

        self.account.backend.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.account.backend.send_tweet_notifier.disconnect(self.send_post_uid)

        self.user_backend.allow_updates()

        self.account.backend.allow_updates()

        super(Home, self).isolate()



class Mentions(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_mentions(since_id, max_id, count)


    def _update_user_nick(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        self._set_indicator_name(u'@{}/{}'.format(user_nick, MENTIONS_NAME))


    def _on_user_nick_changed(self):
        Gdk.threads_enter()

        self._update_user_nick()

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(Mentions, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = self.account.backend.user_backend

        self.label.set_text(MENTIONS_NAME)
        self.label.set_tooltip_text(MENTIONS_NAME)

        self.user_backend.block_updates()

        self._update_user_nick()

        self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

        self.user_backend.allow_updates()


    def isolate(self):
        self.user_backend.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.user_backend.allow_updates()

        super(Mentions, self).isolate()



class SentByMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        dataset = self.account.backend.get_tweet_dataset_sent(self.account.backend.user_backend.id, since_id, max_id, count, None)
 
        if dataset:
            self.account.database.insert(dataset)

        return dataset


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(SentByMe, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(SENT_BY_ME_NAME)
        self.label.set_tooltip_text(SENT_BY_ME_NAME)

        self.account.backend.block_updates()

        self.send_post_uid = self.account.backend.send_tweet_notifier.connect(self._on_send_post)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.send_tweet_notifier.disconnect(self.send_post_uid)

        self.account.backend.allow_updates()

        super(SentByMe, self).isolate()



class FavoritedByMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_favorited(self.account.backend.user_backend.id, since_id, max_id, count)
 

    def _on_send_favorite(self):
        self.posts_lock.acquire()

        Gdk.threads_enter()

        posts = self.post_box.get_children()

        Gdk.threads_leave()

        if posts:
            favorited_id = self.account.backend.favorited_data[0]

            if favorited_id > posts[0].real_id:
                self.refresh(False)
            elif favorited_id > posts[-1].real_id:
                for index in range(len(posts) - 1):
                    if favorited_id < posts[index].real_id and favorited_id > posts[index + 1].real_id:
                        timestamp = time()

                        post = self._build(self.account.backend.favorited_data)

                        post.synchronize(timestamp)

                        index = len(posts) - index - 1

                        Gdk.threads_enter()

                        post.revert_top_scrollings()

                        self.post_box.pack_end(post, False, False, 0)
                        self.post_box.reorder_child(post, index)

                        Gdk.threads_leave()

                        break
        else:
            self.refresh(False)

        self.posts_lock.release()


    def _on_delete_favorite(self):
        self.posts_lock.acquire()

        Gdk.threads_enter()

        posts = self.post_box.get_children()

        Gdk.threads_leave()

        length = len(posts)

        for index in range(length):
            post = posts[index]

            if post.real_id == self.account.backend.deleted_id:
                if length == 1:
                    self.since_id = None

                    self.max_id = None
                else:
                    if index == 0:
                        self.since_id = posts[1].real_id
                    elif index == length - 1:
                        self.max_id = posts[-2].real_id

                Gdk.threads_enter()

                self.post_box.remove(post)

                Gdk.threads_leave()

                self.delete_thread = dispatch(self._remove, self.delete_thread, post)

                break

        self.posts_lock.release()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(FavoritedByMe, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(FAVORITED_BY_ME_NAME)
        self.label.set_tooltip_text(FAVORITED_BY_ME_NAME)

        self.account.backend.block_updates()

        self.send_favorite_uid = self.account.backend.send_tweet_favorite_notifier.connect(self._on_send_favorite)

        self.delete_favorite_uid = self.account.backend.delete_tweet_favorite_notifier.connect(self._on_delete_favorite)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.delete_tweet_favorite_notifier.disconnect(self.delete_favorite_uid)

        self.account.backend.send_tweet_favorite_notifier.disconnect(self.send_favorite_uid)

        self.account.backend.allow_updates()

        super(FavoritedByMe, self).isolate()



class RetweetedByMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_retweeted_by_me(since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(RetweetedByMe, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(RETWEETED_BY_ME_NAME)
        self.label.set_tooltip_text(RETWEETED_BY_ME_NAME)

        self.account.backend.block_updates()

        self.send_post_uid = self.account.backend.send_tweet_retweet_notifier.connect(self._on_send_post)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.send_tweet_retweet_notifier.disconnect(self.send_post_uid)

        self.account.backend.allow_updates()

        super(RetweetedByMe, self).isolate()



class RetweetedToMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_retweeted_to_me(since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(RetweetedToMe, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(RETWEETED_TO_ME_NAME)
        self.label.set_tooltip_text(RETWEETED_TO_ME_NAME)



class RetweetedFromMe(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_retweeted_from_me(since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(RetweetedFromMe, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.label.set_text(RETWEETED_FROM_ME_NAME)
        self.label.set_tooltip_text(RETWEETED_FROM_ME_NAME)



class Context(Tweets):
    def _clear(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        self.posts_lock.acquire()

        Gdk.threads_enter()

        posts = self.post_box.get_children()

        for post in posts:
            self.post_box.remove(post)

        Gdk.threads_leave()

        self.next_id = self.parameter

        self.posts_lock.release()

        Gdk.threads_enter()

        self.post_box.show()

        self.clear_menuitem.set_sensitive(True)

        Gdk.threads_leave()

        for post in posts:
            post.isolate()

            Gdk.threads_enter()

            post.destroy()

            Gdk.threads_leave()


    def _update(self):
        self.posts_lock.acquire()

        self._update_old()

        self.posts_lock.release()


    def _update_old(self):
        for _ in range(4):
            if self.next_id is None:
                break

            try:
                data = self.account.backend.get_tweet_data(self.next_id)
            except ForbiddenError:
                self.next_id = None

                self.status_bar.popup(PROTECTED_MESSAGE)
            except Error as error:
                self.next_id = None

                self.status_bar.popup(error.message)
            else:
                self.next_id = data[3][tweet.REPLIED_TWEET_ID_KEY]

                timestamp = time()

                post = self._build(data)

                post.synchronize(timestamp)

                Gdk.threads_enter()

                post.revert_top_scrollings()

                self.post_box.pack_start(post, False, False, 0)

                Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        # NOTE: The presence of long will not be necessary
        # after the planned port to Python 3 is completed.

        if not isinstance(parameter, (int, long)):
            raise TypeError

        super(Context, self).__init__(key, account, parameter, False, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.next_id = self.parameter

        self.label.set_text(CONTEXT_NAME)
        self.label.set_tooltip_text(CONTEXT_NAME)



class UserTweets(Tweets):
    def _get_dataset(self, since_id, max_id, count):
        try:
            dataset = self._get_protected_dataset(since_id, max_id, count)
        except UnauthorizedError:
            self.status_bar.popup(PROTECTED_MESSAGE)

            dataset = []

        return dataset


    def _disconnect_from_user_nick(self):
        self.user_backend.block_updates()

        self.user_backend.nick_notifier.disconnect(self.user_nick_uid)

        self.user_backend.allow_updates()


    def _initialize(self, user_data):
        self.user_backend = self.user_backend_cache.acquire(user_data)


    def _finalize(self):
        if self.user_backend is not None:
            self._disconnect_from_user_nick()

            self.user_backend.release()


    def _update(self):
        if self.user_backend is None:
            try:
                user_data = self.account.backend.get_user_data(self.user_id)
            except Error as error:
                self.status_bar.popup(error.message)
            else:
                self._initialize(user_data)

                self.user_backend.block_updates()

                self._on_user_nick_changed()

                self.user_nick_uid = self.user_backend.nick_notifier.connect(self._on_user_nick_changed)

                self.user_backend.allow_updates()

        if self.user_backend is not None:
            super(UserTweets, self)._update()


    def _on_user_nick_changed(self):
        user_nick = self.user_backend.data[user.NICK_KEY]

        markup = self.string.format(user_nick)

        Gdk.threads_enter()

        self.label.set_markup(markup)
        self.label.set_tooltip_markup(markup)

        Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        super(UserTweets, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.user_backend = None



class HomeOfUser(UserTweets):
    def _build_menu(self):
        super(HomeOfUser, self)._build_menu()

        self.show_menuitem = Gtk.CheckMenuItem(u'Show user details', False)
        self.show_menuitem.set_active(self.parameter[1])
        self.show_menuitem.connect(u'activate', self._on_show_menuitem_activate)
        self.show_menuitem.show()

        self.menu.append(self.show_menuitem)


    def _get_protected_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_sent(self.user_id, since_id, max_id, count, 1)


    def _initialize(self, user_data):
        self.user = User(user_data, self, self.user_backend_cache, self.relationship_backend_cache, self.color_scheme, self.font_scheme, self.emblem_factory, self.button_factory, self.avatar_factory, self.status_bar, self.main_panel)

        Gdk.threads_enter()

        if self.parameter[1]:
            self.user.click_relationship_button()
        else:
            self.user.hide()

        self.inner_box.pack_start(self.user, False, False, 0)
        self.inner_box.reorder_child(self.user, 0)

        self.user_backend = self.user.user_backend

        Gdk.threads_leave()


    def _finalize(self):
        if self.user_backend is not None:
            self._disconnect_from_user_nick()

            self.user.isolate()
            self.user = None


    def _on_show_menuitem_activate(self, menuitem):
        active = self.show_menuitem.get_active()

        if self.parameter[1] != active:
            self.parameter[1] = active

            if self.user_backend is not None:
                if self.parameter[1]:
                    self.user.show()
                else:
                    self.user.hide()

            self.manager.save()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, emblem_factory, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        if not isinstance(parameter, list):
            raise TypeError

        if len(parameter) != 2:
            raise ValueError

        # NOTE: The presence of long will not be necessary
        # after the planned port to Python 3 is completed.

        if not isinstance(parameter[0], (int, long)):
            raise TypeError

        if not isinstance(parameter[1], bool):
            raise TypeError

        super(HomeOfUser, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager)

        self.emblem_factory = emblem_factory

        self.user_id = parameter[0]

        self.string = u'@{}'

        self.relationship_backend_cache = account.backend.relationship_backend_cache



class SentByUser(UserTweets):
    def _get_protected_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_sent(self.user_id, since_id, max_id, count, None)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        # NOTE: The presence of long will not be necessary
        # after the planned port to Python 3 is completed.

        if not isinstance(parameter, (int, long)):
            raise TypeError

        super(SentByUser, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager)

        self.user_id = parameter

        self.string = u'Sent by @{}'



class FavoritedByUser(UserTweets):
    def _get_protected_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_favorited(self.user_id, since_id, max_id, count)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        # NOTE: The presence of long will not be necessary
        # after the planned port to Python 3 is completed.

        if not isinstance(parameter, (int, long)):
            raise TypeError

        super(FavoritedByUser, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager)

        self.user_id = parameter

        self.string = u'Favorited by @{}'



class GroupTweets(Tweets):
    def _set_markup(self, markup):
        self.label.set_markup(markup)
        self.label.set_tooltip_markup(markup)


    def _disconnect_from_group_path(self):
        self.group_backend.block_updates()

        self.group_backend.path_notifier.disconnect(self.group_path_uid)

        self.group_backend.allow_updates()


    def _initialize(self, data):
        self.user_backend = self.user_backend_cache.acquire(data[0])

        self.group_backend = self.group_backend_cache.acquire(data[1])


    def _finalize(self):
        if self.group_backend is not None:
            self._disconnect_from_group_path()

            self.group_backend.release()

            self.user_backend.release()


    def _update(self):
        if self.group_backend is None:
            try:
                data = self.account.backend.get_group_data(self.group_id)
            except Error as error:
                self.status_bar.popup(error.message)
            else:
                self._initialize(data)

                self.group_backend.block_updates()

                self._on_group_path_changed()

                self.group_path_uid = self.group_backend.path_notifier.connect(self._on_group_path_changed)

                self.group_backend.allow_updates()

        if self.group_backend is not None:
            super(GroupTweets, self)._update()


    def _on_group_path_changed(self):
        group_path = self.group_backend.data[group.PATH_KEY]

        markup = self.string.format(group_path)

        Gdk.threads_enter()

        self._set_markup(markup)

        Gdk.threads_leave()


    def _on_delete_lead(self):
        if self.group_id == self.account.backend.deleted_id:
            Gdk.threads_enter()

            self.manager.close(self)

            Gdk.threads_leave()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        super(GroupTweets, self).__init__(key, account, parameter, True, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.user_backend = None

        self.group_backend = None

        self.group_backend_cache = account.backend.group_backend_cache

        self.account.backend.block_updates()

        self.delete_lead_uid = self.account.backend.delete_group_notifier.connect(self._on_delete_lead)

        self.account.backend.allow_updates()


    def isolate(self):
        self.account.backend.block_updates()

        self.account.backend.delete_group_notifier.disconnect(self.delete_lead_uid)

        self.account.backend.allow_updates()

        super(GroupTweets, self).isolate()



class HomeOfGroup(GroupTweets):
    def _build_menu(self):
        super(HomeOfGroup, self)._build_menu()

        self.show_menuitem = Gtk.CheckMenuItem(u'Show list details', False)
        self.show_menuitem.set_active(self.parameter[2])
        self.show_menuitem.connect(u'activate', self._on_show_menuitem_activate)
        self.show_menuitem.show()

        self.menu.append(self.show_menuitem)


    def _get_dataset(self, since_id, max_id, count):
        dataset = self.account.backend.get_tweet_dataset_group(self.group_id, since_id, max_id, count, 1)

        if self.own and dataset:
            self.account.database.insert(dataset)

        return dataset


    def _set_markup(self, markup):
        super(HomeOfGroup, self)._set_markup(markup)

        if self.own:
            self._set_indicator_name(markup)


    def _initialize(self, data):
        self.group = Group(data[0], data[1], self, self.user_backend_cache, self.group_backend_cache, self.color_scheme, self.font_scheme, self.emblem_factory, self.button_factory, self.avatar_factory, self.status_bar, self.main_panel)

        Gdk.threads_enter()

        if not self.parameter[2]:
            self.group.hide()

        self.inner_box.pack_start(self.group, False, False, 0)
        self.inner_box.reorder_child(self.group, 0)

        self.user_backend = self.group.user_backend

        self.group_backend = self.group.group_backend

        Gdk.threads_leave()


    def _finalize(self):
        if self.group_backend is not None:
            self._disconnect_from_group_path()

            self.group.isolate()
            self.group = None


    def _on_show_menuitem_activate(self, menuitem):
        active = self.show_menuitem.get_active()

        if self.parameter[2] != active:
            self.parameter[2] = active

            if self.group_backend is not None:
                if self.parameter[2]:
                    self.group.show()
                else:
                    self.group.hide()

            self.manager.save()


    def __init__(self, key, account, parameter, color_scheme, font_scheme, emblem_factory, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager):
        if not isinstance(parameter, list):
            raise TypeError

        if len(parameter) != 3:
            raise ValueError

        # NOTE: The presence of long will not be necessary
        # after the planned port to Python 3 is completed.

        if not isinstance(parameter[0], (int, long)):
            raise TypeError

        if not isinstance(parameter[1], (int, long)):
            raise TypeError

        if not isinstance(parameter[2], bool):
            raise TypeError

        self.own = (parameter[0] == account.backend.user_backend.id)

        if not self.own:
            indicator_server = None

        super(HomeOfGroup, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, indicator_server, manager)

        self.emblem_factory = emblem_factory

        self.group_id = self.parameter[1]

        self.string = u'{}'



class TweetsOfGroup(GroupTweets):
    def _get_dataset(self, since_id, max_id, count):
        return self.account.backend.get_tweet_dataset_group(self.group_id, since_id, max_id, count, None)


    def __init__(self, key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, manager):
        # NOTE: The presence of long will not be necessary
        # after the planned port to Python 3 is completed.

        if not isinstance(parameter, (int, long)):
            raise TypeError

        super(TweetsOfGroup, self).__init__(key, account, parameter, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel, None, manager)

        self.group_id = self.parameter

        self.string = u'Tweets of {}'
