# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

from gtk import gdk as Gdk


from polly import dispatch, Notifier, Updatable

from polly.gui import PROPERTY_SIGNAL



PROPERTY_THEME_NAME = u'gtk-theme-name'



class Scheme(Updatable):
    def _update_name(self):
        Gdk.threads_enter()

        name = self.gtk_settings.get_property(PROPERTY_THEME_NAME)

        Gdk.threads_leave()

        if self.name == name:
            return False

        self.name = name

        return True


    def _update_style(self):
        Gdk.threads_enter()

        self.style = self.menubar.get_style()

        Gdk.threads_leave()


    def _update(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        self._acquire_update()

        if self._update_name():
            self._update_style()

            self.notifier.notify()

        self._release_update()


    def _on_changed(self, object, pspec):
        self.thread = dispatch(self._update, self.thread)


    def __init__(self, gtk_settings):
        super(Scheme, self).__init__()

        self.gtk_settings = gtk_settings

        self.name = None

        self.menubar = Gtk.MenuBar()
        self.menubar.show()

        self.window = Gtk.Window()
        self.window.add(self.menubar)
        self.window.realize()

        self.notifier = Notifier()

        self._update_name()

        self._update_style()

        self.thread = None

        self.uid = self.gtk_settings.connect(PROPERTY_SIGNAL + PROPERTY_THEME_NAME, self._on_changed)


    def isolate(self):
        self.gtk_settings.disconnect(self.uid)

        if self.thread is not None:
            self.thread.join()

        self.window.destroy()
