# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

from gtk import gdk as Gdk


from polly.gui.twitter import Item



FOLLOWING_FOOTER = u'following'

NOT_FOLLOWING_FOOTER = u'not following'


FOLLOW_BOX_TOOLTIP = u'Follow'

FOLLOW_SPINNER_TOOLTIP = u'Following\u2026'

UNFOLLOW_BOX_TOOLTIP = u'Unfollow'

UNFOLLOW_SPINNER_TOOLTIP = u'Unfollowing\u2026'



class Lead(Item):
    def _build_menu(self):
        super(Lead, self)._build_menu()

        if self._reopenable():
            accounts = self.main_panel.get_other_accounts(self.stream.account)

            if accounts:
                self.account_specs = []

                for account in accounts:
                    user_nick = account.get_user_nick()

                    reopen_menuitem = Gtk.MenuItem(u'Reopen as {}'.format(user_nick), False)
                    reopen_menuitem.connect(u'activate', self._on_reopen_menuitem_activate)
                    reopen_menuitem.show()

                    self.menu.append(reopen_menuitem)

                    self.account_specs.append((reopen_menuitem, account))


    def _on_info_button_clicked(self):
        super(Lead, self)._on_info_button_clicked()

        if self.info_button.toggled:
            self.wraplabel.hide()

            self.info_box.show()
        else:
            self.info_box.hide()

            if self.wraplabel.get_text():
                self.wraplabel.show()


    def _on_reopen_menuitem_activate(self, menuitem):
        for account_spec in self.account_specs:
            if account_spec[0] == menuitem:
                self._reopen(account_spec[1])

                break

        del self.account_specs[:]


    def __init__(self, user_data, stream, user_backend_cache, color_scheme, font_scheme, emblem_factory, button_factory, avatar_factory, status_bar, main_panel):
        super(Lead, self).__init__(2, 2, user_data, stream, user_backend_cache, color_scheme, font_scheme, button_factory, avatar_factory, status_bar, main_panel)

        self.emblem_factory = emblem_factory

        self.button_factory = button_factory

        Gdk.threads_enter()

        self.footer_box.pack_end(self.footer_link, True, True, 0)

        self.footer_link.box.connect(u'button-press-event', self._on_child_button_press_event)
        self.footer_link.box.connect(u'focus-in-event', self._on_child_focus_in_event)

        Gdk.threads_leave()

        self.account_specs = []


    def isolate(self):
        del self.account_specs[:]

        super(Lead, self).isolate()



import user, group
