# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from urllib import quote


import gtk as Gtk

from gtk import gdk as Gdk


from polly import ENCODING, escape, unescape, morescape, dispatch

from polly.gui import run_confirmation_dialog

from polly.twitter.lead import DESCRIPTION_KEY

from polly.twitter.lead.group import NICK_KEY, NAME_KEY, MODE_KEY, MEMBER_COUNT_KEY, SUBSCRIBER_COUNT_KEY, SUBSCRIBED_KEY

from polly.twitter.account import Error

from polly.gui import show_url

from polly.gui.twitter import DELETE_BOX_TOOLTIP, DELETE_SPINNER_TOOLTIP, Link

from polly.gui.twitter.lead import FOLLOWING_FOOTER, NOT_FOLLOWING_FOOTER, FOLLOW_BOX_TOOLTIP, FOLLOW_SPINNER_TOOLTIP, UNFOLLOW_BOX_TOOLTIP, UNFOLLOW_SPINNER_TOOLTIP, Lead


from dialog import EditGroupDialog



class Group(Lead):
    def _reopenable(self):
        return (self.private_emblem is None)


    def _reopen(self, account):
        self.stream.manager.new_tweets_home_of_group(account, [self.user_backend.id, self.group_backend.id, True], self.stream)


    def _delete_task(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        try:
            data = self.stream.account.backend.delete_group(self.group_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)

            Gdk.threads_enter()

            self.delete_button.set_spinning(False)

            Gdk.threads_leave()
        else:
            if data is not None:
                self.user_backend.set_data(data[0])


    def _follow_task(self, previous_thread, subscribed):
        if previous_thread is not None:
            previous_thread.join()

        try:
            if subscribed:
                data = self.stream.account.backend.delete_group_subscription(self.group_backend.id)
            else:
                data = self.stream.account.backend.send_group_subscription(self.group_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)
        else:
            self.user_backend.set_data(data[0])

            if subscribed:
                self.group_backend.set_value(SUBSCRIBED_KEY, False)
            else:
                self.group_backend.set_value(SUBSCRIBED_KEY, True)

            self.group_backend.set_data(data[1])

        Gdk.threads_enter()

        if subscribed:
            if self.unfollow_button is not None:
                self.unfollow_button.set_spinning(False)
        else:
            if self.follow_button is not None:
                self.follow_button.set_spinning(False)

        Gdk.threads_leave()


    def _set_normal_mg(self, color):
        super(Group, self)._set_normal_mg(color)

        self.member_count_link.set_color(color)

        self.subscriber_count_link.set_color(color)

        self.tweets_link.set_color(color)


    def _on_header_clicked(self):
        self.stream.manager.new_tweets_home_of_group(self.stream.account, [self.user_backend.id, self.group_backend.id, True], self.stream)


    def _on_info_button_clicked(self):
        super(Group, self)._on_info_button_clicked()

        if self.info_button.toggled:
            self.header_link.set_markup(self.group_name)
        else:
            self.header_link.set_markup(self.group_nick)


    def _on_member_count_link_clicked(self):
        self.stream.manager.new_users_members_of_group(self.stream.account, self.group_backend.id, self.stream)


    def _on_subscriber_count_link_clicked(self):
        self.stream.manager.new_users_subscribers_of_group(self.stream.account, self.group_backend.id, self.stream)


    def _on_tweets_link_clicked(self):
        self.stream.manager.new_tweets_tweets_of_group(self.stream.account, self.group_backend.id, self.stream)


    def _on_footer_clicked(self):
        show_url(u'http://twitter.com/{}/{}'.format(self.user_nick, self.group_nick))


    def _on_edit_button_clicked(self):
        dialog = EditGroupDialog(self.get_toplevel(), self.stream.account, self)

        self.edit_thread = dialog.run_and_destroy()


    def _on_delete_button_clicked(self):
        if run_confirmation_dialog(self.get_toplevel(), u'Delete list',
                                   u'Are you sure you want to delete this list?',
                                   u'You will not be able to restore it.'):
            self.delete_button.set_spinning(True)

            self.delete_thread = dispatch(self._delete_task, self.delete_thread)


    def _on_follow_button_clicked(self):
        self.follow_button.set_spinning(True)

        self.follow_thread = dispatch(self._follow_task, self.follow_thread, False)


    def _on_unfollow_button_clicked(self):
        self.unfollow_button.set_spinning(True)

        self.follow_thread = dispatch(self._follow_task, self.follow_thread, True)


    def _on_group_nick_changed(self):
        Gdk.threads_enter()

        self.group_nick = self.group_backend.data[NICK_KEY]

        if not self.info_button.toggled:
            self.header_link.set_markup(self.group_nick)

        Gdk.threads_leave()


    def _on_group_name_changed(self):
        group_name = self.group_backend.data[NAME_KEY]

        if isinstance(group_name, unicode):
            group_name = group_name.strip()
        else:
            group_name = u''

        Gdk.threads_enter()

        if group_name:
            self.group_name = morescape(escape(group_name))
        else:
            self.group_name = self.group_nick

        if self.info_button.toggled:
            self.header_link.set_markup(self.group_name)

        Gdk.threads_leave()


    def _on_group_mode_changed(self):
        if self.group_backend.data[MODE_KEY] == u'private':
            if self.private_emblem is None:
                private_emblem = self.emblem_factory.build(u'protected')

                Gdk.threads_enter()

                self.private_emblem = private_emblem
                self.private_emblem.set_tooltip_text(u'Private')

                self.indicator_box.pack_end(self.private_emblem, True, True, 0)

                Gdk.threads_leave()
        else:
            if self.private_emblem is not None:
                private_emblem = self.private_emblem

                Gdk.threads_enter()

                self.indicator_box.remove(self.private_emblem)

                self.private_emblem = None

                Gdk.threads_leave()

                private_emblem.isolate()


    def _on_group_description_changed(self):
        description = self.group_backend.data[DESCRIPTION_KEY]

        if isinstance(description, unicode):
            description = description.strip()
        else:
            description = u'' 

        Gdk.threads_enter()

        if description:
            description = u'<i>{}</i>'.format(morescape(escape(description)))

            self.wraplabel.set_markup(description)

            if not self.info_button.toggled:
                self.wraplabel.show()
        else:
            self.wraplabel.set_text(description)

            self.wraplabel.hide()

        Gdk.threads_leave()


    def _on_group_member_count_changed(self):
        member_count = self.group_backend.data[MEMBER_COUNT_KEY]

        Gdk.threads_enter()

        self.member_count_link.set_markup(u'Members: {}'.format(member_count))

        Gdk.threads_leave()


    def _on_group_subscriber_count_changed(self):
        subscriber_count = self.group_backend.data[SUBSCRIBER_COUNT_KEY]

        Gdk.threads_enter()

        self.subscriber_count_link.set_markup(u'Subscribers: {}'.format(subscriber_count))

        Gdk.threads_leave()


    def _on_group_subscribed_changed(self):
        if self.group_backend.data[SUBSCRIBED_KEY]:
            if self.follow_button is not None:
                Gdk.threads_enter()

                self.spinners.remove(self.follow_button.spinner)

                self.action_box.remove(self.follow_button)

                Gdk.threads_leave()

                self.follow_button.isolate()

                Gdk.threads_enter()

                self.follow_button.destroy()
                self.follow_button = None

                Gdk.threads_leave()

            if self.unfollow_button is None:
                self.unfollow_button = self.button_factory.build(u'unfollow', self._on_unfollow_button_clicked)

                Gdk.threads_enter()

                self.unfollow_button.set_toggled(True)
                self.unfollow_button.box.set_tooltip_text(UNFOLLOW_BOX_TOOLTIP)
                self.unfollow_button.spinner.set_tooltip_text(UNFOLLOW_SPINNER_TOOLTIP)

                self.action_box.pack_end(self.unfollow_button, True, True, 0)

                self.spinners.append(self.unfollow_button.spinner)

                Gdk.threads_leave()

            Gdk.threads_enter()

            self.footer_link.set_markup(FOLLOWING_FOOTER)

            Gdk.threads_leave()
        else:
            if self.unfollow_button is not None:
                Gdk.threads_enter()

                self.spinners.remove(self.unfollow_button.spinner)

                self.action_box.remove(self.unfollow_button)

                Gdk.threads_leave()

                self.unfollow_button.isolate()

                Gdk.threads_enter()

                self.unfollow_button.destroy()
                self.unfollow_button = None

                Gdk.threads_leave()

            if self.follow_button is None:
                self.follow_button = self.button_factory.build(u'follow', self._on_follow_button_clicked)

                Gdk.threads_enter()

                self.follow_button.set_toggled(True)
                self.follow_button.box.set_tooltip_text(FOLLOW_BOX_TOOLTIP)
                self.follow_button.spinner.set_tooltip_text(FOLLOW_SPINNER_TOOLTIP)

                self.action_box.pack_end(self.follow_button, True, True, 0)

                self.spinners.append(self.follow_button.spinner)

                Gdk.threads_leave()

            Gdk.threads_enter()

            self.footer_link.set_markup(NOT_FOLLOWING_FOOTER)

            Gdk.threads_leave()


    def __init__(self, user_data, group_data, stream, user_backend_cache, group_backend_cache, color_scheme, font_scheme, emblem_factory, button_factory, avatar_factory, status_bar, main_panel):
        super(Group, self).__init__(user_data, stream, user_backend_cache, color_scheme, font_scheme, emblem_factory, button_factory, avatar_factory, status_bar, main_panel)

        self.private_emblem = None

        Gdk.threads_enter()

        self.member_count_link = Link(u'<small>{}</small>', self._on_member_count_link_clicked)

        self.subscriber_count_link = Link(u'<small>{}</small>', self._on_subscriber_count_link_clicked)

        self.tweets_link = Link(u'<small>{}</small>', self._on_tweets_link_clicked)
        self.tweets_link.set_markup(u'View tweets')

        self.info_box = Gtk.VBox(True, 0)
        self.info_box.pack_start(self.member_count_link, True, True, 0)
        self.info_box.pack_start(self.subscriber_count_link, True, True, 0)
        self.info_box.pack_start(self.tweets_link, True, True, 0)

        self.inner_box.pack_start(self.info_box, True, True, 0)

        Gdk.threads_leave()

        if self.user_own:
            self.edit_button = self.button_factory.build(u'edit', self._on_edit_button_clicked)

            self.delete_button = self.button_factory.build(u'delete', self._on_delete_button_clicked)

            Gdk.threads_enter()

            self.footer_link.set_markup(u'it\u2019s yours')

            self.edit_button.set_toggled(True)
            self.edit_button.box.set_tooltip_text(u'Edit')

            self.delete_button.set_toggled(True)
            self.delete_button.box.set_tooltip_text(DELETE_BOX_TOOLTIP)
            self.delete_button.spinner.set_tooltip_text(DELETE_SPINNER_TOOLTIP)

            self.action_box.pack_end(self.edit_button, True, True, 0)
            self.action_box.pack_end(self.delete_button, True, True, 0)

            self.spinners.append(self.delete_button.spinner)

            Gdk.threads_leave()
        else:
            self.follow_button = None

            self.unfollow_button = None

        self.group_backend = group_backend_cache.acquire(group_data)

        self._connect()

        self.group_backend.block_updates()

        self._on_group_nick_changed()

        self._on_group_name_changed()

        self._on_group_mode_changed()

        self._on_group_description_changed()

        self._on_group_member_count_changed()

        self._on_group_subscriber_count_changed()

        self.group_nick_uid = self.group_backend.nick_notifier.connect(self._on_group_nick_changed)

        self.group_name_uid = self.group_backend.name_notifier.connect(self._on_group_name_changed)

        self.group_mode_uid = self.group_backend.mode_notifier.connect(self._on_group_mode_changed)

        self.group_description_uid = self.group_backend.description_notifier.connect(self._on_group_description_changed)

        self.group_member_count_uid = self.group_backend.member_count_notifier.connect(self._on_group_member_count_changed)

        self.group_subscriber_count_uid = self.group_backend.subscriber_count_notifier.connect(self._on_group_subscriber_count_changed)

        if not self.user_own:
            self._on_group_subscribed_changed()

            self.group_subscribed_uid = self.group_backend.notifiers[SUBSCRIBED_KEY].connect(self._on_group_subscribed_changed)

        self.group_backend.allow_updates()

        self.edit_thread = None

        self.delete_thread = None

        self.follow_thread = None


    def isolate(self):
        if self.follow_thread is not None:
            self.follow_thread.join()

        if self.delete_thread is not None:
            self.delete_thread.join()

        if self.edit_thread is not None:
            self.edit_thread.join()

        self.group_backend.block_updates()

        if not self.user_own:
            self.group_backend.notifiers[SUBSCRIBED_KEY].disconnect(self.group_subscribed_uid)

        self.group_backend.subscriber_count_notifier.disconnect(self.group_subscriber_count_uid)

        self.group_backend.member_count_notifier.disconnect(self.group_member_count_uid)

        self.group_backend.description_notifier.disconnect(self.group_description_uid)

        self.group_backend.mode_notifier.disconnect(self.group_mode_uid)

        self.group_backend.name_notifier.disconnect(self.group_name_uid)

        self.group_backend.nick_notifier.disconnect(self.group_nick_uid)

        self.group_backend.allow_updates()

        self._disconnect()

        self.group_backend.release()

        if self.user_own:
            self.delete_button.isolate()

            self.edit_button.isolate()
        else:
            if self.unfollow_button is not None:
                self.unfollow_button.isolate()

            if self.follow_button is not None:
                self.follow_button.isolate()

        self.tweets_link.isolate()

        self.subscriber_count_link.isolate()

        self.member_count_link.isolate()

        if self.private_emblem is not None:
            self.private_emblem.isolate()

        super(Group, self).isolate()


    def get_parameters(self):
        name = unescape(self.group_name)

        description = self.wraplabel.get_text()

        index = 0 if self.private_emblem is None else 1

        return name, description, index



import dialog
