# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gtk as Gtk

import pango as Pango

from gtk import gdk as Gdk


from polly import ENCODING, escape, rescape, dispatch

from polly.twitter import ENTITIES_KEY, LINKS_KEY, MENTIONS_KEY, DISPLAY_URL_KEY, EXPANDED_URL_KEY

from polly.twitter.lead import DESCRIPTION_KEY, relationship

from polly.twitter.lead.user import VERIFIED_KEY, PROTECTED_KEY, LOCATION_KEY, URL_KEY, FOLLOWER_COUNT_KEY, FRIEND_COUNT_KEY, TWEET_COUNT_KEY, FAVORITE_COUNT_KEY, MEMBERSHIP_COUNT_KEY, PENDING_KEY

from polly.twitter.account import Error

from polly.gui import DEFAULT_BORDER, show_url, run_confirmation_dialog, run_generic_error_dialog

from polly.gui.twitter import COMPOSE_MESSAGE_TOOLTIP, CANCEL_MESSAGE_TOOLTIP, Link

from polly.gui.twitter.lead import FOLLOWING_FOOTER, NOT_FOLLOWING_FOOTER, FOLLOW_BOX_TOOLTIP, FOLLOW_SPINNER_TOOLTIP, UNFOLLOW_BOX_TOOLTIP, UNFOLLOW_SPINNER_TOOLTIP, Lead


from memberships import MembershipsDialog



MEMBERSHIPS_TITLE = u'Add or remove from your lists'



class User(Lead):
    def _reopenable(self):
        return True


    def _reopen(self, account):
        self.stream.manager.new_tweets_home_of_user(account, [self.user_backend.id, True], self.stream)


    def _remove_follow_button(self):
        if self.follow_button is not None:
            Gdk.threads_enter()

            self.spinners.remove(self.follow_button.spinner)

            self.basic_box.remove(self.follow_button)

            Gdk.threads_leave()

            self.follow_button.isolate()

            Gdk.threads_enter()

            self.follow_button.destroy()
            self.follow_button = None

            Gdk.threads_leave()


    def _remove_unfollow_button(self):
        if self.unfollow_button is not None:
            Gdk.threads_enter()

            self.spinners.remove(self.unfollow_button.spinner)

            self.basic_box.remove(self.unfollow_button)

            Gdk.threads_leave()

            self.unfollow_button.isolate()

            Gdk.threads_enter()

            self.unfollow_button.destroy()
            self.unfollow_button = None

            Gdk.threads_leave()


    def _relationship_task(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        try:
            relationship_data = self.stream.account.backend.get_relationship_data(self.user_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)

            Gdk.threads_enter()

            self.relationship_button.set_spinning(False)

            Gdk.threads_leave()
        else:
            self.block_button = self.button_factory.build(u'block', self._on_block_button_clicked)

            self.retweets_button = self.button_factory.build(u'retweets', self._on_retweets_button_clicked)

            self.notifications_button = self.button_factory.build(u'notifications', self._on_notifications_button_clicked)

            Gdk.threads_enter()

            self.spinners.remove(self.relationship_button.spinner)

            self.basic_box.remove(self.relationship_button)

            Gdk.threads_leave()

            self.relationship_button.isolate()

            Gdk.threads_enter()

            self.relationship_button.destroy()
            self.relationship_button = None

            self.block_button.hide()

            self.retweets_button.hide()

            self.notifications_button.hide()

            self.basic_box.pack_end(self.block_button, True, True, 0)

            self.advanced_box.pack_end(self.retweets_button, True, True, 0)
            self.advanced_box.pack_end(self.notifications_button, True, True, 0)

            self.spinners.append(self.block_button.spinner)

            self.spinners.append(self.retweets_button.spinner)

            self.spinners.append(self.notifications_button.spinner)

            Gdk.threads_leave()

            self.relationship_backend = self.relationship_backend_cache.acquire(relationship_data)

            self.relationship_backend.block_updates()

            self._on_relationship_social_changed()

            self._on_relationship_follower_changed()

            self._on_relationship_retweets_changed()

            self._on_relationship_notifications_changed()

            self.relationship_social_uid = self.relationship_backend.social_notifier.connect(self._on_relationship_social_changed)

            self.relationship_follower_uid = self.relationship_backend.follower_notifier.connect(self._on_relationship_follower_changed)

            self.relationship_retweets_uid = self.relationship_backend.notifiers[relationship.RETWEETS_KEY].connect(self._on_relationship_retweets_changed)

            self.relationship_notifications_uid = self.relationship_backend.notifiers[relationship.NOTIFICATIONS_KEY].connect(self._on_relationship_notifications_changed)

            self.relationship_backend.allow_updates()

            Gdk.threads_enter()

            self.block_button.show()

            self.retweets_button.show()

            self.notifications_button.show()

            Gdk.threads_leave()


    def _block_task(self, previous_thread, spam):
        if previous_thread is not None:
            previous_thread.join()

        try:
            if spam:
                user_data = self.stream.account.backend.send_user_report(self.user_backend.id)
            else:
                user_data = self.stream.account.backend.send_user_block(self.user_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)
        else:
            self.user_backend.set_data(user_data)

            self.relationship_backend.set_blocking(spam)

        Gdk.threads_enter()

        self.block_button.set_spinning(False)

        self.markup = None

        Gdk.threads_leave()


    def _unblock_task(self, previous_thread):
        if previous_thread is not None:
            previous_thread.join()

        try:
            user_data = self.stream.account.backend.delete_user_block(self.user_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)
        else:
            self.user_backend.set_data(user_data)

            self.relationship_backend.unset_blocking()

        Gdk.threads_enter()

        self.block_button.set_spinning(False)

        self.markup = None

        Gdk.threads_leave()


    def _follow_task(self, previous_thread, friend):
        if previous_thread is not None:
            previous_thread.join()

        try:
            if friend:
                user_data = self.stream.account.backend.delete_user_follow(self.user_backend.id)
            else:
                user_data = self.stream.account.backend.send_user_follow(self.user_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)
        else:
            self.user_backend.set_data(user_data)

            if friend:
                self.relationship_backend.update_friend(False)
            else:
                if user_data[PENDING_KEY]:
                    self.relationship_backend.refresh()
                else:
                    self.relationship_backend.update_friend(True)

        Gdk.threads_enter()

        if friend:
            if self.unfollow_button is not None:
                self.unfollow_button.set_spinning(False)
        else:
            if self.follow_button is not None:
                self.follow_button.set_spinning(False)

        self.markup = None

        Gdk.threads_leave()


    def _retweets_task(self, previous_thread, toggled):
        if previous_thread is not None:
            previous_thread.join()

        try:
            if toggled:
                relationship_data = self.stream.account.backend.delete_relationship_retweets(self.user_backend.id)
            else:
                relationship_data = self.stream.account.backend.send_relationship_retweets(self.user_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)
        else:
            if toggled:
                self.relationship_backend.set_value(relationship.RETWEETS_KEY, False)
            else:
                self.relationship_backend.set_value(relationship.RETWEETS_KEY, True)

            self.relationship_backend.set_data(relationship_data)

        Gdk.threads_enter()

        self.retweets_button.set_spinning(False)

        Gdk.threads_leave()


    def _notifications_task(self, previous_thread, toggled):
        if previous_thread is not None:
            previous_thread.join()

        try:
            if toggled:
                relationship_data = self.stream.account.backend.delete_relationship_notifications(self.user_backend.id)
            else:
                relationship_data = self.stream.account.backend.send_relationship_notifications(self.user_backend.id)
        except Error as error:
            self.status_bar.popup(error.message)
        else:
            if toggled:
                self.relationship_backend.set_value(relationship.NOTIFICATIONS_KEY, False)
            else:
                self.relationship_backend.set_value(relationship.NOTIFICATIONS_KEY, True)

            self.relationship_backend.set_data(relationship_data)

        Gdk.threads_enter()

        self.notifications_button.set_spinning(False)

        Gdk.threads_leave()


    def _set_normal_fg(self, color):
        super(User, self)._set_normal_fg(color)

        self.location_label.modify_fg(Gtk.STATE_NORMAL, color)


    def _set_normal_bg(self, color):
        super(User, self)._set_normal_bg(color)

        self.url_label.modify_base(Gtk.STATE_PRELIGHT, color)


    def _set_normal_mg(self, color):
        super(User, self)._set_normal_mg(color)

        self.follower_count_link.set_color(color)

        self.friend_count_link.set_color(color)

        self.tweet_count_link.set_color(color)

        self.favorite_count_link.set_color(color)

        self.membership_count_link.set_color(color)

        self.subscriptions_link.set_color(color)

        self.groups_link.set_color(color)


    def _set_select_bg(self, color):
        super(User, self)._set_select_bg(color)

        self.url_label.modify_base(Gtk.STATE_ACTIVE, color)


    def _set_font(self, unit, void):
        super(User, self)._set_font(unit, void)

        self.info_box.set_spacing(2 * void)


    def _update_user_nick(self):
        super(User, self)._update_user_nick()

        self.header_link.set_markup(self.user_nick)


    def _update_user_name(self):
        super(User, self)._update_user_name()

        self.header_link.set_markup(self.user_name)


    def _on_header_clicked(self):
        self._on_avatar_clicked()


    def _on_info_button_clicked(self):
        super(User, self)._on_info_button_clicked()

        if self.info_button.toggled:
            self.header_link.set_markup(self.user_name)

            self.basic_box.hide()

            self.advanced_box.show()
        else:
            self.header_link.set_markup(self.user_nick)

            self.advanced_box.hide()

            self.basic_box.show()


    def _on_follower_count_clicked(self):
        self.stream.manager.new_users_followers_of_user(self.stream.account, self.user_backend.id, self.stream)


    def _on_friend_count_clicked(self):
        self.stream.manager.new_users_friends_of_user(self.stream.account, self.user_backend.id, self.stream)


    def _on_tweet_count_clicked(self):
        self.stream.manager.new_tweets_sent_by_user(self.stream.account, self.user_backend.id, self.stream)


    def _on_favorite_count_clicked(self):
        self.stream.manager.new_tweets_favorited_by_user(self.stream.account, self.user_backend.id, self.stream)


    def _on_membership_count_clicked(self):
        self.stream.manager.new_groups_memberships_of_user(self.stream.account, self.user_backend.id, self.stream)


    def _on_subscriptions_clicked(self):
        self.stream.manager.new_groups_subscriptions_of_user(self.stream.account, self.user_backend.id, self.stream)


    def _on_groups_clicked(self):
        self.stream.manager.new_groups_groups_of_user(self.stream.account, self.user_backend.id, self.stream)


    def _on_footer_clicked(self):
        show_url(u'http://twitter.com/{}'.format(self.user_nick))


    def _on_relationship_button_clicked(self):
        self.relationship_button.set_spinning(True)

        self.relationship_thread = dispatch(self._relationship_task, self.relationship_thread)


    def _on_block_button_clicked(self):
        if self.markup is not None:
            run_generic_error_dialog(self.get_toplevel(), self.markup)

            return

        if self.block_button.toggled:
            self.markup = u'Please wait until the unblocking process is finished and try again.'

            self.block_button.set_spinning(True)

            self.social_thread = dispatch(self._unblock_task, self.social_thread)
        else:
            toplevel = self.get_toplevel()

            image = Gtk.image_new_from_stock(Gtk.STOCK_DIALOG_WARNING, Gtk.ICON_SIZE_DIALOG)
            image.show()

            alignment = Gtk.Alignment(0.5, 0.0, 0.0, 0.0)
            alignment.add(image)
            alignment.show()

            text = u'Are you sure you want to block <i>{}</i>?'.format(self.user_nick)

            label = Gtk.Label(None)
            label.set_markup(u'<big><b>{}</b></big>'.format(text))
            label.show()

            checkbutton = Gtk.CheckButton(u'Report as spam', False)
            checkbutton.show()

            inner_box = Gtk.VBox(False, 2 * DEFAULT_BORDER)
            inner_box.pack_start(label, False, False, 0)
            inner_box.pack_start(checkbutton, False, False, 0)
            inner_box.show()

            outer_box = Gtk.HBox(False, 2 * DEFAULT_BORDER)
            outer_box.set_border_width(2 * DEFAULT_BORDER)
            outer_box.pack_start(alignment, False, False, 0)
            outer_box.pack_start(inner_box, True, True, 0)
            outer_box.show()

            dialog = Gtk.Dialog(u'Block user', toplevel, Gtk.DIALOG_MODAL, (Gtk.STOCK_CANCEL, Gtk.RESPONSE_REJECT, Gtk.STOCK_OK, Gtk.RESPONSE_ACCEPT))
            dialog.get_content_area().add(outer_box)
            dialog.set_resizable(False)

            toplevel.on_dialog_run()

            spam = (checkbutton.get_active() if dialog.run() == Gtk.RESPONSE_ACCEPT else None)

            dialog.destroy()

            toplevel.on_dialog_destroy()

            if spam is not None:
                self.markup = u'Please wait until the blocking process is finished and try again.'

                if spam:
                    self.block_button.spinner.set_tooltip_text(u'Reporting as spam\u2026')
                else:
                    self.block_button.spinner.set_tooltip_text(u'Blocking\u2026')

                self.block_button.set_spinning(True)

                self.social_thread = dispatch(self._block_task, self.social_thread, spam)


    def _on_follow_button_clicked(self):
        if self.markup is not None:
            run_generic_error_dialog(self.get_toplevel(), self.markup)

            return

        self.markup = u'Please wait until the following process is finished and try again.'

        if self.user_protected:
            success = run_confirmation_dialog(self.get_toplevel(), u'Follow user',
                                              u'A friendship request will be sent to <i>{}</i>'.format(self.user_nick),
                                              u'This request can only be cancelled from the Twitter website.')
        else:
            success = True

        if success:
            self.follow_button.set_spinning(True)

            self.social_thread = dispatch(self._follow_task, self.social_thread, False)


    def _on_unfollow_button_clicked(self):
        if self.markup is not None:
            run_generic_error_dialog(self.get_toplevel(), self.markup)

            return

        self.markup = u'Please wait until the unfollowing process is finished and try again.'

        if self.user_protected:
            success = run_confirmation_dialog(self.get_toplevel(), u'Unfollow user',
                                              u'Are you sure you want to unfollow <i>{}</i>?'.format(self.user_nick),
                                              u'To re-follow you will need to wait for his approval again.')
        else:
            success = True

        if success:
            self.unfollow_button.set_spinning(True)

            self.social_thread = dispatch(self._follow_task, self.social_thread, True)


    def _on_message_button_clicked(self):
        if self.message_button.toggled:
            self.main_panel.cancel_message_to_user()
        else:
            self.main_panel.compose_message_to_user(self)


    def _on_lists_button_clicked(self):
        dialog = MembershipsDialog(MEMBERSHIPS_TITLE, self.get_toplevel(), self.stream.account, self)

        self.memberships_thread = dialog.run_and_destroy()


    def _on_retweets_button_clicked(self):
        self.retweets_button.set_spinning(True)

        self.retweets_thread = dispatch(self._retweets_task, self.retweets_thread, self.retweets_button.toggled)


    def _on_notifications_button_clicked(self):
        self.notifications_button.set_spinning(True)

        self.notifications_thread = dispatch(self._notifications_task, self.notifications_thread, self.notifications_button.toggled)


    def _on_user_verified_changed(self):
        if self.user_backend.data[VERIFIED_KEY]:
            if self.verified_emblem is None:
                self.verified_emblem = self.emblem_factory.build(u'verified')

                Gdk.threads_enter()

                self.verified_emblem.set_tooltip_text(u'Verified')

                self.indicator_box.pack_end(self.verified_emblem, True, True, 0)
                self.indicator_box.reorder_child(self.verified_emblem, 1)

                Gdk.threads_leave()
        else:
            if self.verified_emblem is not None:
                Gdk.threads_enter()

                self.indicator_box.remove(self.verified_emblem)

                Gdk.threads_leave()

                self.verified_emblem.isolate()
                self.verified_emblem = None


    def _on_user_protected_changed(self):
        Gdk.threads_enter()

        self.user_protected = self.user_backend.data[PROTECTED_KEY]

        Gdk.threads_leave()

        if self.user_protected:
            if self.protected_emblem is None:
                self.protected_emblem = self.emblem_factory.build(u'protected')

                Gdk.threads_enter()

                self.protected_emblem.set_tooltip_text(u'Protected')

                self.indicator_box.pack_end(self.protected_emblem, True, True, 0)

                Gdk.threads_leave()
        else:
            if self.protected_emblem is not None:
                Gdk.threads_enter()

                self.indicator_box.remove(self.protected_emblem)

                Gdk.threads_enter()

                self.protected_emblem.isolate()
                self.protected_emblem = None


    def _on_user_description_changed(self):
        description = self.user_backend.data[DESCRIPTION_KEY]

        if isinstance(description, unicode):
            entities = self.user_backend.data[ENTITIES_KEY]

            if entities is not None:
                replacements = []

                dir = Pango.find_base_dir(description.encode(ENCODING), -1)

                rtl = (dir == Pango.DIRECTION_RTL)

                try:
                    for link in entities[LINKS_KEY]:
                        try:
                            self._append_link(description, replacements, link)
                        except (TypeError, KeyError, ValueError):
                            pass
                except (TypeError, KeyError):
                    pass

                self._append_hashtags(description, replacements, rtl, entities)

                try:
                    for mention in entities[MENTIONS_KEY]:
                        try:
                            self._append_mention(description, replacements, rtl, mention)
                        except (TypeError, KeyError, ValueError):
                            pass
                except (TypeError, KeyError):
                    pass

                if replacements:
                    description = self._join_replacements(description, replacements)

            description = description.strip()
        else:
            description = u'' 

        Gdk.threads_enter()

        if description:
            description = u'<i>{}</i>'.format(escape(description))

            self.wraplabel.set_markup(description)

            if not self.info_button.toggled:
                self.wraplabel.show()
        else:
            self.wraplabel.set_text(description)

            self.wraplabel.hide()

        Gdk.threads_leave()


    def _on_user_location_changed(self):
        location = self.user_backend.data[LOCATION_KEY]

        if isinstance(location, unicode):
            location = location.strip()
        else:
            location = u''

        Gdk.threads_enter()

        if location:
            self.location_label.set_markup(u'<small>{}</small>'.format(escape(location)))

            self.location_label.show()

            self.top_box.show()
        else:
            if not self.url_label.get_visible():
                self.top_box.hide()

            self.location_label.hide()

        Gdk.threads_leave()


    def _on_user_url_changed(self):
        Gdk.threads_enter()

        url = self.user_backend.data[URL_KEY]

        if isinstance(url, unicode):
            url = url.strip()
        else:
            url = u'' 

        if url:
            display_url = self.user_backend.data[DISPLAY_URL_KEY]

            if isinstance(display_url, unicode):
                display_url = display_url.strip()
            else:
                display_url = u''

            if display_url:
                display_url = escape(display_url)
            else:
                if url.startswith(u'http://'):
                    display_url = url[7:]
                elif url.startswith(u'https://'):
                    display_url = url[8:]
                else:
                    display_url = url

                display_url = display_url.strip(u'#')
                display_url = display_url.strip(u'&')
                display_url = display_url.strip(u'?')
                display_url = display_url.strip(u'/')

                display_url = escape(display_url)

                valid = True

                count = 0

                for index in xrange(len(display_url)):
                    if display_url[index] == u'&':
                        valid = False
                    elif display_url[index] == u';':
                        valid = True

                    if valid:
                        count += 1

                        if count == 29:
                            break

                if index < len(display_url) - 1:
                    display_url = u'{}\u2026'.format(display_url[0:(index + 1)])

            expanded_url = self.user_backend.data[EXPANDED_URL_KEY]

            if isinstance(expanded_url, unicode):
                expanded_url = expanded_url.strip()
            else:
                expanded_url = u''

            if expanded_url:
                expanded_url = rescape(escape(expanded_url))
            else:
                expanded_url = rescape(escape(url))

            markup = u'<small><a href="{}" title="{}">{}</a></small>'.format(url, expanded_url, display_url)

            self.url_label.set_markup(escape(markup))

            self.url_label.show()

            self.top_box.show()
        else:
            if not self.location_label.get_visible():
                self.top_box.hide()

            self.url_label.hide()

        Gdk.threads_leave()


    def _on_user_follower_count_changed(self):
        follower_count = self.user_backend.data[FOLLOWER_COUNT_KEY]

        Gdk.threads_enter()

        self.follower_count_link.set_markup(u'Followers: {}'.format(follower_count))

        Gdk.threads_leave()


    def _on_user_friend_count_changed(self):
        friend_count = self.user_backend.data[FRIEND_COUNT_KEY]

        Gdk.threads_enter()

        self.friend_count_link.set_markup(u'Friends: {}'.format(friend_count))

        Gdk.threads_leave()


    def _on_user_tweet_count_changed(self):
        tweet_count = self.user_backend.data[TWEET_COUNT_KEY]

        Gdk.threads_enter()

        self.tweet_count_link.set_markup(u'Tweets: {}'.format(tweet_count))

        Gdk.threads_leave()


    def _on_user_favorite_count_changed(self):
        favorite_count = self.user_backend.data[FAVORITE_COUNT_KEY]

        Gdk.threads_enter()

        self.favorite_count_link.set_markup(u'Favorites: {}'.format(favorite_count))

        Gdk.threads_leave()


    def _on_user_membership_count_changed(self):
        membership_count = self.user_backend.data[MEMBERSHIP_COUNT_KEY]

        Gdk.threads_enter()

        self.membership_count_link.set_markup(u'Memberships: {}'.format(membership_count))

        Gdk.threads_leave()


    def _on_relationship_social_changed(self):
        if self.relationship_backend.data[relationship.BLOCKING_KEY]:
            Gdk.threads_enter()

            self.block_button.set_toggled(True)
            self.block_button.box.set_tooltip_text(u'Unblock')
            self.block_button.spinner.set_tooltip_text(u'Unblocking\u2026')

            Gdk.threads_leave()

            self._remove_unfollow_button()

            self._remove_follow_button()

            if self.relationship_backend.data[relationship.SPAM_KEY]:
                Gdk.threads_enter()

                self.footer_link.set_markup(u'reported as spam')

                Gdk.threads_leave()
            else:
                Gdk.threads_enter()

                self.footer_link.set_markup(u'blocked')

                Gdk.threads_leave()
        else:
            Gdk.threads_enter()

            self.block_button.set_toggled(False)
            self.block_button.box.set_tooltip_text(u'Block or report as spam')

            Gdk.threads_leave()

            if self.relationship_backend.data[relationship.FRIEND_KEY]:
                self._remove_follow_button()

                if self.unfollow_button is None:
                    self.unfollow_button = self.button_factory.build(u'unfollow', self._on_unfollow_button_clicked)

                    Gdk.threads_enter()

                    self.unfollow_button.set_toggled(True)
                    self.unfollow_button.box.set_tooltip_text(UNFOLLOW_BOX_TOOLTIP)
                    self.unfollow_button.spinner.set_tooltip_text(UNFOLLOW_SPINNER_TOOLTIP)

                    self.basic_box.pack_end(self.unfollow_button, True, True, 0)
                    self.basic_box.reorder_child(self.unfollow_button, 1)

                    self.spinners.append(self.unfollow_button.spinner)

                    Gdk.threads_leave()

                Gdk.threads_enter()

                self.footer_link.set_markup(FOLLOWING_FOOTER)

                Gdk.threads_leave()
            else:
                self._remove_unfollow_button()

                if self.user_backend.get_value(PENDING_KEY):
                    self._remove_follow_button()

                    Gdk.threads_enter()

                    self.footer_link.set_markup(u'pending')

                    Gdk.threads_leave()
                else:
                    if self.follow_button is None:
                        self.follow_button = self.button_factory.build(u'follow', self._on_follow_button_clicked)

                        Gdk.threads_enter()

                        self.follow_button.set_toggled(True)
                        self.follow_button.box.set_tooltip_text(FOLLOW_BOX_TOOLTIP)
                        self.follow_button.spinner.set_tooltip_text(FOLLOW_SPINNER_TOOLTIP)

                        self.basic_box.pack_end(self.follow_button, True, True, 0)
                        self.basic_box.reorder_child(self.follow_button, 1)

                        self.spinners.append(self.follow_button.spinner)

                        Gdk.threads_leave()

                    Gdk.threads_enter()

                    self.footer_link.set_markup(NOT_FOLLOWING_FOOTER)

                    Gdk.threads_leave()


    def _on_relationship_follower_changed(self):
        if self.relationship_backend.data[relationship.FOLLOWER_KEY]:
            if self.message_button is None:
                self.message_button = self.button_factory.build(u'message', self._on_message_button_clicked)

                Gdk.threads_enter()

                if self.main_panel.messaged_user == self:
                    self.message_button.set_toggled(True)
                    self.message_button.box.set_tooltip_text(CANCEL_MESSAGE_TOOLTIP)
                else:
                    self.message_button.set_toggled(False)
                    self.message_button.box.set_tooltip_text(COMPOSE_MESSAGE_TOOLTIP)

                self.basic_box.pack_end(self.message_button, True, True, 0)

                Gdk.threads_leave()
        else:
            if self.message_button is not None:
                Gdk.threads_enter()

                self.basic_box.remove(self.message_button)

                Gdk.threads_leave()

                self.message_button.isolate()

                Gdk.threads_enter()

                self.message_button.destroy()
                self.message_button = None

                Gdk.threads_leave()


    def _on_relationship_retweets_changed(self):
        Gdk.threads_enter()

        self.retweets_button.set_toggled(self.relationship_backend.data[relationship.RETWEETS_KEY])

        if self.retweets_button.toggled:
            self.retweets_button.box.set_tooltip_text(u'Disable user retweets')
            self.retweets_button.spinner.set_tooltip_text(u'Disabling user retweets\u2026')
        else:
            self.retweets_button.box.set_tooltip_text(u'Enable user retweets')
            self.retweets_button.spinner.set_tooltip_text(u'Enabling user retweets\u2026')

        Gdk.threads_leave()


    def _on_relationship_notifications_changed(self):
        Gdk.threads_enter()

        self.notifications_button.set_toggled(self.relationship_backend.data[relationship.NOTIFICATIONS_KEY])

        if self.notifications_button.toggled:
            self.notifications_button.box.set_tooltip_text(u'Disable mobile notifications')
            self.notifications_button.spinner.set_tooltip_text(u'Disabling mobile notifications\u2026')
        else:
            self.notifications_button.box.set_tooltip_text(u'Enable mobile notifications')
            self.notifications_button.spinner.set_tooltip_text(u'Enabling mobile notifications\u2026')

        Gdk.threads_leave()


    def __init__(self, user_data, stream, user_backend_cache, relationship_backend_cache, color_scheme, font_scheme, emblem_factory, button_factory, avatar_factory, status_bar, main_panel):
        super(User, self).__init__(user_data, stream, user_backend_cache, color_scheme, font_scheme, emblem_factory, button_factory, avatar_factory, status_bar, main_panel)

        self.relationship_backend_cache = relationship_backend_cache

        self.markup = None

        self.verified_emblem = None

        self.protected_emblem = None

        Gdk.threads_enter()

        self.wraplabel.connect(u'activate-link', self._on_label_activate_link)

        self.location_label = Gtk.Label(None)
        self.location_label.set_alignment(0.0, 0.5)
        self.location_label.show()

        self.url_label = Gtk.Label(None)
        self.url_label.set_alignment(0.0, 0.5)
        self.url_label.show()

        self.top_box = Gtk.VBox(True, 0)
        self.top_box.pack_start(self.location_label, True, True, 0)
        self.top_box.pack_start(self.url_label, True, True, 0)
        self.top_box.show()

        self.follower_count_link = Link(u'<small>{}</small>', self._on_follower_count_clicked)

        self.friend_count_link = Link(u'<small>{}</small>', self._on_friend_count_clicked)

        self.tweet_count_link = Link(u'<small>{}</small>', self._on_tweet_count_clicked)

        self.favorite_count_link = Link(u'<small>{}</small>', self._on_favorite_count_clicked)

        self.membership_count_link = Link(u'<small>{}</small>', self._on_membership_count_clicked)

        self.subscriptions_link = Link(u'<small>{}</small>', self._on_subscriptions_clicked)
        self.subscriptions_link.set_markup(u'View subscriptions')

        self.groups_link = Link(u'<small>{}</small>', self._on_groups_clicked)
        self.groups_link.set_markup(u'View lists')

        bottom_box = Gtk.VBox(True, 0)
        bottom_box.pack_start(self.follower_count_link, True, True, 0)
        bottom_box.pack_start(self.friend_count_link, True, True, 0)
        bottom_box.pack_start(self.tweet_count_link, True, True, 0)
        bottom_box.pack_start(self.favorite_count_link, True, True, 0)
        bottom_box.pack_start(self.membership_count_link, True, True, 0)
        bottom_box.pack_start(self.subscriptions_link, True, True, 0)
        bottom_box.pack_start(self.groups_link, True, True, 0)
        bottom_box.show()

        self.info_box = Gtk.VBox(False, 0)
        self.info_box.pack_start(self.top_box, False, False, 0)
        self.info_box.pack_start(bottom_box, False, False, 0)

        self.inner_box.pack_start(self.info_box, True, True, 0)

        self.basic_box = Gtk.HBox(True, DEFAULT_BORDER)
        self.basic_box.show()

        self.advanced_box = Gtk.HBox(True, DEFAULT_BORDER)

        self.action_box.pack_end(self.basic_box, True, True, 0)
        self.action_box.pack_end(self.advanced_box, True, True, 0)

        Gdk.threads_leave()

        if self.user_own:
            self.relationship_button = None

            self.message_button = self.button_factory.build(u'message', self._on_message_button_clicked)

            self.lists_button = None

            Gdk.threads_enter()

            self.footer_link.set_markup(u'it\u2019s you')

            self.message_button.box.set_tooltip_text(COMPOSE_MESSAGE_TOOLTIP)

            self.basic_box.pack_end(self.message_button, True, True, 0)

            Gdk.threads_leave()
        else:
            self.relationship_button = self.button_factory.build(u'relationship', self._on_relationship_button_clicked)

            self.message_button = None

            self.lists_button = self.button_factory.build(u'lists', self._on_lists_button_clicked)

            Gdk.threads_enter()

            self.footer_link.set_markup(u'unknown relationship')

            self.relationship_button.set_toggled(True)
            self.relationship_button.box.set_tooltip_text(u'Discover relationship')
            self.relationship_button.spinner.set_tooltip_text(u'Discovering relationship\u2026')

            self.basic_box.pack_end(self.relationship_button, True, True, 0)

            self.lists_button.set_toggled(True)
            self.lists_button.box.set_tooltip_text(MEMBERSHIPS_TITLE)

            self.advanced_box.pack_end(self.lists_button, True, True, 0)

            self.spinners.append(self.relationship_button.spinner)

            self.spinners.append(self.lists_button.spinner)

            Gdk.threads_leave()

        self.block_button = None

        self.follow_button = None

        self.unfollow_button = None

        self.retweets_button = None

        self.notifications_button = None

        self._connect()

        self.user_backend.block_updates()

        self._on_user_verified_changed()

        self._on_user_protected_changed()

        self._on_user_description_changed()

        self._on_user_location_changed()

        self._on_user_url_changed()

        self._on_user_follower_count_changed()

        self._on_user_friend_count_changed()

        self._on_user_tweet_count_changed()

        self._on_user_favorite_count_changed()

        self._on_user_membership_count_changed()

        self.user_verified_uid = self.user_backend.verified_notifier.connect(self._on_user_verified_changed)

        self.user_protected_uid = self.user_backend.protected_notifier.connect(self._on_user_protected_changed)

        self.user_description_uid = self.user_backend.description_notifier.connect(self._on_user_description_changed)

        self.user_location_uid = self.user_backend.location_notifier.connect(self._on_user_location_changed)

        self.user_url_uid = self.user_backend.url_notifier.connect(self._on_user_url_changed)

        self.user_follower_count_uid = self.user_backend.follower_count_notifier.connect(self._on_user_follower_count_changed)

        self.user_friend_count_uid = self.user_backend.friend_count_notifier.connect(self._on_user_friend_count_changed)

        self.user_tweet_count_uid = self.user_backend.tweet_count_notifier.connect(self._on_user_tweet_count_changed)

        self.user_favorite_count_uid = self.user_backend.favorite_count_notifier.connect(self._on_user_favorite_count_changed)

        self.user_membership_count_uid = self.user_backend.membership_count_notifier.connect(self._on_user_membership_count_changed)

        self.user_backend.allow_updates()

        self.relationship_backend = None

        self.relationship_thread = None

        self.social_thread = None

        self.memberships_thread = None

        self.retweets_thread = None

        self.notifications_thread = None


    def isolate(self):
        if self.notifications_thread is not None:
            self.notifications_thread.join()

        if self.retweets_thread is not None:
            self.retweets_thread.join()

        if self.memberships_thread is not None:
            self.memberships_thread.join()

        if self.social_thread is not None:
            self.social_thread.join()

        if self.relationship_thread is not None:
            self.relationship_thread.join()

        if self.relationship_backend is not None:
            self.relationship_backend.block_updates()

            self.relationship_backend.notifiers[relationship.NOTIFICATIONS_KEY].disconnect(self.relationship_notifications_uid)

            self.relationship_backend.notifiers[relationship.RETWEETS_KEY].disconnect(self.relationship_retweets_uid)

            self.relationship_backend.follower_notifier.disconnect(self.relationship_follower_uid)

            self.relationship_backend.social_notifier.disconnect(self.relationship_social_uid)

            self.relationship_backend.allow_updates()

            self.relationship_backend.release()

        self.user_backend.block_updates()

        self.user_backend.membership_count_notifier.disconnect(self.user_membership_count_uid)

        self.user_backend.favorite_count_notifier.disconnect(self.user_favorite_count_uid)

        self.user_backend.tweet_count_notifier.disconnect(self.user_tweet_count_uid)

        self.user_backend.friend_count_notifier.disconnect(self.user_friend_count_uid)

        self.user_backend.follower_count_notifier.disconnect(self.user_follower_count_uid)

        self.user_backend.url_notifier.disconnect(self.user_url_uid)

        self.user_backend.location_notifier.disconnect(self.user_location_uid)

        self.user_backend.description_notifier.disconnect(self.user_description_uid)

        self.user_backend.protected_notifier.disconnect(self.user_protected_uid)

        self.user_backend.verified_notifier.disconnect(self.user_verified_uid)

        self.user_backend.allow_updates()

        self._disconnect()

        if self.notifications_button is not None:
            self.notifications_button.isolate()

        if self.retweets_button is not None:
            self.retweets_button.isolate()

        if self.lists_button is not None:
            self.lists_button.isolate()

        if self.message_button is not None:
            self.message_button.isolate()

        if self.unfollow_button is not None:
            self.unfollow_button.isolate()

        if self.follow_button is not None:
            self.follow_button.isolate()

        if self.block_button is not None:
            self.block_button.isolate()

        if self.relationship_button is not None:
            self.relationship_button.isolate()

        self.groups_link.isolate()

        self.subscriptions_link.isolate()

        self.membership_count_link.isolate()

        self.favorite_count_link.isolate()

        self.tweet_count_link.isolate()

        self.friend_count_link.isolate()

        self.follower_count_link.isolate()

        if self.protected_emblem is not None:
            self.protected_emblem.isolate()

        if self.verified_emblem is not None:
            self.verified_emblem.isolate()

        super(User, self).isolate()


    def set_messaging(self, messaging):
        if self.message_button is not None:
            self.message_button.set_toggled(messaging)

            if messaging:
                self.popup()

                self.message_button.box.set_tooltip_text(CANCEL_MESSAGE_TOOLTIP)
            else:
                self.message_button.box.set_tooltip_text(COMPOSE_MESSAGE_TOOLTIP)

        self._set_selected(messaging)


    def click_relationship_button(self):
        if self.relationship_button is not None:
            self._on_relationship_button_clicked()



import memberships
