# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import re

from time import localtime, strftime


import gtk as Gtk

import pango as Pango

from gtk import gdk as Gdk


from polly import ENCODING, escape

from polly.twitter import CREATION_TIMESTAMP_KEY, ENTITIES_KEY, LINKS_KEY, MENTIONS_KEY

from polly.twitter.post import TEXT_KEY

from polly.gui import notify

from polly.gui.twitter import Item



AVATAR_DIMENSION = 3


MEDIA_SPECS = [(u'imgur', u'(?:http\\:\\/\\/)?(?:.+\\.)?imgur\\.com\\/(?:gallery\\/)?([0-9A-Za-z]+)', u'http://i.imgur.com/{}b.jpg'),
               (u'imgly', u'(?:http\\:\\/\\/)?img\\.ly\\/([0-9A-Za-z]+)', u'http://img.ly/show/thumb/{}'),
               (u'yfrog', u'(?:http\\:\\/\\/)?yfrog\\.com\\/([0-9a-z]+)', u'http://yfrog.com/{}:small'),
               (u'twitgoo', u'(?:http\\:\\/\\/)?twitgoo\\.com\\/([0-9a-z]+)', u'http://twitgoo.com/show/thumb/{}'),
               (u'twitvid', u'(?:http\\:\\/\\/)?twitvid\\.com\\/([0-9A-Z]+)', u'http://llphotos.twitvid.com/twitvidthumbnails/{}_med.jpg'),
               (u'twitpic', u'(?:http\\:\\/\\/)?twitpic\\.com\\/([0-9a-z]+)', u'http://twitpic.com/show/thumb/{}'),
               (u'tinypic', u'(?:http\\:\\/\\/)?(.+)\\.tinypic\\.com\\/([0-9a-z]+)', u'http://{}.tinypic.com/{}_th'),
               (u'picplz', u'(?:http\\:\\/\\/)?picplz\\.com\\/([0-9A-Za-z]+)', u'http://picplz.com/{}/thumb/150'),
               (u'lockerz', u'(?:http\\:\\/\\/)?lockerz\\.com\\/s\\/([0-9]+)', u'http://api.plixi.com/api/tpapi.svc/imagefromurl?url=http://lockerz.com/s/{}&size=small'),
               (u'brizzly', u'(?:http\\:\\/\\/)?brizzly\\.com\\/pic\\/([0-9A-Z]+)', u'http://pics.brizzly.com/thumb_sm_{}.jpg'),
               (u'instagram', u'(?:http\\:\\/\\/)?instagr\\.am\\/p\\/([0-9A-Za-z\\-\\_]+)', u'http://instagr.am/p/{}/media/?size=t'),
               (u'flickr', u'(?:http\\:\\/\\/)?flic\\.kr\\/p\\/([0-9A-Za-z]+)', u'http://flic.kr/p/img/{}_t.jpg'),
               (u'youtube', u'(?:http\\:\\/\\/)?(?:youtu\\.be\\/|www\\.youtube\\.com\\/watch\\?(?:.+\\=.+\\&)*v\\=)([0-9A-Za-z\\-\\_]+)', u'http://img.youtube.com/vi/{}/default.jpg')]



class Post(Item):
    def _notify(self):
        Gdk.threads_enter()

        summary = self.user_nick

        body = self.wraplabel.get_text().decode(ENCODING)

        pixbuf = self.user_avatar.pixbuf

        Gdk.threads_leave()

        notify(summary, body, pixbuf)


    def _get_small_time(self, timestamp):
        seconds = timestamp - self.timestamp

        minutes = seconds / 60.0

        if minutes < 1.0:
            small_time = u'less than a minute ago'
        else:
            if minutes < 2.0:
                small_time = u'1 minute ago'
            else:
                if minutes < 60.0:
                    small_time = u'{} minutes ago'.format(long(minutes))
                else:
                    hours = minutes / 60.0

                    if hours < 2.0:
                        small_time = u'1 hour ago'
                    else:
                        if hours < 24.0:
                            small_time = u'{} hours ago'.format(long(hours))
                        else:
                            datetime = localtime(self.timestamp)

                            day = datetime.tm_mday

                            month = strftime(u'%b', datetime)

                            year = datetime.tm_year

                            if year == localtime(timestamp).tm_year:
                                small_time = u'{} {}'.format(day, month)
                            else:
                                small_time = u'{} {} {}'.format(day, month, year)

        return small_time


    def _get_large_time(self):
        datetime = localtime(self.timestamp)

        return strftime(u'%a %d %b %Y %H:%M:%S', datetime)


    def _update_user_nick(self):
        super(Post, self)._update_user_nick()

        self.header_link.set_markup(self.user_nick)


    def _update_user_name(self):
        super(Post, self)._update_user_name()

        self.header_link.set_markup(self.user_name)


    def _on_header_clicked(self):
        self._on_avatar_clicked()


    def _on_info_button_clicked(self):
        super(Post, self)._on_info_button_clicked()

        if self.info_button.toggled:
            self.header_link.set_markup(self.user_name)

            self.footer_link.set_markup(self.large_time)
        else:
            self.header_link.set_markup(self.user_nick)

            self.footer_link.set_markup(self.small_time)


    def _on_size_allocate_two(self, widget, allocation):
        difference = allocation.height - self.allocation_height

        if difference > 0:
            adjustment = self.stream.adjustment

            value = adjustment.get_value() + float(difference)

            upper = adjustment.get_upper() - adjustment.get_page_size()

            adjustment.set_value(min(value, upper))

            if self.allocation_passed:
                self.disconnect(self.allocation_uid)

                self.connect(u'size-allocate', self._on_size_allocate_top)
            else:
                self.allocation_passed = True

            self.allocation_height = allocation.height


    def _on_size_allocate_top(self, widget, allocation):
        difference = allocation.height - self.allocation_height

        if difference != 0:
            adjustment = self.stream.adjustment

            value = adjustment.get_value()

            if allocation.y + self.allocation_height < long(value):
                value += float(difference)

                if difference > 0:
                    upper = adjustment.get_upper() - adjustment.get_page_size()

                    adjustment.set_value(min(value, upper))
                else:
                    adjustment.set_value(max(value, 0.0))

            self.allocation_height = allocation.height


    def __init__(self, real_id, user_data, post_data, stream, user_backend_cache, color_scheme, font_scheme, button_factory, avatar_factory, preview_factory, status_bar, main_panel):
        super(Post, self).__init__(AVATAR_DIMENSION, 0, user_data, stream, user_backend_cache, color_scheme, font_scheme, button_factory, avatar_factory, status_bar, main_panel)

        self.real_id = real_id

        self.mentioned = False

        self.nicks = []

        self.original_text = post_data[TEXT_KEY]

        text = self.original_text

        media_dataset = []

        entities = post_data[ENTITIES_KEY]

        if entities is not None:
            original_replacements = []

            replacements = []

            dir = Pango.find_base_dir(text.encode(ENCODING), -1)

            rtl = (dir == Pango.DIRECTION_RTL)

            try:
                for media in entities[u'media']:
                    try:
                        large_url = self._append_link(text, replacements, media)

                        hash = media[u'id_str']

                        if not isinstance(hash, unicode):
                            raise TypeError

                        media_url = media[u'media_url']

                        if not isinstance(media_url, unicode):
                            raise TypeError

                        small_url = u'{}:thumb'.format(media_url)

                        media_dataset.append((u'twitter', hash, small_url, large_url))
                    except (TypeError, KeyError, ValueError):
                        pass
            except (TypeError, KeyError):
                pass

            try:
                for link in entities[LINKS_KEY]:
                    try:
                        large_url = self._append_link(text, replacements, link)

                        original_replacements.append((replacements[-1][0], large_url))

                        for media_spec in MEDIA_SPECS:
                            match = re.match(media_spec[1], large_url)

                            if match is not None:
                                name = media_spec[0]

                                if name == u'tinypic':
                                    hash = match.group(2)
                                else:
                                    hash = match.group(1)

                                if name == u'twitvid':
                                    replacement = u'{}/{}/{}/{}'.format(hash[0], hash[1], hash[2], hash)
                                else:
                                    replacement = hash

                                if name == u'tinypic':
                                    small_url = media_spec[2].format(match.group(1), replacement)
                                else:
                                    small_url = media_spec[2].format(replacement)

                                media_dataset.append((name, hash, small_url, large_url))

                                break
                    except (TypeError, KeyError, ValueError):
                        pass
            except (TypeError, KeyError):
                pass

            self._append_hashtags(text, replacements, rtl, entities)

            mentions = []

            try:
                for mention in entities[MENTIONS_KEY]:
                    try:
                        index, user_id, nick = self._append_mention(text, replacements, rtl, mention)

                        if user_id == self.stream.account.backend.user_id:
                            self.mentioned = True
                        else:
                            if user_id != self.user_backend.id:
                                mentions.append((index, nick))
                    except (TypeError, KeyError, ValueError):
                        pass
            except (TypeError, KeyError):
                pass

            if mentions:
                mentions.sort()

                for mention in mentions:
                    nick = mention[1]

                    if nick not in self.nicks:
                        self.nicks.append(nick)

            if replacements:
                self.original_text = self._join_replacements(self.original_text, original_replacements)

                text = self._join_replacements(text, replacements)

        self.timestamp = post_data[CREATION_TIMESTAMP_KEY]

        previews = []

        if self.stream.manager.get_preview():
            for media_data in media_dataset:
                previews.append(preview_factory.build(media_data[0], media_data[1], media_data[2], media_data[3]))

        Gdk.threads_enter()

        self.wraplabel.set_markup(escape(text.strip()))
        self.wraplabel.connect(u'activate-link', self._on_label_activate_link)

        self.preview_box = Gtk.VBox(False, 0)
        self.preview_box.show()

        for preview in previews:
            self.spinners.append(preview.spinner)

            self.preview_box.pack_start(preview, True, True, 0)

        self.inner_box.pack_start(self.preview_box, True, True, 0)

        self.updating = True

        Gdk.threads_leave()


    def isolate(self):
        Gdk.threads_enter()

        self.updating = False

        previews = self.preview_box.get_children()

        Gdk.threads_leave()

        for preview in previews:
            preview.isolate()

        super(Post, self).isolate()


    def get_nicks(self):
        nicks = list(self.nicks)

        nicks.insert(0, self.user_nick)

        return nicks


    def revert_two_scrollings(self):
        self.allocation_passed = False

        self.allocation_height = 0

        self.allocation_uid = self.connect(u'size-allocate', self._on_size_allocate_two)


    def revert_top_scrollings(self):
        self.allocation_height = 0

        self.connect(u'size-allocate', self._on_size_allocate_top)


    def synchronize(self, timestamp):
        Gdk.threads_enter()

        updating = self.updating

        Gdk.threads_leave()

        if updating:
            small_time = self._get_small_time(timestamp)

            large_time = self._get_large_time()

            Gdk.threads_enter()

            self.small_time = small_time

            self.large_time = large_time

            if self.info_button.toggled:
                self.footer_link.set_markup(self.large_time)
            else:
                self.footer_link.set_markup(self.small_time)

            Gdk.threads_leave()


    def log(self):
        return 0


    def notify(self, policy):
        pass



import tweet, message, result
