# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import anydbm, os, shutil


from polly import ENCODING



class Shelf(object):
    def _set(self, user_id, value):
        if self.connection is not None:
            key = bytes(user_id)

            try:
                self.connection[key] = value
            except anydbm.error:
                pass


    def __init__(self, id, offline, config_path, config_path_lock):
        self.path = os.path.join(config_path, bytes(id).decode(ENCODING))

        if offline:
            try:
                self.connection = anydbm.open(self.path, u'w')
            except anydbm.error:
                self.connection = None
        else:
            self.connection = None

        if self.connection is None:
            config_path_lock.acquire()

            try:
                if not os.path.isdir(config_path):
                    os.makedirs(config_path)
            except OSError:
                pass
            finally:
                config_path_lock.release()

            if os.path.exists(self.path):
                try:
                    if os.path.isfile(self.path):
                        os.remove(self.path)
                    elif os.path.isdir(self.path):
                        shutil.rmtree(self.path)
                except OSError:
                    pass

            try:
                self.connection = anydbm.open(self.path, u'n')
            except anydbm.error:
                self.connection = None


    def isolate(self, completely):
        if self.connection is not None:
            try:
                self.connection.close()
            except anydbm.error:
                pass

        if completely:
            try:
                os.remove(self.path)
            except OSError:
                pass


    def clear(self):
        if self.connection is not None:
            try:
                self.connection.close()

                self.connection = anydbm.open(self.path, u'n')
            except anydbm.error:
                pass


    def get(self, user_id):
        if self.connection is None:
            return 0
        else:
            key = bytes(user_id)

            try:
                if key in self.connection:
                    value = self.connection[key]

                    if value == b'2':
                        return 2
                    else:
                        return 0
                else:
                    return 1
            except anydbm.error:
                return 0


    def set_blocking(self, user_id):
        self._set(user_id, b'0')


    def set_friend(self, user_id):
        self._set(user_id, b'2')


    def unset(self, user_id):
        if self.connection is not None:
            key = bytes(user_id)

            try:
                if key in self.connection:
                    del self.connection[key]
            except anydbm.error:
                pass
