# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import gconf as GConf



GCONF_PATH = u'/apps/polly'

HIDDEN_DIR = u'hidden'



class Setting(object):
    def _update_value(self):
        self.value = self.gconf_client.get(self.path)


    def _on_value_changed(self, client, cnxn_id, entry, user_data):
        self._update_value()

        self.callback()


    def __init__(self, path, callback, gconf_client):
        self.path = path

        self.callback = callback

        self.gconf_client = gconf_client

        self._update_value()

        self.uid = self.gconf_client.notify_add(self.path, self._on_value_changed)


    def isolate(self):
        self.gconf_client.notify_remove(self.uid)

        self.callback = None



class BooleanSetting(Setting):
    def get_value(self):
        return self.value.get_bool()


    def set_value(self, value):
        self.gconf_client.set_bool(self.path, value)



class IntegerSetting(Setting):
    def get_value(self):
        return self.value.get_int()


    def set_value(self, value):
        self.gconf_client.set_int(self.path, value)



class StringSetting(Setting):
    def get_value(self):
        return self.value.get_string()


    def set_value(self, value):
        self.gconf_client.set_string(self.path, value)



class Factory(object):
    def _build_path(self, dir, key):
        return u'{}/{}/{}'.format(GCONF_PATH, dir, key)


    def __init__(self):
        self.gconf_client = GConf.client_get_default()

        self.gconf_client.add_dir(GCONF_PATH, GConf.CLIENT_PRELOAD_RECURSIVE)


    def isolate(self):
        self.gconf_client.remove_dir(GCONF_PATH)


    def get_fast(self, key):
        path = self._build_path(HIDDEN_DIR, key)

        return self.gconf_client.get_int(path)


    def get_flag(self, key):
        path = self._build_path(HIDDEN_DIR, key)

        return self.gconf_client.get_bool(path)


    def set_flag(self, key):
        path = self._build_path(HIDDEN_DIR, key)

        self.gconf_client.set_bool(path, False)


    def build_boolean(self, dir, key, callback):
        path = self._build_path(dir, key)

        return BooleanSetting(path, callback, self.gconf_client)


    def build_integer(self, dir, key, callback):
        path = self._build_path(dir, key)

        return IntegerSetting(path, callback, self.gconf_client)


    def build_string(self, dir, key, callback):
        path = self._build_path(dir, key)

        return StringSetting(path, callback, self.gconf_client)
