# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



import pycurl


from polly import Notifier, Updatable, BaseCache



ID_KEY = u'id'

REQUEST_TIMESTAMP_KEY = u'request_timestamp'

CREATION_TIMESTAMP_KEY = u'created_at'

ENTITIES_KEY = u'entities'


LINKS_KEY = u'urls'

MENTIONS_KEY = u'user_mentions'


DISPLAY_URL_KEY = u'display_url'

EXPANDED_URL_KEY = u'expanded_url'



PROXIES = [pycurl.PROXYTYPE_SOCKS4,
           pycurl.PROXYTYPE_SOCKS5,
           pycurl.PROXYTYPE_HTTP]



class ItemBackend(Updatable):
    def __init__(self, id, data, backend_cache):
        super(ItemBackend, self).__init__()

        self.id = id

        self.data = data

        self.backend_cache = backend_cache

        self.reference_count = 0

        self.notifiers = {}

        self.notifier = Notifier()


    def release(self):
        self.backend_cache.release(self)


    def get_value(self, key):
        self._acquire_update()

        value = self.data[key]

        self._release_update()

        return value


    def set_value(self, key, value):
        self._acquire_update()

        if self.data[key] != value:
            self.data[key] = value

            self.notifiers[key].notify()

            self.notifier.notify()

        self._release_update()


    def set_data(self, data):
        self._acquire_update()

        if self.data[REQUEST_TIMESTAMP_KEY] < data[REQUEST_TIMESTAMP_KEY]:
            self.previous_data = self.data

            self.data = data

            self._notify()

            self.notifier.notify()

        self._release_update()



class BaseBackendCache(BaseCache):
    def __init__(self, Class):
        super(BaseBackendCache, self).__init__()

        self.Class = Class


    def set_value(self, id, key, value):
        self.lock.acquire()

        if id in self.cache.viewkeys():
            self.cache[id].set_value(key, value)

        self.lock.release()


    def set_data(self, data):
        id = data[ID_KEY]

        self.lock.acquire()

        if id in self.cache.viewkeys():
            self.cache[id].set_data(data)

        self.lock.release()


    def acquire(self, data):
        id = data[ID_KEY]

        self.lock.acquire()

        if id in self.cache.viewkeys():
            self.cache[id].set_data(data)
        else:
            self.cache[id] = self.Class(id, data, self)

        backend = self.cache[id]

        backend.reference_count += 1

        self.lock.release()

        return backend


    def release(self, backend):
        self.lock.acquire()

        backend.reference_count -= 1

        if backend.reference_count == 0:
            del self.cache[backend.id]

        self.lock.release()



import lead, post, topic, account, realtime, media
