# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from polly import Notifier

from polly.twitter import ID_KEY, REQUEST_TIMESTAMP_KEY, ItemBackend, BaseBackendCache

from polly.twitter.lead import DESCRIPTION_KEY



PATH_KEY = u'full_name'

NICK_KEY = u'slug'

NAME_KEY = u'name'

MODE_KEY = u'mode'

MEMBER_COUNT_KEY = u'member_count'

SUBSCRIBER_COUNT_KEY = u'subscriber_count'

SUBSCRIBED_KEY = u'following'



# NOTE: The presence of long will not be necessary
# after the planned port to Python 3 is completed.

CRITERIA = {ID_KEY: (int, long),
            REQUEST_TIMESTAMP_KEY: float,
            DESCRIPTION_KEY: (type(None), bytes, unicode),
            PATH_KEY: unicode,
            NICK_KEY: unicode,
            NAME_KEY: (type(None), bytes, unicode),
            MODE_KEY: unicode,
            MEMBER_COUNT_KEY: (int, long),
            SUBSCRIBER_COUNT_KEY: (int, long),
            SUBSCRIBED_KEY: (type(None), bool)}



class Backend(ItemBackend):
    def _notify(self):
        if self.previous_data[DESCRIPTION_KEY] != self.data[DESCRIPTION_KEY]:
            self.description_notifier.notify()

        if self.previous_data[PATH_KEY] != self.data[PATH_KEY]:
            self.path_notifier.notify()

        if self.previous_data[NICK_KEY] != self.data[NICK_KEY]:
            self.nick_notifier.notify()

        if self.previous_data[NAME_KEY] != self.data[NAME_KEY]:
            self.name_notifier.notify()

        if self.previous_data[MODE_KEY] != self.data[MODE_KEY]:
            self.mode_notifier.notify()

        if self.previous_data[MEMBER_COUNT_KEY] != self.data[MEMBER_COUNT_KEY]:
            self.member_count_notifier.notify()

        if self.previous_data[SUBSCRIBER_COUNT_KEY] != self.data[SUBSCRIBER_COUNT_KEY]:
            self.subscriber_count_notifier.notify()

        self.data[SUBSCRIBED_KEY] = self.previous_data[SUBSCRIBED_KEY]


    def __init__(self, id, data, group_backend_cache):
        super(Backend, self).__init__(id, data, group_backend_cache)

        self.description_notifier = Notifier()

        self.path_notifier = Notifier()

        self.nick_notifier = Notifier()

        self.name_notifier = Notifier()

        self.mode_notifier = Notifier()

        self.member_count_notifier = Notifier()

        self.subscriber_count_notifier = Notifier()

        self.notifiers[SUBSCRIBED_KEY] = Notifier()



class BackendCache(BaseBackendCache):
    def __init__(self):
        super(BackendCache, self).__init__(Backend)
