# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from polly import Notifier

from polly.twitter import ID_KEY, REQUEST_TIMESTAMP_KEY, ENTITIES_KEY, DISPLAY_URL_KEY, EXPANDED_URL_KEY, ItemBackend, BaseBackendCache

from polly.twitter.lead import DESCRIPTION_KEY



OWN_KEY = u'own'

IMAGE_URL_KEY = u'profile_image_url'

NICK_KEY = u'screen_name'

NAME_KEY = u'name'

VERIFIED_KEY = u'verified'

PROTECTED_KEY = u'protected'

LOCATION_KEY = u'location'

URL_KEY = u'url'

FOLLOWER_COUNT_KEY = u'followers_count'

FRIEND_COUNT_KEY = u'friends_count'

TWEET_COUNT_KEY = u'statuses_count'

FAVORITE_COUNT_KEY = u'favourites_count'

MEMBERSHIP_COUNT_KEY = u'listed_count'

PENDING_KEY = u'follow_request_sent'


# NOTE: The presence of long will not be necessary
# after the planned port to Python 3 is completed.

CRITERIA = {ID_KEY: (int, long),
            REQUEST_TIMESTAMP_KEY: float,
            ENTITIES_KEY: (type(None), dict),
            DISPLAY_URL_KEY: (type(None), bytes, unicode),
            EXPANDED_URL_KEY: (type(None), bytes, unicode),
            DESCRIPTION_KEY: (type(None), bytes, unicode),
            OWN_KEY: bool,
            IMAGE_URL_KEY: unicode,
            NICK_KEY: unicode,
            NAME_KEY: (type(None), bytes, unicode),
            VERIFIED_KEY: (type(None), bool),
            PROTECTED_KEY: (type(None), bool),
            LOCATION_KEY: (type(None), bytes, unicode),
            URL_KEY: (type(None), bytes, unicode),
            FOLLOWER_COUNT_KEY: (int, long),
            FRIEND_COUNT_KEY: (int, long),
            TWEET_COUNT_KEY: (int, long),
            FAVORITE_COUNT_KEY: (int, long),
            MEMBERSHIP_COUNT_KEY: (int, long),
            PENDING_KEY: (type(None), bool)}



class Backend(ItemBackend):
    def _notify(self):
        if (self.previous_data[DESCRIPTION_KEY] != self.data[DESCRIPTION_KEY] or
           (self.previous_data[ENTITIES_KEY] is None and self.data[ENTITIES_KEY] is not None)):
            self.description_notifier.notify()

        if self.previous_data[IMAGE_URL_KEY] != self.data[IMAGE_URL_KEY]:
            self.image_url_notifier.notify()

        if self.previous_data[NICK_KEY] != self.data[NICK_KEY]:
            self.nick_notifier.notify()

        if self.previous_data[NAME_KEY] != self.data[NAME_KEY]:
            self.name_notifier.notify()

        if self.previous_data[VERIFIED_KEY] != self.data[VERIFIED_KEY]:
            self.verified_notifier.notify()

        if self.previous_data[PROTECTED_KEY] != self.data[PROTECTED_KEY]:
            self.protected_notifier.notify()

        if self.previous_data[LOCATION_KEY] != self.data[LOCATION_KEY]:
            self.location_notifier.notify()

        if (self.previous_data[URL_KEY] != self.data[URL_KEY] or
            self.previous_data[DISPLAY_URL_KEY] != self.data[DISPLAY_URL_KEY] or
            self.previous_data[EXPANDED_URL_KEY] != self.data[EXPANDED_URL_KEY]):
            self.url_notifier.notify()

        if self.previous_data[FOLLOWER_COUNT_KEY] != self.data[FOLLOWER_COUNT_KEY]:
            self.follower_count_notifier.notify()

        if self.previous_data[FRIEND_COUNT_KEY] != self.data[FRIEND_COUNT_KEY]:
            self.friend_count_notifier.notify()

        if self.previous_data[TWEET_COUNT_KEY] != self.data[TWEET_COUNT_KEY]:
            self.tweet_count_notifier.notify()

        if self.previous_data[FAVORITE_COUNT_KEY] != self.data[FAVORITE_COUNT_KEY]:
            self.favorite_count_notifier.notify()

        if self.previous_data[MEMBERSHIP_COUNT_KEY] != self.data[MEMBERSHIP_COUNT_KEY]:
            self.membership_count_notifier.notify()


    def __init__(self, id, data, user_backend_cache):
        super(Backend, self).__init__(id, data, user_backend_cache)

        self.image_url_notifier = Notifier()

        self.nick_notifier = Notifier()

        self.name_notifier = Notifier()

        self.verified_notifier = Notifier()

        self.protected_notifier = Notifier()

        self.description_notifier = Notifier()

        self.location_notifier = Notifier()

        self.url_notifier = Notifier()

        self.follower_count_notifier = Notifier()

        self.friend_count_notifier = Notifier()

        self.tweet_count_notifier = Notifier()

        self.favorite_count_notifier = Notifier()

        self.membership_count_notifier = Notifier()



class BackendCache(BaseBackendCache):
    def __init__(self):
        super(BackendCache, self).__init__(Backend)
