# #########################################################################
# Copyright (C) 2011 Marcelo Hashimoto
#
# This file is part of Polly.
#
# Polly is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polly is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polly. If not, see <http://www.gnu.org/licenses/>.
# #########################################################################



from polly import Notifier

from polly.twitter import ID_KEY, REQUEST_TIMESTAMP_KEY, CREATION_TIMESTAMP_KEY, ENTITIES_KEY, ItemBackend, BaseBackendCache

from polly.twitter.post import TEXT_KEY



REPLIED_TWEET_ID_KEY = u'in_reply_to_status_id'

COORDINATES_KEY = u'coordinates'

SOURCE_KEY = u'source'

PLACE_KEY = u'place'

RETWEETER_COUNT_KEY = u'retweet_count'

RETWEET_ID_KEY = u'current_user_retweet'

FAVORITED_KEY = u'favorited'


# NOTE: The presence of long will not be necessary
# after the planned port to Python 3 is completed.

CRITERIA = {ID_KEY: (int, long),
            REQUEST_TIMESTAMP_KEY: float,
            CREATION_TIMESTAMP_KEY: float,
            ENTITIES_KEY: dict,
            TEXT_KEY: unicode,
            REPLIED_TWEET_ID_KEY: (type(None), int, long),
            COORDINATES_KEY: (type(None), dict),
            SOURCE_KEY: unicode,
            PLACE_KEY: (type(None), dict),
            RETWEETER_COUNT_KEY: (int, long, unicode),
            RETWEET_ID_KEY: (type(None), int, long),
            FAVORITED_KEY: (type(None), bool)}



class Backend(ItemBackend):
    def _notify(self):
        if self.previous_data[RETWEETER_COUNT_KEY] != self.data[RETWEETER_COUNT_KEY]:
            self.retweeter_count_notifier.notify()

        self.data[RETWEET_ID_KEY] = self.previous_data[RETWEET_ID_KEY]

        self.data[FAVORITED_KEY] = self.previous_data[FAVORITED_KEY]


    def __init__(self, id, data, tweet_backend_cache):
        super(Backend, self).__init__(id, data, tweet_backend_cache)

        self.retweeter_count_notifier = Notifier()

        self.notifiers[RETWEET_ID_KEY] = Notifier()

        self.notifiers[FAVORITED_KEY] = Notifier()


    def unset_retweet_id(self, deleted_id):
        self._acquire_update()

        if self.data[RETWEET_ID_KEY] == deleted_id:
            self.data[RETWEET_ID_KEY] = None

            self.notifiers[RETWEET_ID_KEY].notify()

            self.notifier.notify()

            success = True
        else:
            success = False

        self._release_update()

        return success



class BackendCache(BaseBackendCache):
    def __init__(self):
        super(BackendCache, self).__init__(Backend)


    def unset_retweet_id(self, deleted_id):
        self.lock.acquire()

        for backend in self.cache.viewvalues():
            if backend.unset_retweet_id(deleted_id):
                break

        self.lock.release()
